@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -33,8 +33,11 @@
 
 #include "core/siril.h"
 #include "core/proto.h"
+#include "core/siril_world_cs.h"
 #include "algos/photometry.h"
 #include "algos/sorting.h"
+#include "algos/siril_wcs.h"
+#include "algos/star_finder.h"
 #include "filters/median.h"
 
 #include "PSF.h"
@@ -122,17 +125,15 @@ static gsl_vector* psf_init_data(gsl_matrix* z, double bg) {
  * failed and for star detection when magnitude is not needed.
  */
 static double psf_get_mag(gsl_matrix* z, double B) {
-	double intensity = 0.0, magnitude;
+	double intensity = 1.0;
 	size_t NbRows = z->size1;
 	size_t NbCols = z->size2;
-	size_t i, j;
 
-	for (i = 0; i < NbRows; i++) {
-		for (j = 0; j < NbCols; j++)
+	for (size_t i = 0; i < NbRows; i++) {
+		for (size_t j = 0; j < NbCols; j++)
 			intensity += gsl_matrix_get(z, i, j) - B;
 	}
-	magnitude = -2.5 * log10(intensity);
-	return magnitude;
+	return -2.5 * log10(intensity);
 }
 
 /* No angle */
@@ -290,8 +291,7 @@ static int psf_Gaussian_fdf_an(const gsl_vector * x, void *PSF_data,
 /* The function returns the fitted parameters without angle. However it
  * returns NULL if the number of parameters is => to the pixel number.
  */
-static fitted_PSF *psf_minimiz_no_angle(gsl_matrix* z, double background,
-		int layer) {
+static psf_star *psf_minimiz_no_angle(gsl_matrix* z, double background) {
 	size_t i, j;
 	size_t NbRows = z->size1; //characteristics of the selection : height and width
 	size_t NbCols = z->size2;
@@ -305,7 +305,7 @@ static fitted_PSF *psf_minimiz_no_angle(gsl_matrix* z, double background,
 	gsl_matrix *covar = gsl_matrix_alloc(p, p);
 	double *y = malloc(n * sizeof(double));
 	double *sigma = malloc(n * sizeof(double));
-	fitted_PSF *psf = malloc(sizeof(fitted_PSF));
+	psf_star *psf = new_psf_star();
 	if (!y || !sigma || !psf) {
 		PRINT_ALLOC_ERR;
 		return NULL;
@@ -331,7 +331,7 @@ static fitted_PSF *psf_minimiz_no_angle(gsl_matrix* z, double background,
 	f.fdf = &psf_Gaussian_fdf;
 	f.n = n;
 	if (n <= p) {
-		free(psf);
+		free_psf(psf);
 		free(y);
 		free(sigma);
 		return NULL;
@@ -381,14 +381,15 @@ static fitted_PSF *psf_minimiz_no_angle(gsl_matrix* z, double background,
 	psf->sy = FIT(5);
 	psf->fwhmx = sqrt(FIT(4) / 2.) * 2 * sqrt(log(2.) * 2);	//Set the real FWHMx with regards to the Sx parameter
 	psf->fwhmy = sqrt(FIT(5) / 2.) * 2 * sqrt(log(2.) * 2);	//Set the real FWHMy with regards to the Sy parameter
+	psf->fwhmx_arcsec = -1.0;
+	psf->fwhmy_arcsec = -1.0;
 	psf->angle = 0;	//The angle is not fitted here
 	// Units
 	psf->units = "px";
 	// Magnitude
 	psf->mag = psf_get_mag(z, psf->B);
+	psf->phot = NULL;
 	psf->phot_is_valid = FALSE;
-	// Layer: not fitted
-	psf->layer = layer;
 	// RMSE
 	psf->rmse = d.rmse;
 	// absolute uncertainties
@@ -415,7 +416,7 @@ static fitted_PSF *psf_minimiz_no_angle(gsl_matrix* z, double background,
  * NULL if the number of parameters is => to the pixel number.
  * This should not happen because this case is already treated by the
  * minimiz_no_angle function */
-static fitted_PSF *psf_minimiz_angle(gsl_matrix* z, fitted_PSF *psf, gboolean for_photometry, gboolean verbose) {
+static psf_star *psf_minimiz_angle(gsl_matrix* z, psf_star *psf, gboolean for_photometry, gboolean force_radius, gboolean verbose) {
 	size_t i, j;
 	size_t NbRows = z->size1; //characteristics of the selection : height and width
 	size_t NbCols = z->size2;
@@ -424,7 +425,7 @@ static fitted_PSF *psf_minimiz_angle(gsl_matrix* z, fitted_PSF *psf, gboolean fo
 	g_assert (n > 0);
 	int status;
 	unsigned int iter = 0;
-	fitted_PSF *psf_angle = malloc(sizeof(fitted_PSF));
+	psf_star *psf_angle = new_psf_star();
 	gsl_matrix *covar = gsl_matrix_alloc(p, p);
 	double *y = malloc(n * sizeof(double));
 	double *sigma = malloc(n * sizeof(double));
@@ -513,7 +514,7 @@ static fitted_PSF *psf_minimiz_angle(gsl_matrix* z, fitted_PSF *psf, gboolean fo
 	psf_angle->units = "px";
 	// Photometry
 	if (for_photometry)
-		psf_angle->phot = getPhotometryData(z, psf_angle, verbose);
+		psf_angle->phot = getPhotometryData(z, psf_angle, force_radius, verbose);
 	else {
 		psf_angle->phot = NULL;
 		psf_angle->phot_is_valid = FALSE;
@@ -522,15 +523,15 @@ static fitted_PSF *psf_minimiz_angle(gsl_matrix* z, fitted_PSF *psf, gboolean fo
 	if (psf_angle->phot != NULL) {
 		psf_angle->mag = psf_angle->phot->mag;
 		psf_angle->s_mag = psf_angle->phot->s_mag;
+		psf_angle->SNR = psf_angle->phot->SNR;
 		psf_angle->phot_is_valid = psf_angle->phot->valid;
 
 	} else {
 		psf_angle->mag = psf_get_mag(z, psf_angle->B);
 		psf_angle->s_mag = 9.999;
+		psf_angle->SNR = 0;
 		psf_angle->phot_is_valid = FALSE;
 	}
-	//Layer: not fitted
-	psf_angle->layer = psf->layer;
 	//RMSE
 	psf_angle->rmse = d.rmse;
 	// absolute uncertainties
@@ -548,16 +549,15 @@ static fitted_PSF *psf_minimiz_angle(gsl_matrix* z, fitted_PSF *psf, gboolean fo
 	gsl_multifit_fdfsolver_free(s);
 	gsl_matrix_free(covar);
 	gsl_rng_free(r);
-	free(psf_angle->phot);
 	return psf_angle;
 }
 
 /******************************************************************************/
 
-/* Returns the largest FWHM.
+/* Returns the largest FWHM in pixels
  * The optional output parameter roundness is the ratio between the two axis FWHM */
-double psf_get_fwhm(fits *fit, int layer, double *roundness) {
-	fitted_PSF *result = psf_get_minimisation(fit, layer, &com.selection, FALSE, TRUE, TRUE);
+double psf_get_fwhm(fits *fit, int layer, rectangle *selection, double *roundness) {
+	psf_star *result = psf_get_minimisation(fit, layer, selection, FALSE, FALSE, TRUE, TRUE);
 	if (result == NULL) {
 		*roundness = 0.0;
 		return 0.0;
@@ -566,18 +566,18 @@ double psf_get_fwhm(fits *fit, int layer, double *roundness) {
 	retval = result->fwhmx;
 	if (roundness)
 		*roundness = result->fwhmy / result->fwhmx;
-	free(result);
+	free_psf(result);
 	return retval;
 }
 
 /* Computes the FWHM on data in the selection rectangle of image fit.
  * Selection rectangle is passed as third argument.
- * Return value is a structure, type fitted_PSF, that has to be freed after use.
+ * Return value is a structure, type psf_star, that has to be freed after use.
  */
-fitted_PSF *psf_get_minimisation(fits *fit, int layer, rectangle *area,
-		gboolean for_photometry, gboolean verbose, gboolean multithread_stat) {
+psf_star *psf_get_minimisation(fits *fit, int layer, rectangle *area,
+		gboolean for_photometry, gboolean force_radius, gboolean verbose, gboolean multithread_stat) {
 	int stridefrom, i, j;
-	fitted_PSF *result;
+	psf_star *result;
 	double bg = background(fit, layer, area, multithread_stat);
 
 	// fprintf(stdout, "background: %g\n", bg);
@@ -614,9 +614,11 @@ fitted_PSF *psf_get_minimisation(fits *fit, int layer, rectangle *area,
 		return NULL;
 	}
 
-	result = psf_global_minimisation(z, bg, layer, TRUE, for_photometry, verbose);
-	if (result)
+	result = psf_global_minimisation(z, bg, TRUE, for_photometry, force_radius, verbose);
+	if (result) {
 		fwhm_to_arcsec_if_needed(fit, result);
+		result->layer = layer;
+	}
 	gsl_matrix_free(z);
 	return result;
 }
@@ -631,12 +633,12 @@ fitted_PSF *psf_get_minimisation(fits *fit, int layer, rectangle *area,
  * of the star_finder algorithm), no angle parameter is fitted.
  * The function returns NULL if values look bizarre.
  */
-fitted_PSF *psf_global_minimisation(gsl_matrix* z, double bg, int layer,
-		gboolean fit_angle, gboolean for_photometry, gboolean verbose) {
-	fitted_PSF *psf;
+psf_star *psf_global_minimisation(gsl_matrix* z, double bg, 
+		gboolean fit_angle, gboolean for_photometry, gboolean force_radius, gboolean verbose) {
+	psf_star *psf;
 
 	// To compute good starting values, we first compute with no angle
-	if ((psf = psf_minimiz_no_angle(z, bg, layer)) != NULL) {
+	if ((psf = psf_minimiz_no_angle(z, bg)) != NULL) {
 		if (fit_angle) {
 			/* This next check is to avoid possible angle divergence
 			 * when sx and sy are too close (star is quite round).
@@ -645,10 +647,11 @@ fitted_PSF *psf_global_minimisation(gsl_matrix* z, double bg, int layer,
 			if ((fabs(psf->sx - psf->sy) < EPSILON)) {
 				// Photometry
 				if (for_photometry) {
-					psf->phot = getPhotometryData(z, psf, verbose);
+					psf->phot = getPhotometryData(z, psf, force_radius, verbose);
 					if (psf->phot != NULL) {
 						psf->mag = psf->phot->mag;
 						psf->s_mag = psf->phot->s_mag;
+						psf->SNR = psf->phot->SNR;
 						psf->phot_is_valid = psf->phot->valid;
 					}
 				} else {
@@ -656,13 +659,13 @@ fitted_PSF *psf_global_minimisation(gsl_matrix* z, double bg, int layer,
 					psf->phot_is_valid = FALSE;
 				}
 			} else {
-				fitted_PSF *tmp_psf;
-				if ((tmp_psf = psf_minimiz_angle(z, psf, for_photometry, verbose))
+				psf_star *tmp_psf;
+				if ((tmp_psf = psf_minimiz_angle(z, psf, for_photometry, force_radius, verbose))
 						== NULL) {
-					free(psf);
+					free_psf(psf);
 					return NULL;
 				}
-				free(psf);
+				free_psf(psf);
 				psf = tmp_psf;
 			}
 		}
@@ -680,7 +683,7 @@ fitted_PSF *psf_global_minimisation(gsl_matrix* z, double bg, int layer,
 		/* We quickly test the result. If it is bad we return NULL */
 		if (!isfinite(psf->fwhmx) || !isfinite(psf->fwhmy) ||
 				psf->fwhmx <= 0.0 || psf->fwhmy <= 0.0) {
-			free(psf);
+			free_psf(psf);
 			psf = NULL;
 		}
 	}
@@ -689,40 +692,76 @@ fitted_PSF *psf_global_minimisation(gsl_matrix* z, double bg, int layer,
 	return psf;
 }
 
-void psf_display_result(fitted_PSF *result, rectangle *area) {
-	char buffer[256];
+void psf_display_result(psf_star *result, rectangle *area) {
+	char *buffer, *coordinates;
 	char *str;
 	if (com.magOffset > 0.0)
-		str = "true reduced";
+		str = _("true reduced");
 	else
-		str = "relative";
+		str = _("relative");
 
-	g_snprintf(buffer, sizeof(buffer), _("PSF fit Result:\n"
-			"x0=%0.2f px, y0=%0.2f px\n"
+	double x = result->x0 + area->x;
+	double y = area->y + area->h - result->y0;
+
+	if (has_wcs(&gfit)) {
+		double world_x, world_y;
+		SirilWorldCS *world_cs;
+		pix2wcs(&gfit, x, (double) gfit.ry - y, &world_x, &world_y);
+		world_cs = siril_world_cs_new_from_a_d(world_x, world_y);
+		if (world_cs) {
+			gchar *ra = siril_world_cs_alpha_format(world_cs, "%02dh%02dm%02ds");
+			gchar *dec = siril_world_cs_delta_format(world_cs, "%c%02d°%02d\'%02d\"");
+			coordinates = g_strdup_printf("x0=%0.2f px, y0=%0.2f px (%s , %s)", x, y, ra, dec);
+
+			siril_world_cs_unref(world_cs);
+			g_free(ra);
+			g_free(dec);
+		} else {
+			coordinates = g_strdup_printf("x0=%0.2f px, y0=%0.2f px", x, y);
+		}
+	} else {
+		coordinates = g_strdup_printf("x0=%0.2f px, y0=%0.2f px", x, y);
+	}
+
+	buffer = g_strdup_printf(_("PSF fit Result:\n"
+			"%s\n"
 			"FWHM X=%0.2f%s, FWHM Y=%0.2f%s\n"
 			"Angle=%0.2f deg\n"
 			"Background value=%0.6f\n"
 			"Maximal intensity=%0.6f\n"
 			"Magnitude (%s)=%0.2f\n"
-			"RMSE=%.3e\n"), result->x0 + area->x, area->y + area->h - result->y0,
+			"SNR=%.1fdB\n"
+			"RMSE=%.3e\n"),
+			coordinates,
 			result->fwhmx, result->units, result->fwhmy, result->units,
-			result->angle, result->B, result->A, str, result->mag + com.magOffset, result->rmse);
+			result->angle,
+			result->B,
+			result->A,
+			str,
+			result->mag + com.magOffset,
+			result->SNR,
+			result->rmse);
 
 	siril_log_message(buffer);
+	g_free(buffer);
+	g_free(coordinates);
 }
 
-#define _2_SQRT_2_LOG2 1.55185049709
+#define _2_SQRT_2_LOG2 2.35482004503
 
 /* If the pixel pitch and the focal length are known and filled in the 
  * setting box, we convert FWHM in pixel to arcsec by multiplying
  * the FWHM value with the sampling value */
-void fwhm_to_arcsec_if_needed(fits* fit, fitted_PSF *result) {
+void fwhm_to_arcsec_if_needed(fits* fit, psf_star *result) {
 
 	if (!result) return;
 	if (fit->focal_length <= 0.0 || fit->pixel_size_x <= 0.f
 			|| fit->pixel_size_y <= 0.f || fit->binning_x <= 0
-			|| fit->binning_y <= 0)
+			|| fit->binning_y <= 0) {
+		result->fwhmx_arcsec = -1.0;
+		result->fwhmy_arcsec = -1.0;
 		return;
+	}
 
 	double bin_X, bin_Y;
 	double fwhmx, fwhmy;
@@ -733,14 +772,72 @@ void fwhm_to_arcsec_if_needed(fits* fit, fitted_PSF *result) {
 	bin_X = fit->unbinned ? (double) fit->binning_x : 1.0;
 	bin_Y = fit->unbinned ? (double) fit->binning_y : 1.0;
 
-	result->fwhmx = fwhmx * (radian_conversion * (double)fit->pixel_size_x / fit->focal_length) * bin_X;
-	result->fwhmy = fwhmy * (radian_conversion * (double)fit->pixel_size_y / fit->focal_length) * bin_Y;
+	result->fwhmx_arcsec = fwhmx * (radian_conversion * (double)fit->pixel_size_x / fit->focal_length) * bin_X;
+	result->fwhmy_arcsec = fwhmy * (radian_conversion * (double)fit->pixel_size_y / fit->focal_length) * bin_Y;
 	result->units = "\"";
 }
 
-void fwhm_to_pixels(fitted_PSF *result) {
+void fwhm_to_pixels(psf_star *result) {
 	result->fwhmx = sqrt(result->sx * 0.5) * _2_SQRT_2_LOG2;
 	result->fwhmy = sqrt(result->sy * 0.5) * _2_SQRT_2_LOG2;
 	result->units = "px";
 }
 
+// returns boolean if it was possible (true if arcsec)
+gboolean get_fwhm_as_arcsec_if_possible(psf_star *star, double *fwhmx, double *fwhmy, char **unit) {
+	if (!strcmp(star->units, "px")) {
+		*fwhmx = star->fwhmx;
+		*fwhmy = star->fwhmy;
+		*unit = star->units;
+		return FALSE;
+	}
+	if (star->fwhmx_arcsec <= 0.0) {
+		fprintf(stderr, "FWHM wrongly stored as arcsec\n");
+		star->units = "px";
+		return get_fwhm_as_arcsec_if_possible(star, fwhmx, fwhmy, unit);
+	}
+	*fwhmx = star->fwhmx_arcsec;
+	*fwhmy = star->fwhmy_arcsec;
+	*unit = star->units;
+	return TRUE;
+}
+
+double convert_single_fwhm_to_pixels(double fwhm, double s) {
+	return sqrt(s * 0.5) * _2_SQRT_2_LOG2;
+}
+
+gboolean convert_single_fwhm_to_arcsec_if_possible(double fwhm, double bin, double px_size, double flenght, double *result) {
+	double arcsec = fwhm * (radian_conversion * px_size / flenght) * bin;
+	if (arcsec <= 0.0 || isnan(arcsec) || !isfinite(arcsec)) {
+		*result = 0;
+		return FALSE;
+	}
+	*result = arcsec;
+	return TRUE;
+}
+
+psf_star *new_psf_star() {
+	psf_star *star = malloc(sizeof(psf_star));
+	star->phot = NULL;
+
+	return star;
+}
+
+psf_star *duplicate_psf(psf_star *psf) {
+	if (!psf)
+		return NULL;
+	psf_star *new_psf = new_psf_star();
+	memcpy(new_psf, psf, sizeof(psf_star));
+	if (psf->phot) {
+		new_psf->phot = malloc(sizeof(photometry));
+		memcpy(new_psf->phot, psf->phot, sizeof(photometry));
+	} else {
+		new_psf->phot = NULL;
+	}
+	return new_psf;
+}
+
+void free_psf(psf_star *psf) {
+	if (psf->phot) free(psf->phot);
+	free(psf);
+}
