@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -37,6 +37,7 @@
 #include "algos/statistics.h"
 #include "algos/geometry.h"
 #include "algos/sorting.h"
+#include "gui/utils.h"
 #include "gui/callbacks.h"
 #include "gui/image_display.h"
 #include "gui/image_interactions.h"
@@ -54,6 +55,15 @@
 
 #define SAMPLE_SIZE 25
 
+struct sample {
+	double median[3]; // median of each channel of the sample (if color)
+	double mean; // mean of the 3 channel of the sample (if color)
+	double min, max;
+	size_t size;
+	point position;
+	gboolean valid;
+};
+
 //C contains background function
 #define C(i) (gsl_vector_get(c,(i)))
 
@@ -91,7 +101,6 @@ static double poly_1(gsl_vector *c, double x, double y) {
 static gboolean computeBackground(GSList *list, double *background, int channel, unsigned int width, unsigned int height, poly_order order, gchar **err) {
 	size_t k = 0;
 	double chisq, pixel;
-	double row, col;
 	gsl_matrix *J, *cov;
 	gsl_vector *y, *w, *c;
 	GSList *l;
@@ -131,8 +140,8 @@ static gboolean computeBackground(GSList *list, double *background, int channel,
 	for (l = list; l; l = l->next) {
 		background_sample *sample = (background_sample *) l->data;
 
-		col = sample->position.x;
-		row = sample->position.y;
+		double col = sample->position.x;
+		double row = sample->position.y;
 		pixel = sample->median[channel];
 		// here, it is a bit sketchy in the sense that if there is not value to report in a box (because the threshold is too
 		// low for example, then I just skip the initialization of J and y. gsl automatically discard the non assigned values
@@ -290,21 +299,21 @@ static double get_sample_median(double *buf, const int xx,
 	return median;
 }
 
-static unsigned int _rand(uint64_t *const p_rng) {
+static unsigned int _rand(guint64 *const p_rng) {
 	*p_rng = *p_rng * 1103515245 + 12345U;
 	return (unsigned int) *p_rng;
 }
 
 static gboolean convert_fits_to_img(fits *fit, double *image, int channel, gboolean add_dither) {
 
-	uint64_t seed = time(NULL);
+	guint64 seed = time(NULL);
 
 	const int height = fit->ry;
 	const int width = fit->rx;
 	if (fit->type == DATA_USHORT) {
 		for (int y = 0; y < height; ++y) {
 			for (int x = 0; x < width; ++x) {
-				image[y * width + x] = fit->pdata[channel][(height - y - 1)	* width + x] / USHRT_MAX_SINGLE;
+				image[y * width + x] = fit->pdata[channel][(height - y - 1) * width + x] / USHRT_MAX_SINGLE;
 				if (add_dither) {
 					/* add dithering in order to avoid colour banding */
 					image[y * width + x] += (_rand(&seed) % 1048576) * 0.000000000095367431640625f;
@@ -555,24 +564,29 @@ GSList *add_background_sample(GSList *orig, fits *fit, point pt) {
 GSList *remove_background_sample(GSList *orig, fits *fit, point pt) {
 	GSList *list;
 	float *image;
+	double min_radius = DBL_MAX;
 
 	image = convert_fits_to_luminance(fit);
 
+	/* search for the min radius vale */
 	for (list = orig; list; list = list->next) {
-		background_sample *sample;
-		double radius;
-		double dx;
-		double dy;
+		background_sample *sample = (background_sample *)list->data;
+		double dx = pt.x - sample->position.x;
+		double dy = pt.y - sample->position.y;
+		double radius = sqrt(dx * dx + dy * dy);
 
-		sample = (background_sample *)list->data;
-		dx = pt.x - sample->position.x;
-		dy = pt.y - sample->position.y;
-		radius = sqrt(dx * dx + dy * dy);
+		min_radius = min(min_radius, radius);
+	}
+	/* remove this value */
+	for (list = orig; list; list = list->next) {
+		background_sample *sample = (background_sample *)list->data;
+		double dx = pt.x - sample->position.x;
+		double dy = pt.y - sample->position.y;
+		double radius = sqrt(dx * dx + dy * dy);
 
-		if (radius <= sample->size * 2) {
+		if (radius == min_radius) {
 			orig = g_slist_remove(orig, sample);
 			g_free((background_sample *) sample);
-			break;
 		}
 	}
 	free(image);
@@ -580,21 +594,19 @@ GSList *remove_background_sample(GSList *orig, fits *fit, point pt) {
 	return orig;
 }
 
+/* generates samples and stores them in com.grad_samples */
 void generate_background_samples(int nb_of_samples, double tolerance) {
-	set_cursor_waiting(TRUE);
-
 	free_background_sample_list(com.grad_samples);
 	com.grad_samples = generate_samples(&gfit, nb_of_samples, tolerance, SAMPLE_SIZE);
 	if (gfit.naxes[2] > 1) {
-		com.grad_samples = update_median_for_rgb_samples(com.grad_samples,
-				&gfit);
+		com.grad_samples = update_median_for_rgb_samples(com.grad_samples, &gfit);
 	}
 
-	redraw(com.cvport, REMAP_ALL);
-	set_cursor_waiting(FALSE);
+	redraw(REDRAW_OVERLAY);
 }
 
-gboolean remove_gradient_from_image(int correction, poly_order degree) {
+/* uses samples from com.grad_samples */
+gboolean remove_gradient_from_image(int correction, poly_order degree, gboolean use_dither) {
 	gchar *error;
 	double *background = malloc(gfit.ry * gfit.rx * sizeof(double));
 	if (!background && !com.script) {
@@ -613,7 +625,7 @@ gboolean remove_gradient_from_image(int correction, poly_order degree) {
 
 	for (int channel = 0; channel < gfit.naxes[2]; channel++) {
 		/* compute background */
-		convert_fits_to_img(&gfit, image, channel, is_dither_checked());
+		convert_fits_to_img(&gfit, image, channel, use_dither);
 		if (!computeBackground(com.grad_samples, background, channel, gfit.rx,
 				gfit.ry, degree, &error)) {
 			free(image);
@@ -673,7 +685,7 @@ static int background_image_hook(struct generic_seq_args *args, int o, int i, fi
 
 	for (int channel = 0; channel < fit->naxes[2]; channel++) {
 		/* compute background */
-		convert_fits_to_img(fit, image, channel, is_dither_checked());
+		convert_fits_to_img(fit, image, channel, b_args->dither);
 		if (!computeBackground(samples, background, channel, fit->rx, fit->ry, b_args->degree, &error)) {
 			if (error) {
 				siril_log_message(error);
@@ -681,6 +693,7 @@ static int background_image_hook(struct generic_seq_args *args, int o, int i, fi
 
 			set_cursor_waiting(FALSE);
 			free(image);
+			free(background);
 			free_background_sample_list(samples);
 			return 1;
 		}
@@ -697,46 +710,103 @@ static int background_image_hook(struct generic_seq_args *args, int o, int i, fi
 	return 0;
 }
 
+static int background_mem_limits_hook(struct generic_seq_args *args, gboolean for_writer) {
+	unsigned int MB_per_image, MB_avail;
+	int limit = compute_nb_images_fit_memory(args->seq, 1.0, FALSE, &MB_per_image, NULL, &MB_avail);
+	unsigned int required = MB_per_image;
+	if (limit > 0) {
+		/* allocations:
+		 * generate_samples convert_fits_to_luminance allocates         rx * ry * sizeof(float)
+		 * generate_samples allocates a buffer for MAD computation      rx * ry * sizeof(double)
+		 * both are freed at generate_samples exit
+		 * for color images:
+		 *	update_median_for_rgb_samples allocates for median      rx * ry * sizeof(double)
+		 * freed at update_median_for_rgb_samples exit
+		 * remove_gradient_from_image allocates the background image to rx * ry * sizeof(double)
+		 * remove_gradient_from_image allocates the image            to rx * ry * sizeof(double)
+		 *
+		 * so at maximum, ignoring the samples, we need 2 times the double channel size.
+		 */
+		uint64_t double_channel_size = args->seq->rx * args->seq->ry * sizeof(double);
+		unsigned int double_channel_size_MB = double_channel_size / BYTES_IN_A_MB;
+		required = MB_per_image + double_channel_size_MB * 2;
+		int thread_limit = MB_avail / required;
+		if (thread_limit > com.max_thread)
+                        thread_limit = com.max_thread;
+
+		if (for_writer) {
+                        /* we allow the already allocated thread_limit images,
+                         * plus how many images can be stored in what remains
+                         * unused by the main processing */
+                        limit = thread_limit + (MB_avail - required * thread_limit) / MB_per_image;
+                } else limit = thread_limit;
+
+	}
+	if (limit == 0) {
+		gchar *mem_per_thread = g_format_size_full(required * BYTES_IN_A_MB, G_FORMAT_SIZE_IEC_UNITS);
+		gchar *mem_available = g_format_size_full(MB_avail * BYTES_IN_A_MB, G_FORMAT_SIZE_IEC_UNITS);
+
+		siril_log_color_message(_("%s: not enough memory to do this operation (%s required per image, %s considered available)\n"),
+				"red", args->description, mem_per_thread, mem_available);
+
+		g_free(mem_per_thread);
+		g_free(mem_available);
+	} else {
+#ifdef _OPENMP
+		if (for_writer) {
+			int max_queue_size = com.max_thread * 3;
+			if (limit > max_queue_size)
+				limit = max_queue_size;
+		}
+		siril_debug_print("Memory required per thread: %u MB, per image: %u MB, limiting to %d %s\n",
+				required, MB_per_image, limit, for_writer ? "images" : "threads");
+#else
+		if (!for_writer)
+			limit = 1;
+		else if (limit > 3)
+			limit = 3;
+#endif
+	}
+	return limit;
+}
+
 void apply_background_extraction_to_sequence(struct background_data *background_args) {
-	struct generic_seq_args *args = malloc(sizeof(struct generic_seq_args));
-	args->seq = background_args->seq;
-	args->force_float = FALSE;
-	args->partial_image = FALSE;
+	struct generic_seq_args *args = create_default_seqargs(background_args->seq);
 	args->filtering_criterion = seq_filter_included;
 	args->nb_filtered_images = background_args->seq->selnum;
-	args->compute_size_hook = NULL;
+	args->compute_mem_limits_hook = background_mem_limits_hook;
 	args->prepare_hook = seq_prepare_hook;
 	args->finalize_hook = seq_finalize_hook;
-	args->save_hook = NULL;
 	args->image_hook = background_image_hook;
-	args->idle_function = NULL;
 	args->stop_on_error = FALSE;
 	args->description = _("Background Extraction");
 	args->has_output = TRUE;
 	args->output_type = get_data_type(args->seq->bitpix);
-	args->upscale_ratio = 1.0;
 	args->new_seq_prefix = background_args->seqEntry;
 	args->load_new_sequence = TRUE;
-	args->force_ser_output = FALSE;
-	args->new_ser = NULL;
-	args->force_fitseq_output = FALSE;
-	args->new_fitseq = NULL;
 	args->user = background_args;
-	args->already_in_a_thread = FALSE;
-	args->parallel = TRUE;
 
 	background_args->fit = NULL;	// not used here
 
 	start_in_new_thread(generic_sequence_worker, args);
 }
 
-/************* CALLBACKS *************/
+/**** getter and setter ***/
 
-void on_menuitem_background_extraction_activate(GtkMenuItem *menuitem,
-		gpointer user_data) {
-	siril_open_dialog("background_extraction_dialog");
+gboolean background_sample_is_valid(background_sample *sample) {
+	return sample->valid;
 }
 
+gdouble background_sample_get_size(background_sample *sample) {
+	return sample->size;
+}
+
+point background_sample_get_position(background_sample *sample) {
+	return sample->position;
+}
+
+/************* CALLBACKS *************/
+
 void on_background_generate_clicked(GtkButton *button, gpointer user_data) {
 	set_cursor_waiting(TRUE);
 	int nb_of_samples;
@@ -746,8 +816,6 @@ void on_background_generate_clicked(GtkButton *button, gpointer user_data) {
 	tolerance = get_tolerance_value();
 
 	generate_background_samples(nb_of_samples, tolerance);
-
-	redraw(com.cvport, REMAP_ALL);
 	set_cursor_waiting(FALSE);
 }
 
@@ -755,12 +823,13 @@ void on_background_clear_all_clicked(GtkButton *button, gpointer user_data) {
 	free_background_sample_list(com.grad_samples);
 	com.grad_samples = NULL;
 
-	redraw(com.cvport, REMAP_ALL);
+	redraw(REDRAW_OVERLAY);
 	set_cursor_waiting(FALSE);
 }
 
 void on_background_ok_button_clicked(GtkButton *button, gpointer user_data) {
-	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget("checkBkgSeq")))
+	GtkToggleButton *seq_button = GTK_TOGGLE_BUTTON(lookup_widget("checkBkgSeq"));
+	if (gtk_toggle_button_get_active(seq_button)
 			&& sequence_is_loaded()) {
 		struct background_data *args = malloc(sizeof(struct background_data));
 
@@ -768,13 +837,13 @@ void on_background_ok_button_clicked(GtkButton *button, gpointer user_data) {
 		args->tolerance = get_tolerance_value();
 		args->correction = get_correction_type();
 		args->degree = get_poly_order();
+		args->dither = is_dither_checked();
 		if (args->degree > BACKGROUND_POLY_1) {
 			int confirm = siril_confirm_dialog(_("Polynomial order seems too high."),
 					_("You are about to process a sequence of preprocessed files with "
 							"a polynomial degree greater than 1. This is unlikely because such "
 							"gradients are often linear and a correction with a polynomial "
-							"function of degree 1 is probably enough. Click OK to confirm or "
-							"Cancel to change your mind."));
+							"function of degree 1 is probably enough."), _("Extract Background"));
 			if (!confirm) {
 				free(args);
 				set_cursor_waiting(FALSE);
@@ -788,6 +857,8 @@ void on_background_ok_button_clicked(GtkButton *button, gpointer user_data) {
 		if (args->seqEntry && args->seqEntry[0] == '\0')
 			args->seqEntry = "bkg_";
 		args->seq = &com.seq;
+		/* now we uncheck the button */
+		gtk_toggle_button_set_active(seq_button, FALSE);
 		apply_background_extraction_to_sequence(args);
 	} else {
 		if (com.grad_samples == NULL) {
@@ -797,13 +868,14 @@ void on_background_ok_button_clicked(GtkButton *button, gpointer user_data) {
 
 		int correction = get_correction_type();
 		poly_order degree = get_poly_order();
+		gboolean use_dither = is_dither_checked();
 		undo_save_state(&gfit, _("Background extraction (Correction: %s)"),
 				correction ? "Division" : "Subtraction");
-		remove_gradient_from_image(correction, degree);
+		remove_gradient_from_image(correction, degree, use_dither);
 
 		invalidate_stats_from_fit(&gfit);
 		adjust_cutoff_from_updated_gfit();
-		redraw(com.cvport, REMAP_ALL);
+		redraw(REMAP_ALL);
 		set_cursor_waiting(FALSE);
 	}
 }
@@ -816,7 +888,7 @@ void on_background_extraction_dialog_hide(GtkWidget *widget, gpointer user_data)
 	free_background_sample_list(com.grad_samples);
 	com.grad_samples = NULL;
 	mouse_status = MOUSE_ACTION_SELECT_REG_AREA;
-	redraw(com.cvport, REMAP_ALL);
+	redraw(REDRAW_OVERLAY);
 }
 
 void on_background_extraction_dialog_show(GtkWidget *widget, gpointer user_data) {
