@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -30,12 +30,13 @@
 #include "core/undo.h"
 #include "core/OS_utils.h"
 #include "gui/progress_and_log.h"
-#include "gui/callbacks.h"
+#include "gui/utils.h"
 #include "gui/image_display.h"
 #include "gui/image_interactions.h"
 #include "gui/message_dialog.h"
 #include "gui/histogram.h"
 #include "gui/dialogs.h"
+#include "gui/registration_preview.h"
 #include "io/single_image.h"
 #include "io/image_format_fits.h"
 #include "algos/colors.h"
@@ -418,49 +419,39 @@ double BV_to_T(double BV) {
 	return T;
 }
 
-int equalize_cfa_fit_with_coeffs(fits *fit, float coeff1, float coeff2, int config) {
-	unsigned int row, col;
-	float tmp1, tmp2;
+int equalize_cfa_fit_with_coeffs(fits *fit, float coeff1, float coeff2, const char *cfa_string) {
+	unsigned int row, col, pat_cell;
+	/* compute width of the (square) CFA pattern */
+	/* added 0.1 in case the result of sqrt is something like 5.999999 and it gets casted to int as 5 */
+	unsigned int pat_width = (unsigned int) (sqrt(strlen(cfa_string)) + 0.1);
 	if (fit->type == DATA_USHORT) {
 		WORD *data = fit->data;
-		for (row = 0; row < fit->ry - 1; row += 2) {
-			for (col = 0; col < fit->rx - 1; col += 2) {
-				if (config == 0) {
-					tmp1 = (float)data[1 + col + row * fit->rx] / coeff1;
-					data[1 + col + row * fit->rx] = round_to_WORD(tmp1);
-
-					tmp2 = (float)data[col + (1 + row) * fit->rx] / coeff2;
-					data[col + (1 + row) * fit->rx] = round_to_WORD(tmp2);
-
-				} else {
-					tmp1 = (float)data[col + row * fit->rx] / coeff1;
-					data[col + row * fit->rx] = round_to_WORD(tmp1);
-
-					tmp2 = (float)data[1 + col + (1 + row) * fit->rx] / coeff2;
-					data[1 + col + (1 + row) * fit->rx] = round_to_WORD(tmp2);
-
+		for (row = 0; row < fit->ry; row ++) {
+			for (col = 0; col < fit->rx; col++) {
+				pat_cell = (row % pat_width) * pat_width + col % pat_width;
+				switch (cfa_string[pat_cell]) {
+					case 'R':
+						data[col + row * fit->rx] = round_to_WORD(data[col + row * fit->rx] / coeff1);
+						break;
+					case 'B':
+						data[col + row * fit->rx] = round_to_WORD(data[col + row * fit->rx] / coeff2);
+						break;
 				}
 			}
 		}
 	}
 	else if (fit->type == DATA_FLOAT) {
 		float *data = fit->fdata;
-		for (row = 0; row < fit->ry - 1; row += 2) {
-			for (col = 0; col < fit->rx - 1; col += 2) {
-				if (config == 0) {
-					tmp1 = data[1 + col + row * fit->rx] / coeff1;
-					data[1 + col + row * fit->rx] = tmp1;
-
-					tmp2 = data[col + (1 + row) * fit->rx] / coeff2;
-					data[col + (1 + row) * fit->rx] = tmp2;
-
-				} else {
-					tmp1 = data[col + row * fit->rx] / coeff1;
-					data[col + row * fit->rx] = tmp1;
-
-					tmp2 = data[1 + col + (1 + row) * fit->rx] / coeff2;
-					data[1 + col + (1 + row) * fit->rx] = tmp2;
-
+		for (row = 0; row < fit->ry; row ++) {
+			for (col = 0; col < fit->rx; col++) {
+				pat_cell = (row % pat_width) * pat_width + col % pat_width;
+				switch (cfa_string[pat_cell]) {
+					case 'R':
+						data[col + row * fit->rx] /= coeff1;
+						break;
+					case 'B':
+						data[col + row * fit->rx] /= coeff2;
+						break;
 				}
 			}
 		}
@@ -495,7 +486,6 @@ static gpointer extract_channels_ushort(gpointer p) {
 				_("Siril cannot extract layers. Make sure your image is in RGB mode.\n"));
 		args->process = FALSE;
 		clearfits(args->fit);
-		siril_add_idle(end_extract_channels, args);
 		return GINT_TO_POINTER(1);
 	}
 
@@ -564,7 +554,6 @@ static gpointer extract_channels_ushort(gpointer p) {
 	clearfits(args->fit);
 	gettimeofday(&t_end, NULL);
 	show_time(t_start, t_end);
-	siril_add_idle(end_extract_channels, args);
 
 	return GINT_TO_POINTER(0);
 }
@@ -582,7 +571,6 @@ static gpointer extract_channels_float(gpointer p) {
 				_("Siril cannot extract layers. Make sure your image is in RGB mode.\n"));
 		args->process = FALSE;
 		clearfits(args->fit);
-		siril_add_idle(end_extract_channels, args);
 		return GINT_TO_POINTER(1);
 	}
 
@@ -649,19 +637,20 @@ static gpointer extract_channels_float(gpointer p) {
 	clearfits(args->fit);
 	gettimeofday(&t_end, NULL);
 	show_time(t_start, t_end);
-	siril_add_idle(end_extract_channels, args);
 
 	return GINT_TO_POINTER(0);
 }
 
 gpointer extract_channels(gpointer p) {
 	struct extract_channels_data *args = (struct extract_channels_data *)p;
+	gpointer retval = GINT_TO_POINTER(1);
+
 	if (args->fit->type == DATA_USHORT)
-		return extract_channels_ushort(p);
-	if (args->fit->type == DATA_FLOAT)
-		return extract_channels_float(p);
+		retval = extract_channels_ushort(p);
+	else if (args->fit->type == DATA_FLOAT)
+		retval = extract_channels_float(p);
 	siril_add_idle(end_extract_channels, args);
-	return GINT_TO_POINTER(1);
+	return retval;
 }
 
 /****************** Color calibration ************************/
@@ -676,13 +665,13 @@ void on_button_bkg_selection_clicked(GtkButton *button, gpointer user_data) {
 
 	if (!selection_black_value[0]) {
 		selection_black_value[0] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(builder, "spin_bkg_x"));
+				gtk_builder_get_object(gui.builder, "spin_bkg_x"));
 		selection_black_value[1] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(builder, "spin_bkg_y"));
+				gtk_builder_get_object(gui.builder, "spin_bkg_y"));
 		selection_black_value[2] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(builder, "spin_bkg_w"));
+				gtk_builder_get_object(gui.builder, "spin_bkg_w"));
 		selection_black_value[3] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(builder, "spin_bkg_h"));
+				gtk_builder_get_object(gui.builder, "spin_bkg_h"));
 	}
 
 	gtk_spin_button_set_value(selection_black_value[0], com.selection.x);
@@ -699,23 +688,23 @@ void initialize_calibration_interface() {
 
 	if (!selection_black_adjustment[0]) {
 		selection_black_adjustment[0] = GTK_ADJUSTMENT(
-				gtk_builder_get_object(builder, "adjustment_bkg_x"));
+				gtk_builder_get_object(gui.builder, "adjustment_bkg_x"));
 		selection_black_adjustment[1] = GTK_ADJUSTMENT(
-				gtk_builder_get_object(builder, "adjustment_bkg_y"));
+				gtk_builder_get_object(gui.builder, "adjustment_bkg_y"));
 		selection_black_adjustment[2] = GTK_ADJUSTMENT(
-				gtk_builder_get_object(builder, "adjustment_bkg_w"));
+				gtk_builder_get_object(gui.builder, "adjustment_bkg_w"));
 		selection_black_adjustment[3] = GTK_ADJUSTMENT(
-				gtk_builder_get_object(builder, "adjustment_bkg_h"));
+				gtk_builder_get_object(gui.builder, "adjustment_bkg_h"));
 	}
 	if (!selection_white_adjustment[0]) {
 		selection_white_adjustment[0] = GTK_ADJUSTMENT(
-				gtk_builder_get_object(builder, "adjustment_white_x"));
+				gtk_builder_get_object(gui.builder, "adjustment_white_x"));
 		selection_white_adjustment[1] = GTK_ADJUSTMENT(
-				gtk_builder_get_object(builder, "adjustment_white_y"));
+				gtk_builder_get_object(gui.builder, "adjustment_white_y"));
 		selection_white_adjustment[2] = GTK_ADJUSTMENT(
-				gtk_builder_get_object(builder, "adjustment_white_w"));
+				gtk_builder_get_object(gui.builder, "adjustment_white_w"));
 		selection_white_adjustment[3] = GTK_ADJUSTMENT(
-				gtk_builder_get_object(builder, "adjustment_white_h"));
+				gtk_builder_get_object(gui.builder, "adjustment_white_h"));
 	}
 	gtk_adjustment_set_upper(selection_black_adjustment[0], gfit.rx);
 	gtk_adjustment_set_upper(selection_black_adjustment[1], gfit.ry);
@@ -777,6 +766,7 @@ static void background_neutralize(fits* fit, rectangle black_selection) {
 	}
 
 	invalidate_stats_from_fit(fit);
+	invalidate_gfit_histogram();
 }
 
 void on_button_bkg_neutralization_clicked(GtkButton *button, gpointer user_data) {
@@ -786,13 +776,13 @@ void on_button_bkg_neutralization_clicked(GtkButton *button, gpointer user_data)
 
 	if (!selection_black_value[0]) {
 		selection_black_value[0] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(builder, "spin_bkg_x"));
+				gtk_builder_get_object(gui.builder, "spin_bkg_x"));
 		selection_black_value[1] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(builder, "spin_bkg_y"));
+				gtk_builder_get_object(gui.builder, "spin_bkg_y"));
 		selection_black_value[2] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(builder, "spin_bkg_w"));
+				gtk_builder_get_object(gui.builder, "spin_bkg_w"));
 		selection_black_value[3] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(builder, "spin_bkg_h"));
+				gtk_builder_get_object(gui.builder, "spin_bkg_h"));
 	}
 	width = (int) gtk_spin_button_get_value(selection_black_value[2]);
 	height = (int) gtk_spin_button_get_value(selection_black_value[3]);
@@ -813,9 +803,9 @@ void on_button_bkg_neutralization_clicked(GtkButton *button, gpointer user_data)
 	background_neutralize(&gfit, black_selection);
 	delete_selected_area();
 
-	redraw(com.cvport, REMAP_ALL);
-	redraw_previews();
 	update_gfit_histogram_if_needed();
+	redraw(REMAP_ALL);
+	redraw_previews();
 	set_cursor_waiting(FALSE);
 }
 
@@ -824,13 +814,13 @@ void on_button_white_selection_clicked(GtkButton *button, gpointer user_data) {
 
 	if (!selection_white_value[0]) {
 		selection_white_value[0] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(builder, "spin_white_x"));
+				gtk_builder_get_object(gui.builder, "spin_white_x"));
 		selection_white_value[1] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(builder, "spin_white_y"));
+				gtk_builder_get_object(gui.builder, "spin_white_y"));
 		selection_white_value[2] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(builder, "spin_white_w"));
+				gtk_builder_get_object(gui.builder, "spin_white_w"));
 		selection_white_value[3] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(builder, "spin_white_h"));
+				gtk_builder_get_object(gui.builder, "spin_white_h"));
 	}
 
 	if ((!com.selection.h) || (!com.selection.w)) {
@@ -994,6 +984,7 @@ static void white_balance(fits *fit, gboolean is_manual, rectangle white_selecti
 	}
 
 	invalidate_stats_from_fit(fit);
+	invalidate_gfit_histogram();
 }
 
 void on_calibration_apply_button_clicked(GtkButton *button, gpointer user_data) {
@@ -1054,7 +1045,7 @@ void on_calibration_apply_button_clicked(GtkButton *button, gpointer user_data)
 
 	delete_selected_area();
 
-	redraw(com.cvport, REMAP_ALL);
+	redraw(REMAP_ALL);
 	redraw_previews();
 	update_gfit_histogram_if_needed();
 	set_cursor_waiting(FALSE);
@@ -1104,33 +1095,20 @@ int pos_to_neg(fits *fit) {
 	return 0;
 }
 
-void on_menu_negative_activate(GtkMenuItem *menuitem, gpointer user_data) {
+void negative_processing() {
 	set_cursor_waiting(TRUE);
 	undo_save_state(&gfit, _("Negative Transformation"));
 	pos_to_neg(&gfit);
 	invalidate_stats_from_fit(&gfit);
 	invalidate_gfit_histogram();
 	update_gfit_histogram_if_needed();
-	redraw(com.cvport, REMAP_ALL);
+	redraw(REMAP_ALL);
 	redraw_previews();
 	set_cursor_waiting(FALSE);
 }
 
 /**********************************************************************/
 
-void on_menu_channel_separation_activate(GtkMenuItem *menuitem,
-		gpointer user_data) {
-	if (single_image_is_loaded() && isrgb(&gfit))
-		siril_open_dialog("extract_channel_dialog");
-}
-
-void on_menuitemcalibration_activate(GtkMenuItem *menuitem, gpointer user_data) {
-	if (single_image_is_loaded() && isrgb(&gfit)) {
-		initialize_calibration_interface();
-		siril_open_dialog("color_calibration");
-	}
-}
-
 void on_extract_channel_button_close_clicked(GtkButton *button,
 		gpointer user_data) {
 	siril_close_dialog("extract_channel_dialog");
@@ -1171,7 +1149,7 @@ void on_extract_channel_button_ok_clicked(GtkButton *button, gpointer user_data)
 	}
 
 	struct extract_channels_data *args = malloc(sizeof(struct extract_channels_data));
-	if (args == NULL) {
+	if (!args) {
 		PRINT_ALLOC_ERR;
 		return;
 	}
@@ -1198,8 +1176,17 @@ void on_extract_channel_button_ok_clicked(GtkButton *button, gpointer user_data)
 			&& (args->channel[2][0] != '\0')) {
 		args->fit = calloc(1, sizeof(fits));
 		set_cursor_waiting(TRUE);
-		copyfits(&gfit, args->fit, CP_ALLOC | CP_COPYA | CP_FORMAT, 0);
-		start_in_new_thread(extract_channels, args);
+		if (copyfits(&gfit, args->fit, CP_ALLOC | CP_COPYA | CP_FORMAT, -1)) {
+			siril_log_message(_("Could not copy the input image, aborting.\n"));
+			free(args->fit);
+			free(args->channel[0]);
+			free(args->channel[1]);
+			free(args->channel[2]);
+			free(args);
+		} else {
+			copy_fits_metadata(&gfit, args->fit);
+			start_in_new_thread(extract_channels, args);
+		}
 	}
 	else {
 		free(args->channel[0]);
