@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -27,9 +27,8 @@
 #include "core/processing.h"
 #include "core/OS_utils.h"
 #include "algos/statistics.h"
-#include "gui/callbacks.h"
+#include "gui/utils.h"
 #include "gui/progress_and_log.h"
-#include "opencv/opencv.h"
 
 #include "noise.h"
 
@@ -63,7 +62,7 @@ gpointer noise(gpointer p) {
 	}
 
 	for (chan = 0; chan < args->fit->naxes[2]; chan++) {
-		imstats *stat = statistics(NULL, -1, args->fit, chan, NULL, STATS_NOISE, TRUE);
+		imstats *stat = statistics(NULL, -1, args->fit, chan, NULL, STATS_SIGMEAN, TRUE);
 		if (!stat) {
 			args->retval = 1;
 			siril_log_message(_("Error: statistics computation failed.\n"));
@@ -94,13 +93,15 @@ gpointer noise(gpointer p) {
 }
 
 
-void on_info_menu_noise_estimation_clicked(GtkButton *button, gpointer user_data) {
+void evaluate_noise_in_image() {
 	if (get_thread_run()) {
 		PRINT_ANOTHER_THREAD_RUNNING;
 		return;
 	}
 
 	set_cursor_waiting(TRUE);
+
+	/* Switch to console tab */
 	control_window_switch_to_tab(OUTPUT_LOGS);
 
 	struct noise_data *args = malloc(sizeof(struct noise_data));
