@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -40,11 +40,15 @@
 #include <gsl/gsl_statistics.h>
 #include "core/siril.h"
 #include "core/proto.h"
+#include "core/processing.h"
 #include "gui/dialogs.h"
 #include "gui/progress_and_log.h"
+#include "io/image_format_fits.h"
+#include "io/sequence.h"
 #include "sorting.h"
 #include "statistics.h"
 #include "statistics_float.h"
+#include "core/OS_utils.h"
 
 /* Activating nullcheck will treat pixels with 0 value as null and remove them
  * from stats computation. This can be useful when a large area is black, but
@@ -53,7 +57,7 @@
  * Deactivating this will take less memory and make a faster statistics
  * computation. ngoodpix will be equal to total if deactivated.
  * Set to 0 to deactivate or 1 to activate. */
-#define ACTIVATE_NULLCHECK 0
+#define ACTIVATE_NULLCHECK 1
 
 static void stats_set_default_values(imstats *stat);
 
@@ -73,14 +77,44 @@ static void select_area_ushort(fits *fit, WORD *data, int layer, rectangle *boun
 	}
 }
 
+// sd computation for stacking.
+// In this case, N is the number of frames, so int is fine
+float siril_stats_ushort_sd_64(const WORD data[], const int N) {
+	guint64 intsum = 0;
+	for (int i = 0; i < N; ++i) {
+		intsum += data[i];
+	}
+	float mean = (float)(((double)intsum) / ((double)N));
+	double accumulator = 0.0;
+	for (int i = 0; i < N; ++i) {
+		float pixel = (float)data[i];
+		accumulator += (pixel - mean) * (pixel - mean);
+	}
+	return sqrtf((float) (accumulator / (N - 1)));
+}
+
+// 32 bits sum version, enough if there are less than 64k images
+float siril_stats_ushort_sd_32(const WORD data[], const int N) {
+	guint32 intsum = 0;
+	for (int i = 0; i < N; ++i) {
+		intsum += data[i];
+	}
+	float mean = (float)(((double)intsum) / ((double)N));
+	double accumulator = 0.0;
+	for (int i = 0; i < N; ++i) {
+		float pixel = (float)data[i];
+		accumulator += (pixel - mean) * (pixel - mean);
+	}
+	return sqrtf((float) (accumulator / (N - 1)));
+}
+
 /* For a univariate data set X1, X2, ..., Xn, the MAD is defined as the median
  * of the absolute deviations from the data's median:
  *  MAD = median (| Xi − median(X) |)
  */
-static double siril_stats_ushort_mad(WORD* data, const size_t n, const double m,
+float siril_stats_ushort_mad(const WORD* data, const size_t n, const double m,
 		gboolean multithread) {
-	size_t i;
-	double mad;
+	float mad;
 	int median = round_to_int(m);	// we use it on integer data anyway
 	WORD *tmp = malloc(n * sizeof(WORD));
 	if (!tmp) {
@@ -89,13 +123,13 @@ static double siril_stats_ushort_mad(WORD* data, const size_t n, const double m,
 	}
 
 #ifdef _OPENMP
-#pragma omp parallel for num_threads(com.max_thread) if(n > 10000) private(i) schedule(static)
+#pragma omp parallel for num_threads(com.max_thread) if(multithread && n > 10000) schedule(static)
 #endif
-	for (i = 0; i < n; i++) {
+	for (size_t i = 0; i < n; i++) {
 		tmp[i] = (WORD)abs(data[i] - median);
 	}
 
-	mad = histogram_median(tmp, n, multithread);
+	mad = (float) histogram_median(tmp, n, multithread);
 	free(tmp);
 	return mad;
 }
@@ -152,7 +186,7 @@ static WORD* reassign_to_non_null_data_ushort(WORD *data, size_t inputlen, size_
 }
 
 static void siril_stats_ushort_minmax(WORD *min_out, WORD *max_out,
-		const WORD data[], const size_t n) {
+		const WORD data[], const size_t n, gboolean multithread) {
 	/* finds the smallest and largest members of a dataset */
 
 	if (n > 0 && data) {
@@ -161,7 +195,7 @@ static void siril_stats_ushort_minmax(WORD *min_out, WORD *max_out,
 		size_t i;
 
 #ifdef _OPENMP
-#pragma omp parallel for num_threads(com.max_thread) schedule(static) if(n > 10000) reduction(max:max) reduction(min:min)
+#pragma omp parallel for num_threads(com.max_thread) schedule(static) if(multithread && n > 10000) reduction(max:max) reduction(min:min)
 #endif
 		for (i = 0; i < n; i++) {
 			WORD xi = data[i];
@@ -179,14 +213,14 @@ static void siril_stats_ushort_minmax(WORD *min_out, WORD *max_out,
 /* this function tries to get the requested stats from the passed stats,
  * computes them and stores them in it if they have not already been */
 static imstats* statistics_internal_ushort(fits *fit, int layer, rectangle *selection,
-		int option, imstats *stats, gboolean multithread) {
+		int option, imstats *stats, int bitpix, gboolean multithread) {
 	int nx, ny;
 	WORD *data = NULL;
 	int stat_is_local = 0, free_data = 0;
 	imstats* stat = stats;
 	// median is included in STATS_BASIC but required to compute other data
 	int compute_median = (option & STATS_BASIC) || (option & STATS_AVGDEV) ||
-		(option & STATS_MAD) || (option & STATS_BWMV);
+		(option & STATS_MAD) || (option & STATS_BWMV) || (option & STATS_IKSS);
 
 	if (!stat) {
 		allocate_stats(&stat);
@@ -220,27 +254,26 @@ static imstats* statistics_internal_ushort(fits *fit, int layer, rectangle *sele
 		}
 	}
 
+	if (stat->normValue == NULL_STATS) {
+		stat->normValue = (bitpix == BYTE_IMG) ? UCHAR_MAX_DOUBLE : USHRT_MAX_DOUBLE;
+	}
+
 	/* Calculation of min and max */
-	if ((option & (STATS_MINMAX | STATS_BASIC)) && (stat->min < 0. || stat->max < 0.)) {
-		// TODO 1.0: change to double
-		WORD min = 0, max = 0, norm = 0;
+	if ((option & (STATS_MINMAX | STATS_BASIC)) && (stat->min == NULL_STATS || stat->max == NULL_STATS)) {
+		WORD min = 0, max = 0;
 		if (!data) {
 			if (stat_is_local) free(stat);
 			return NULL;	// not in cache, don't compute
 		}
 		siril_debug_print("- stats %p fit %p (%d): computing minmax\n", stat, fit, layer);
-		siril_stats_ushort_minmax(&min, &max, data, stat->total);
-		if (fit->bitpix == BYTE_IMG)
-			norm = UCHAR_MAX;
-		else norm = USHRT_MAX;
+		siril_stats_ushort_minmax(&min, &max, data, stat->total, multithread);
 		stat->min = (double) min;
 		stat->max = (double) max;
-		stat->normValue = (double) norm;
 	}
 
 	/* Calculation of ngoodpix, mean, sigma and background noise */
-	if ((option & (STATS_NOISE | STATS_BASIC)) && (stat->ngoodpix <= 0L || stat->mean < 0. ||
-			stat->sigma < 0. || stat->bgnoise < 0.)) {
+	if ((option & (STATS_SIGMEAN | STATS_BASIC)) && (stat->ngoodpix <= 0L || stat->mean == NULL_STATS ||
+		stat->sigma == NULL_STATS || stat->bgnoise == NULL_STATS)) {
 		int status = 0;
 		if (!data) {
 			if (stat_is_local) free(stat);
@@ -256,6 +289,7 @@ static imstats* statistics_internal_ushort(fits *fit, int layer, rectangle *sele
 			return NULL;
 		}
 	}
+
 	if (stat->ngoodpix == 0L) {
 		if (free_data) free(data);
 		if (stat_is_local) free(stat);
@@ -264,7 +298,7 @@ static imstats* statistics_internal_ushort(fits *fit, int layer, rectangle *sele
 
 	/* we exclude 0 if some computations remain to be done or copy data if
 	 * median has to be computed (this is deactivated in the ngoodpix computation) */
-	if (fit && (compute_median || (option & STATS_IKSS)) && stat->total != stat->ngoodpix) {
+	if (fit && compute_median && stat->total != stat->ngoodpix) {
 		data = reassign_to_non_null_data_ushort(data, stat->total, stat->ngoodpix, free_data);
 		if (!data) {
 			if (stat_is_local) free(stat);
@@ -274,7 +308,7 @@ static imstats* statistics_internal_ushort(fits *fit, int layer, rectangle *sele
 	}
 
 	/* Calculation of median */
-	if (compute_median && stat->median < 0.) {
+	if (compute_median && stat->median == NULL_STATS) {
 		if (!data) {
 			if (stat_is_local) free(stat);
 			return NULL;	// not in cache, don't compute
@@ -284,7 +318,7 @@ static imstats* statistics_internal_ushort(fits *fit, int layer, rectangle *sele
 	}
 
 	/* Calculation of average absolute deviation from the median */
-	if ((option & STATS_AVGDEV) && stat->avgDev < 0.) {
+	if ((option & STATS_AVGDEV) && stat->avgDev == NULL_STATS) {
 		if (!data) {
 			if (stat_is_local) free(stat);
 			return NULL;	// not in cache, don't compute
@@ -294,7 +328,7 @@ static imstats* statistics_internal_ushort(fits *fit, int layer, rectangle *sele
 	}
 
 	/* Calculation of median absolute deviation */
-	if (((option & STATS_MAD) || (option & STATS_BWMV)) && stat->mad < 0.) {
+	if (((option & STATS_MAD) || (option & STATS_BWMV) || (option & STATS_IKSS)) && stat->mad == NULL_STATS) {
 		if (!data) {
 			if (stat_is_local) free(stat);
 			return NULL;	// not in cache, don't compute
@@ -304,7 +338,7 @@ static imstats* statistics_internal_ushort(fits *fit, int layer, rectangle *sele
 	}
 
 	/* Calculation of Bidweight Midvariance */
-	if ((option & STATS_BWMV) && stat->sqrtbwmv < 0.) {
+	if ((option & STATS_BWMV) && stat->sqrtbwmv == NULL_STATS) {
 		if (!data) {
 			if (stat_is_local) free(stat);
 			return NULL;	// not in cache, don't compute
@@ -314,8 +348,9 @@ static imstats* statistics_internal_ushort(fits *fit, int layer, rectangle *sele
 		stat->sqrtbwmv = sqrt(bwmv);
 	}
 
+
 	/* Calculation of IKSS. Only used for stacking normalization */
-	if ((option & STATS_IKSS) && (stat->location < 0. || stat->scale < 0.)) {
+	if ((option & STATS_IKSS) && (stat->location == NULL_STATS || stat->scale == NULL_STATS)) {
 		if (!data) {
 			if (stat_is_local) free(stat);
 			return NULL;	// not in cache, don't compute
@@ -329,24 +364,26 @@ static imstats* statistics_internal_ushort(fits *fit, int layer, rectangle *sele
 			PRINT_ALLOC_ERR;
 			return NULL;
 		}
-
+		double normValue = (fit->bitpix == BYTE_IMG) ? UCHAR_MAX_DOUBLE : USHRT_MAX_DOUBLE;
 		/* we convert in the [0, 1] range */
-		float invertNormValue = (float)(1.0 / stat->normValue);
+		float invertNormValue = (float)(1.0 / normValue);
 #ifdef _OPENMP
 #pragma omp parallel for num_threads(com.max_thread) if (multithread) private(i) schedule(static)
 #endif
 		for (i = 0; i < stat->ngoodpix; i++) {
 			newdata[i] = (float) data[i] * invertNormValue;
 		}
-		if (IKSS(newdata, stat->ngoodpix, &stat->location, &stat->scale, multithread)) {
+		float med = (float)(stat->median) * invertNormValue;
+		float mad = (float)(stat->mad) * invertNormValue;
+		if (IKSSlite(newdata, stat->ngoodpix, med, mad, &stat->location, &stat->scale, multithread)) {
 			if (stat_is_local) free(stat);
 			if (free_data) free(data);
 			free(newdata);
 			return NULL;
 		}
 		/* go back to the original range */
-		stat->location *= stat->normValue;
-		stat->scale *= stat->normValue;
+		stat->location *= normValue;
+		stat->scale *= normValue;
 		free(newdata);
 	}
 
@@ -354,14 +391,16 @@ static imstats* statistics_internal_ushort(fits *fit, int layer, rectangle *sele
 	return stat;
 }
 
-static imstats* statistics_internal(fits *fit, int layer, rectangle *selection, int option, imstats *stats, gboolean multithread) {
+static imstats* statistics_internal(fits *fit, int layer, rectangle *selection, int option, imstats *stats, int bitpix, gboolean multithread) {
 	if (fit) {
 		if (fit->type == DATA_USHORT)
-			return statistics_internal_ushort(fit, layer, selection, option, stats, multithread);
+			return statistics_internal_ushort(fit, layer, selection, option, stats, bitpix, multithread);
 		if (fit->type == DATA_FLOAT)
-			return statistics_internal_float(fit, layer, selection, option, stats, multithread);
+			return statistics_internal_float(fit, layer, selection, option, stats, bitpix, multithread);
 	}
-	return statistics_internal_ushort(fit, layer, selection, option, stats, multithread);
+	if (bitpix == FLOAT_IMG)
+		return statistics_internal_float(fit, layer, selection, option, stats, bitpix, multithread);
+	return statistics_internal_ushort(fit, layer, selection, option, stats, bitpix, multithread);
 }
 
 /* Computes statistics on the given layer of the given opened image.
@@ -382,14 +421,14 @@ imstats* statistics(sequence *seq, int image_index, fits *fit, int layer, rectan
 	imstats *oldstat = NULL, *stat;
 	if (selection && selection->h > 0 && selection->w > 0) {
 		// we have a selection, don't store anything
-		return statistics_internal(fit, layer, selection, option, NULL, multithread);
+		return statistics_internal(fit, layer, selection, option, NULL, fit->bitpix, multithread);
 	} else if (!seq || image_index < 0) {
 		// we have a single image, store in the fits
 		if (fit->stats && fit->stats[layer]) {
 			oldstat = fit->stats[layer];
 			atomic_int_incref(oldstat->_nb_refs);
 		}
-		stat = statistics_internal(fit, layer, NULL, option, oldstat, multithread);
+		stat = statistics_internal(fit, layer, NULL, option, oldstat, fit->bitpix, multithread);
 		if (!stat) {
 			fprintf(stderr, "- stats failed for fit %p (%d)\n", fit, layer);
 			if (oldstat) {
@@ -408,7 +447,7 @@ imstats* statistics(sequence *seq, int image_index, fits *fit, int layer, rectan
 			if (oldstat)	// can be NULL here
 				atomic_int_incref(oldstat->_nb_refs);
 		}
-		stat = statistics_internal(fit, layer, NULL, option, oldstat, multithread);
+		stat = statistics_internal(fit, layer, NULL, option, oldstat, seq->bitpix, multithread);
 		if (!stat) {
 			if (fit)
 				fprintf(stderr, "- stats failed for %d in seq (%d)\n",
@@ -427,13 +466,12 @@ imstats* statistics(sequence *seq, int image_index, fits *fit, int layer, rectan
 	}
 }
 
-int compute_means_from_flat_cfa_ushort(fits *fit, float mean[4]) {
-	int row, col, c, i = 0;
+int compute_means_from_flat_cfa_ushort(fits *fit, double mean[36]) {
+	int row, col, c, i[36] = {0};
 	WORD *data;
 	unsigned int width, height;
 	unsigned int startx, starty;
 
-	mean[0] = mean[1] = mean[2] = mean[3] = 0.0;
 	data = fit->data;
 	width = fit->rx;
 	height = fit->ry;
@@ -444,31 +482,27 @@ int compute_means_from_flat_cfa_ushort(fits *fit, float mean[4]) {
 	startx = width / 3;
 	starty = height / 3;
 
-	/* make sure it does start at the beginning of CFA pattern */
-	startx = startx % 2 == 0 ? startx : startx + 1;
-	starty = starty % 2 == 0 ? starty : starty + 1;
-
 	siril_debug_print("Computing stat in (%d, %d, %d, %d)\n", startx, starty,
 			width - 1 - startx, height - 1 - starty);
 
-	/* Compute mean of each channel */
-	for (row = starty; row < height - 1 - starty; row += 2) {
-		for (col = startx; col < width - 1 - startx; col += 2) {
-			mean[0] += (float) data[col + row * width];
-			mean[1] += (float) data[1 + col + row * width];
-			mean[2] += (float) data[col + (1 + row) * width];
-			mean[3] += (float) data[1 + col + (1 + row) * width];
-			i++;
+	/* compute mean of each element in 6x6 blocks */
+	for (row = starty; row < height - starty; row++) {
+		for (col = startx; col < width - startx; col++) {
+			mean[(col % 6) + (row % 6) * 6] += (double) data[col + row * width];
+			i[(col % 6) + (row % 6) * 6]++;
 		}
 	}
 
-	for (c = 0; c < 4; c++) {
-		mean[c] /= (float) i;
+	for (c = 0; c < 36; c++) {
+		mean[c] /= (double) i[c];
 	}
 	return 0;
 }
 
-int compute_means_from_flat_cfa(fits *fit, float mean[4]) {
+/*  the mean is computed by adding data over and over in a long loop; 
+    but if the mean variable is a single-precision float, this results in
+    the accumulation of rounding errors. */
+int compute_means_from_flat_cfa(fits *fit, double mean[36]) {
 	if (fit->type == DATA_USHORT)
 		return compute_means_from_flat_cfa_ushort(fit, mean);
 	if (fit->type == DATA_FLOAT)
@@ -584,7 +618,7 @@ void full_stats_invalidation_from_fit(fits *fit) {
 static void stats_set_default_values(imstats *stat) {
 	stat->total = -1L;
 	stat->ngoodpix = -1L;
-	stat->mean = stat->avgDev = stat->median = stat->sigma = stat->bgnoise = stat->min = stat->max = stat->normValue = stat->mad = stat->sqrtbwmv = stat->location = stat->scale = -1.0;
+	stat->mean = stat->avgDev = stat->median = stat->sigma = stat->bgnoise = stat->min = stat->max = stat->normValue = stat->mad = stat->sqrtbwmv = stat->location = stat->scale = NULL_STATS;
 }
 
 /* allocates an imstat structure and initializes it with default values that
@@ -630,6 +664,218 @@ void clear_stats(sequence *seq, int layer) {
 	}
 }
 
+/** generic function for sequences */
+
+static void free_stat_list(gchar **list, int nb) {
+	for (int i = 0; i < nb; i++) {
+		g_free(list[i]);
+	}
+	free(list);
+}
+
+static int stat_prepare_hook(struct generic_seq_args *args) {
+	struct stat_data *s_args = (struct stat_data*) args->user;
+	s_args->list = calloc(args->nb_filtered_images * s_args->seq->nb_layers, sizeof(char*));
+
+	return 0;
+}
+
+static int stat_image_hook(struct generic_seq_args *args, int o, int i, fits *fit,
+		rectangle *_) {
+	struct stat_data *s_args = (struct stat_data*) args->user;
+
+	for (int layer = 0; layer < fit->naxes[2]; layer++) {
+		/* we first check for data in cache */
+		imstats* stat = statistics(args->seq, i, NULL, layer, &s_args->selection, s_args->option, TRUE);
+		if (!stat) {
+			/* if no cache */
+			stat = statistics(args->seq, i, fit, layer, &s_args->selection, s_args->option, TRUE);
+			if (!stat) {
+				siril_log_message(_("Error: statistics computation failed.\n"));
+				return 1;
+			}
+		}
+
+		int new_index = i * s_args->seq->nb_layers;
+
+		if (fit->type == DATA_USHORT) {
+			if (s_args->option == (STATS_BASIC)) {
+			 	s_args->list[new_index + layer] = g_strdup_printf("%d\t%d\t%e\t%e\t%e\t%e\t%e\t%e\n",
+			 			i + 1,
+			 			layer,
+			 			stat->mean / USHRT_MAX_DOUBLE,
+			 			stat->median / USHRT_MAX_DOUBLE,
+			 			stat->sigma / USHRT_MAX_DOUBLE,
+			 			stat->min / USHRT_MAX_DOUBLE,
+			 			stat->max / USHRT_MAX_DOUBLE,
+			 			stat->bgnoise / USHRT_MAX_DOUBLE
+			 	);
+			} else {
+				s_args->list[new_index + layer] = g_strdup_printf("%d\t%d\t%e\t%e\t%e\t%e\t%e\t%e\t%e\t%e\t%e\n",
+						i + 1,
+						layer,
+						stat->mean / USHRT_MAX_DOUBLE,
+						stat->median / USHRT_MAX_DOUBLE,
+						stat->sigma / USHRT_MAX_DOUBLE,
+						stat->min / USHRT_MAX_DOUBLE,
+						stat->max / USHRT_MAX_DOUBLE,
+						stat->bgnoise / USHRT_MAX_DOUBLE,
+						stat->avgDev / USHRT_MAX_DOUBLE,
+						stat->mad / USHRT_MAX_DOUBLE,
+						stat->sqrtbwmv / USHRT_MAX_DOUBLE
+				);
+			}
+		} else {
+			if (s_args->option == (STATS_BASIC)) {
+			 	s_args->list[new_index + layer] = g_strdup_printf("%d\t%d\t%e\t%e\t%e\t%e\t%e\t%e\n",
+			 			i + 1,
+			 			layer,
+			 			stat->mean,
+			 			stat->median,
+			 			stat->sigma,
+			 			stat->min,
+			 			stat->max,
+			 			stat->bgnoise
+			 	);
+			} else {
+				s_args->list[new_index + layer] = g_strdup_printf("%d\t%d\t%e\t%e\t%e\t%e\t%e\t%e\t%e\t%e\t%e\n",
+						i + 1,
+						layer,
+						stat->mean,
+						stat->median,
+						stat->sigma,
+						stat->min,
+						stat->max,
+						stat->bgnoise,
+						stat->avgDev,
+						stat->mad,
+						stat->sqrtbwmv
+				);
+			}
+		}
+
+		free_stats(stat);
+	}
+	return 0;
+}
+
+static int stat_finalize_hook(struct generic_seq_args *args) {
+	GError *error = NULL;
+	struct stat_data *s_args = (struct stat_data*) args->user;
+
+	int size = s_args->seq->nb_layers * args->nb_filtered_images;
+	GFile *file = g_file_new_for_path(s_args->csv_name);
+	GOutputStream* output_stream = (GOutputStream*) g_file_replace(file, NULL, FALSE, G_FILE_CREATE_NONE, NULL, &error);
+	g_free(s_args->csv_name);
+	if (output_stream == NULL) {
+		if (error != NULL) {
+			g_warning("%s\n", error->message);
+			g_clear_error(&error);
+			fprintf(stderr, "Cannot save histo\n");
+		}
+		g_object_unref(file);
+		free_stat_list(s_args->list, size);
+		free(s_args);
+		return 1;
+	}
+	const gchar *header;
+	if (s_args->option == (STATS_BASIC)) {
+		header = "image\tchan\tmean\tmedian\tsigma\tmin\tmax\tnoise\n";
+	} else {
+		header = "image\tchan\tmean\tmedian\tsigma\tmin\tmax\tnoise\tavgDev\tmad\tsqrtbwmv\n";
+	}
+	if (!g_output_stream_write_all(output_stream, header, strlen(header), NULL, NULL, &error)) {
+		g_warning("%s\n", error->message);
+		g_clear_error(&error);
+		g_object_unref(output_stream);
+		g_object_unref(file);
+		free_stat_list(s_args->list, size);
+		free(s_args);
+		return 1;
+	}
+
+	for (int i = 0; i < args->nb_filtered_images * s_args->seq->nb_layers; i++) {
+		if (!s_args->list[i]) continue; //stats can fail
+		if (!g_output_stream_write_all(output_stream, s_args->list[i], strlen(s_args->list[i]), NULL, NULL, &error)) {
+			g_warning("%s\n", error->message);
+			g_clear_error(&error);
+			g_object_unref(output_stream);
+			g_object_unref(file);
+			free_stat_list(s_args->list, size);
+			free(s_args);
+			return 1;
+		}
+	}
+
+	siril_log_message(_("Statistic file %s was successfully created.\n"), g_file_peek_path(file));
+	writeseqfile(args->seq);
+	g_object_unref(output_stream);
+	g_object_unref(file);
+	free_stat_list(s_args->list, size);
+	free(s_args);
+
+	return 0;
+}
+
+static int stat_compute_mem_limit(struct generic_seq_args *args, gboolean for_writer) {
+	unsigned int MB_per_image, MB_avail, required;
+	int limit = compute_nb_images_fit_memory(args->seq, 1.0, FALSE, &MB_per_image, NULL, &MB_avail);
+
+	int is_color = args->seq->nb_layers == 3;
+	required = is_color ? MB_per_image * 4 / 3 : MB_per_image * 2;
+
+	if (limit > 0) {
+		int thread_limit = MB_avail / required;
+		if (thread_limit > com.max_thread)
+                        thread_limit = com.max_thread;
+		limit = thread_limit;
+
+		/* should not happen */
+		if (for_writer)
+			return 1;
+	}
+	if (limit == 0) {
+		gchar *mem_per_thread = g_format_size_full(required * BYTES_IN_A_MB, G_FORMAT_SIZE_IEC_UNITS);
+		gchar *mem_available = g_format_size_full(MB_avail * BYTES_IN_A_MB, G_FORMAT_SIZE_IEC_UNITS);
+
+		siril_log_color_message(_("%s: not enough memory to do this operation (%s required per image, %s considered available)\n"),
+				"red", args->description, mem_per_thread, mem_available);
+
+		g_free(mem_per_thread);
+		g_free(mem_available);
+	} else {
+#ifdef _OPENMP
+		siril_debug_print("Memory required per thread: %u MB, per image: %u MB, limiting to %d %s\n",
+				required, MB_per_image, limit, for_writer ? "images" : "threads");
+#else
+		if (!for_writer)
+			limit = 1;
+#endif
+	}
+	return limit;
+}
+
+
+void apply_stats_to_sequence(struct stat_data *stat_args) {
+	struct generic_seq_args *args = create_default_seqargs(stat_args->seq);
+	args->seq = stat_args->seq;
+	args->filtering_criterion = seq_filter_included;
+	args->nb_filtered_images = stat_args->seq->selnum;
+	args->compute_mem_limits_hook = stat_compute_mem_limit;
+	args->prepare_hook = stat_prepare_hook;
+	args->finalize_hook = stat_finalize_hook;
+	args->image_hook = stat_image_hook;
+	args->description = _("Statistics");
+	args->has_output = FALSE;
+	args->output_type = get_data_type(args->seq->bitpix);
+	args->new_seq_prefix = NULL;
+	args->user = stat_args;
+
+	stat_args->fit = NULL;	// not used here
+
+	start_in_new_thread(generic_sequence_worker, args);
+}
+
 /**** callbacks *****/
 
 void on_menu_gray_stat_activate(GtkMenuItem *menuitem, gpointer user_data) {
@@ -638,10 +884,3 @@ void on_menu_gray_stat_activate(GtkMenuItem *menuitem, gpointer user_data) {
 	siril_open_dialog("StatWindow");
 	set_cursor_waiting(FALSE);
 }
-
-void on_info_menu_statistics_clicked(GtkButton *button, gpointer user_data) {
-	set_cursor_waiting(TRUE);
-	computeStat();
-	siril_open_dialog("StatWindow");
-	set_cursor_waiting(FALSE);
-}
