@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -18,10 +18,6 @@
  * along with Siril. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE // for getline
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
@@ -30,15 +26,21 @@
 #include "core/proto.h"
 #include "core/initfile.h"
 #include "core/OS_utils.h"
-#include "gui/callbacks.h"
+#include "gui/utils.h"
 #include "gui/progress_and_log.h"
+#include "gui/registration_preview.h"
+#include "gui/image_interactions.h"
+#include "gui/image_display.h"
+#include "gui/callbacks.h"
 #include "core/processing.h"
 #include "core/command_list.h"
 #include "io/sequence.h"
+#include "io/single_image.h"
+#include "io/ser.h"
 
 #include "command_line_processor.h"
 
-static void parseLine(char *myline, int len, int *nb) {
+static void parse_line(char *myline, int len, int *nb) {
 	int i = 0, wordnb = 0;
 	char string_starter = '\0';	// quotes don't split words on spaces
 	word[0] = NULL;
@@ -48,7 +50,7 @@ static void parseLine(char *myline, int len, int *nb) {
 			i++;
 		if (myline[i] == '"' || myline[i] == '\'')
 			string_starter = myline[i++];
-		if (myline[i] == '\0' || myline[i] == '\n')
+		if (myline[i] == '\0' || myline[i] == '\n' || myline[i] == '\r')
 			break;
 		word[wordnb++] = myline + i;	// the beginning of the word
 		word[wordnb] = NULL;		// put next word to NULL
@@ -67,18 +69,19 @@ static void parseLine(char *myline, int len, int *nb) {
 	*nb = wordnb;
 }
 
-static void removeEOL(char *text) {
-	int i = strlen(text) - 1;
-	while (i >= 0 && (text[i] == '\n' || text[i] == '\r'))
-		text[i] = '\0';
+static void remove_trailing_cr(char *str) {
+	if (str == NULL || str[0] == '\0')
+		return;
+	int length = strlen(str);
+	if (str[length - 1] == '\r')
+		str[length - 1] = '\0';
 }
 
-static int executeCommand(int wordnb) {
-	int i;
+static int execute_command(int wordnb) {
 	// search for the command in the list
 	if (word[0] == NULL) return 1;
-	i = G_N_ELEMENTS(commands);
-	while (strcasecmp (commands[--i].name, word[0])) {
+	int i = G_N_ELEMENTS(commands);
+	while (g_ascii_strcasecmp(commands[--i].name, word[0])) {
 		if (i == 0) {
 			siril_log_message(_("Unknown command: '%s' or not implemented yet\n"), word[0]);
 			return 1 ;
@@ -101,7 +104,7 @@ static int executeCommand(int wordnb) {
 
 	// process the command
 	siril_log_color_message(_("Running command: %s\n"), "salmon", word[0]);
-	fprintf(stderr, "%lu: running command %s\n", time(NULL), word[0]);
+	fprintf(stdout, "%lu: running command %s\n", time(NULL), word[0]);
 	return commands[i].process(wordnb);
 }
 
@@ -114,7 +117,7 @@ static void update_log_icon(gboolean is_running) {
 }
 
 struct log_status_bar_idle_data {
-	char *myline;
+	gchar *myline;
 	int line;
 };
 
@@ -128,14 +131,12 @@ static gboolean log_status_bar_idle_callback(gpointer p) {
 	update_log_icon(TRUE);
 
 	newline = g_strdup(data->myline);
-	removeEOL(newline);
 	status = g_strdup_printf(_("Processing line %d: %s"), data->line, newline);
 
 	gtk_statusbar_push(statusbar_script, 0, status);
 	g_free(newline);
 	g_free(status);
-
-	free(data->myline);
+	g_free(data->myline);
 	free(data);
 
 	return FALSE;	// only run once
@@ -147,32 +148,40 @@ static void display_command_on_status_bar(int line, char *myline) {
 
 		data = malloc(sizeof(struct log_status_bar_idle_data));
 		data->line = line;
-		data->myline = myline ? strdup(myline) : NULL;
+		data->myline = myline ? g_strdup(myline) : NULL;
 		gdk_threads_add_idle(log_status_bar_idle_callback, data);
 	}
 }
 
 static void clear_status_bar() {
-	if (!com.headless) {
-		GtkStatusbar *bar = GTK_STATUSBAR(lookup_widget("statusbar_script"));
-		gtk_statusbar_remove_all(bar, 0);
-		update_log_icon(FALSE);
-	}
+	GtkStatusbar *bar = GTK_STATUSBAR(lookup_widget("statusbar_script"));
+	gtk_statusbar_remove_all(bar, 0);
+	update_log_icon(FALSE);
 }
 
 static gboolean end_script(gpointer p) {
+	/* GTK+ code is ignored during scripts, this is a good place to redraw everything */
 	clear_status_bar();
 	set_GUI_CWD();
-	
+	update_MenuItem();
+	adjust_cutoff_from_updated_gfit();
+	redraw(REMAP_ALL);
+	redraw_previews();
+	update_zoom_label();
+	update_display_fwhm();
+	display_filename();
+	new_selection_zone();
+	set_GUI_misc();
+	set_GUI_compression();
+	update_spinCPU(0);
 	set_cursor_waiting(FALSE);
 	return FALSE;
 }
 
 gpointer execute_script(gpointer p) {
-	FILE *fp = (FILE *)p;
+	GInputStream *input_stream = (GInputStream*) p;
 	gboolean check_required = FALSE;
-	ssize_t read;
-	char *linef, *myline;
+	gchar *buffer;
 	int line = 0, retval = 0;
 	int wordnb;
 	int startmem, endmem;
@@ -180,83 +189,98 @@ gpointer execute_script(gpointer p) {
 
 	com.script = TRUE;
 	com.stop_script = FALSE;
+
+	gettimeofday(&t_start, NULL);
+
 	/* Now we want to save the cwd in order to come back after
 	 * script execution
 	 */
 	gchar *saved_cwd = g_strdup(com.wd);
-	gettimeofday(&t_start, NULL);
-	startmem = get_available_memory_in_MB();
-#if (_POSIX_C_SOURCE < 200809L)
-	linef = calloc(256, sizeof(char));
-	while (fgets(linef, 256, fp)) {
-		read = strlen(linef) + 1;
-#else
-	size_t lenf = 0;
-	linef = NULL;
-	while ((read = getline(&linef, &lenf, fp)) != -1) {
-#endif
+	startmem = get_available_memory() / BYTES_IN_A_MB;
+	gsize length = 0;
+	GDataInputStream *data_input = g_data_input_stream_new(input_stream);
+	while ((buffer = g_data_input_stream_read_line_utf8(data_input, &length,
+			NULL, NULL))) {
 		++line;
 		if (com.stop_script) {
 			retval = 1;
+			g_free (buffer);
 			break;
 		}
 		/* Displays comments */
-		if (linef[0] == '#') {
-			siril_log_color_message(linef, "blue");
+		if (buffer[0] == '#') {
+			siril_log_color_message("%s\n", "blue", buffer);
+			g_free (buffer);
 			continue;
 		}
-		if (linef[0] == '\0' || linef[0] == '\n')
-			continue;
 
-		myline = strdup(linef);
-		display_command_on_status_bar(line, myline);
-		parseLine(myline, read, &wordnb);
+		/* in Windows case, remove trailing CR */
+		remove_trailing_cr(buffer);
+
+		if (buffer[0] == '\0') {
+			g_free (buffer);
+			continue;
+		}
+
+
+		display_command_on_status_bar(line, buffer);
+		parse_line(buffer, length, &wordnb);
+		/* check for requires command */
 		if (!g_ascii_strcasecmp(word[0], "requires")) {
 			check_required = TRUE;
 		} else {
-			if (com.pref.check_script_version && check_required == FALSE) {
-				siril_log_color_message(_("The \"requires\" command is needed at the top the script file."
+			if (com.pref.script_check_requires && !check_required) {
+				siril_log_color_message(_("The \"requires\" command is missing at the top of the script file."
 						" This command is needed to check script compatibility.\n"), "red");
 				retval = 1;
+				g_free (buffer);
 				break;
 			}
 		}
-		if ((retval = executeCommand(wordnb))) {
-			removeEOL(linef);
-			siril_log_message(_("Error in line %d: '%s'.\n"), line, linef);
+		if ((retval = execute_command(wordnb))) {
+			siril_log_message(_("Error in line %d: '%s'.\n"), line, buffer);
 			siril_log_message(_("Exiting batch processing.\n"));
-			free(myline);
+			g_free (buffer);
 			break;
 		}
 		if (waiting_for_thread()) {
-			free(myline);
 			retval = 1;
+			g_free (buffer);
 			break;	// abort script on command failure
 		}
-		endmem = get_available_memory_in_MB();
+		endmem = get_available_memory() / BYTES_IN_A_MB;
 		siril_debug_print("End of command %s, memory difference: %d MB\n", word[0], startmem - endmem);
 		startmem = endmem;
 		memset(word, 0, sizeof word);
-		free(myline);
+		g_free (buffer);
 	}
-	free(linef);
-	fclose(fp);
-	com.script = FALSE;
+	g_object_unref(data_input);
+	g_object_unref(input_stream);
+
+	if (!com.headless) {
+		com.script = FALSE;
+		siril_add_idle(end_script, NULL);
+	}
+
 	/* Now we want to restore the saved cwd */
-	changedir(saved_cwd, NULL);
+	siril_change_dir(saved_cwd, NULL);
 	writeinitfile();
-	siril_add_idle(end_script, NULL);
 	if (!retval) {
 		siril_log_message(_("Script execution finished successfully.\n"));
 		gettimeofday(&t_end, NULL);
 		show_time_msg(t_start, t_end, _("Total execution time"));
 	} else {
 		char *msg = siril_log_message(_("Script execution failed.\n"));
-		msg[strlen(msg)-1] = '\0';
+		msg[strlen(msg) - 1] = '\0';
 		set_progress_bar_data(msg, PROGRESS_DONE);
 	}
 	g_free(saved_cwd);
-	fprintf(stderr, "Script thread exiting\n");
+
+	if (com.script_thread) {
+		siril_debug_print("Script thread exiting\n");
+		g_thread_unref(com.script_thread);
+		com.script_thread = NULL;
+	}
 	return GINT_TO_POINTER(retval);
 }
 
@@ -275,7 +299,7 @@ static void show_command_help_popup(GtkEntry *entry) {
 
 				token = g_strsplit_set(current->usage, " ", -1);
 				GString *str = g_string_new(token[0]);
-				str = g_string_prepend(str, "<span foreground=\"red\"><b>");
+				str = g_string_prepend(str, "<span foreground=\"red\" size=\"larger\"><b>");
 				str = g_string_append(str, "</b>");
 				if (token[1] != NULL) {
 					str = g_string_append(str,
@@ -285,10 +309,12 @@ static void show_command_help_popup(GtkEntry *entry) {
 				str = g_string_append(str, _(current->definition));
 				str = g_string_append(str, "\n\n<b>");
 				str = g_string_append(str, _("Can be used in a script: "));
-				str = g_string_append(str, "<span foreground=\"red\">");
+
 				if (current->scriptable) {
+					str = g_string_append(str, "<span foreground=\"green\">");
 					str = g_string_append(str, _("YES"));
 				} else {
+					str = g_string_append(str, "<span foreground=\"red\">");
 					str = g_string_append(str, _("NO"));
 				}
 				str = g_string_append(str, "</span></b>");
@@ -313,40 +339,126 @@ static void show_command_help_popup(GtkEntry *entry) {
 	g_free(helper);
 }
 
+/* handler for the single-line console */
+#if GTK_CHECK_VERSION(3, 24, 24)
+static gboolean on_command_key_press_event(GtkEventController *controller,
+		guint keyval, guint keycode, GdkModifierType modifiers,
+		GtkWidget *widget) {
+#else
+static gboolean on_command_key_press_event(GtkWidget *widget, GdkEventKey *event,
+		gpointer user_data) {
+	guint keyval = event->keyval;
+#endif
+	int handled = 0;
+	static GtkEntry *entry = NULL;
+	if (!entry)
+		entry = GTK_ENTRY(widget);
+	GtkEditable *editable = GTK_EDITABLE(entry);
+	int entrylength = 0;
+
+	switch (keyval) {
+	case GDK_KEY_Up:
+		handled = 1;
+		if (!gui.cmd_history)
+			break;
+		if (gui.cmd_hist_display > 0) {
+			if (gui.cmd_history[gui.cmd_hist_display - 1])
+				--gui.cmd_hist_display;
+			// display previous entry
+			gtk_entry_set_text(entry, gui.cmd_history[gui.cmd_hist_display]);
+		} else if (gui.cmd_history[gui.cmd_hist_size - 1]) {
+			// ring back, display previous
+			gui.cmd_hist_display = gui.cmd_hist_size - 1;
+			gtk_entry_set_text(entry, gui.cmd_history[gui.cmd_hist_display]);
+		}
+		entrylength = gtk_entry_get_text_length(entry);
+		gtk_editable_set_position(editable, entrylength);
+		break;
+	case GDK_KEY_Down:
+		handled = 1;
+		if (!gui.cmd_history)
+			break;
+		if (gui.cmd_hist_display == gui.cmd_hist_current)
+			break;
+		if (gui.cmd_hist_display == gui.cmd_hist_size - 1) {
+			if (gui.cmd_hist_current == 0) {
+				// ring forward, end
+				gtk_entry_set_text(entry, "");
+				gui.cmd_hist_display++;
+			} else if (gui.cmd_history[0]) {
+				// ring forward, display next
+				gui.cmd_hist_display = 0;
+				gtk_entry_set_text(entry, gui.cmd_history[0]);
+			}
+		} else {
+			if (gui.cmd_hist_display == gui.cmd_hist_current - 1) {
+				// end
+				gtk_entry_set_text(entry, "");
+				gui.cmd_hist_display++;
+			} else if (gui.cmd_history[gui.cmd_hist_display + 1]) {
+				// display next
+				gtk_entry_set_text(entry,
+						gui.cmd_history[++gui.cmd_hist_display]);
+			}
+		}
+		entrylength = gtk_entry_get_text_length(entry);
+		gtk_editable_set_position(editable, entrylength);
+		break;
+	case GDK_KEY_Page_Up:
+	case GDK_KEY_Page_Down:
+		handled = 1;
+		// go to first and last in history
+		break;
+	}
+	return (handled == 1);
+}
+
 int processcommand(const char *line) {
-	int wordnb = 0, len;
-	char *myline;
+	int wordnb = 0;
+	GError *error = NULL;
 
 	if (line[0] == '\0' || line[0] == '\n')
 		return 0;
 	if (line[0] == '@') { // case of files
-		if (get_thread_run()) {
+		if (get_thread_run() || get_script_thread_run()) {
 			PRINT_ANOTHER_THREAD_RUNNING;
 			return 1;
 		}
-		if (com.script_thread)
-			g_thread_join(com.script_thread);
+
+		/* Switch to console tab */
+		control_window_switch_to_tab(OUTPUT_LOGS);
+
 		char filename[256];
 		g_strlcpy(filename, line + 1, 250);
 		expand_home_in_filename(filename, 256);
-		FILE* fp = g_fopen(filename, "r");
-		if (fp == NULL) {
-			siril_log_message(_("File [%s] does not exist\n"), filename);
+
+		GFile *file = g_file_new_for_path(filename);
+		GInputStream *input_stream = (GInputStream *)g_file_read(file, NULL, &error);
+
+		if (input_stream == NULL) {
+			if (error != NULL) {
+				g_clear_error(&error);
+				siril_log_message(_("File [%s] does not exist\n"), filename);
+			}
+
+			g_object_unref(file);
 			return 1;
 		}
-		/* Switch to console tab */
-		control_window_switch_to_tab(OUTPUT_LOGS);
 		/* ensure that everything is closed */
 		process_close(0);
 		/* Then, run script */
 		siril_log_message(_("Starting script %s\n"), filename);
-		com.script_thread = g_thread_new("script", execute_script, fp);
+		com.script_thread = g_thread_new("script", execute_script, input_stream);
+		g_object_unref(file);
 	} else {
-		myline = strdup(line);
-		len = strlen(line);
-		parseLine(myline, len, &wordnb);
-		if (executeCommand(wordnb)) {
-			siril_log_message(_("Command execution failed.\n"));
+		/* Switch to console tab */
+		control_window_switch_to_tab(OUTPUT_LOGS);
+
+		gchar *myline = strdup(line);
+		int len = strlen(line);
+		parse_line(myline, len, &wordnb);
+		if (execute_command(wordnb)) {
+			siril_log_color_message(_("Command execution failed.\n"), "red");
 			if (!com.script && !com.headless) {
 				show_command_help_popup(GTK_ENTRY(lookup_widget("command")));
 			}
@@ -355,22 +467,22 @@ int processcommand(const char *line) {
 		}
 		free(myline);
 	}
-	set_cursor_waiting(FALSE);
 	return 0;
 }
 
+// loads the sequence from com.wd
 sequence *load_sequence(const char *name, char **get_filename) {
 	gchar *file = g_strdup(name);
 	gchar *altfile = NULL;
-	if (!ends_with(name, ".seq")) {
+	if (!g_str_has_suffix(name, ".seq")) {
 		str_append(&file, ".seq");
-		if (!ends_with(name, "_"))
+		if (!g_str_has_suffix(name, "_"))
 			altfile = g_strdup_printf("%s_.seq", name);
 	}
 
 	if (!is_readable_file(file) && (!altfile || !is_readable_file(altfile))) {
 		if (check_seq(FALSE)) {
-			siril_log_message(_("No sequence `%s' found.\n"), name);
+			siril_log_color_message(_("No sequence `%s' found.\n"), "red", name);
 			g_free(file);
 			g_free(altfile);
 			return NULL;
@@ -391,12 +503,14 @@ sequence *load_sequence(const char *name, char **get_filename) {
 		}
 	}
 	if (!seq)
-		siril_log_message(_("Loading sequence `%s' failed.\n"), name);
+		siril_log_color_message(_("Loading sequence `%s' failed.\n"), "red", name);
 	else {
 		if (seq_check_basic_data(seq, FALSE) == -1) {
 			free(seq);
 			seq = NULL;
 		}
+		else if (seq->type == SEQ_SER)
+			ser_display_info(seq->ser_file);
 	}
 	g_free(file);
 	g_free(altfile);
@@ -465,32 +579,49 @@ static gboolean completion_match_func(GtkEntryCompletion *completion,
 	return res;
 }
 
-void init_completion_command() {
+static void init_completion_command() {
 	GtkEntryCompletion *completion = gtk_entry_completion_new();
 	GtkListStore *model = gtk_list_store_new(1, G_TYPE_STRING);
 	GtkTreeIter iter;
 	GtkEntry *entry = GTK_ENTRY(lookup_widget("command"));
 
+	gtk_entry_completion_set_model(completion, GTK_TREE_MODEL(model));
 	gtk_entry_completion_set_text_column(completion, COMPLETION_COLUMN);
-	gtk_entry_set_completion(entry, completion);
+	gtk_entry_completion_set_minimum_key_length(completion, 2);
+	gtk_entry_completion_set_popup_completion(completion, TRUE);
 	gtk_entry_completion_set_inline_completion(completion, TRUE);
 	gtk_entry_completion_set_popup_single_match(completion, FALSE);
-	gtk_entry_completion_set_minimum_key_length(completion, 2);
 	gtk_entry_completion_set_match_func(completion, completion_match_func, NULL, NULL);
+	gtk_entry_set_completion(entry, completion);
 	g_signal_connect(G_OBJECT(completion), "match-selected", G_CALLBACK(on_match_selected), NULL);
 
 	/* Populate the completion database. */
 	command *current = commands;
 
-	while (current->process){
+	while (current->process) {
 		gtk_list_store_append(model, &iter);
 		gtk_list_store_set(model, &iter, COMPLETION_COLUMN, current->name, -1);
 		current++;
 	}
-	gtk_entry_completion_set_model(completion, GTK_TREE_MODEL(model));
 	g_object_unref(model);
 }
 
+static void init_controller_command() {
+	GtkWidget *widget = lookup_widget("command");
+
+#if GTK_CHECK_VERSION(3, 24, 24)
+	GtkEventController *controller = gtk_event_controller_key_new(widget);
+	g_signal_connect(controller, "key-pressed", G_CALLBACK(on_command_key_press_event), widget);
+#else
+	g_signal_connect(widget, "key-press-event", G_CALLBACK(on_command_key_press_event), NULL);
+#endif
+}
+
+void init_command() {
+	init_completion_command();
+	init_controller_command();
+}
+
 void on_GtkCommandHelper_clicked(GtkButton *button, gpointer user_data) {
 	show_command_help_popup((GtkEntry *)user_data);
 }
@@ -502,98 +633,29 @@ void on_GtkCommandHelper_clicked(GtkButton *button, gpointer user_data) {
  */
 
 static void history_add_line(char *line) {
-	if (!com.cmd_history) {
-		com.cmd_hist_size = CMD_HISTORY_SIZE;
-		com.cmd_history = calloc(com.cmd_hist_size, sizeof(const char*));
-		com.cmd_hist_current = 0;
-		com.cmd_hist_display = 0;
+	if (!gui.cmd_history) {
+		gui.cmd_hist_size = CMD_HISTORY_SIZE;
+		gui.cmd_history = calloc(gui.cmd_hist_size, sizeof(const char*));
+		gui.cmd_hist_current = 0;
+		gui.cmd_hist_display = 0;
 	}
-	com.cmd_history[com.cmd_hist_current] = line;
-	com.cmd_hist_current++;
+	gui.cmd_history[gui.cmd_hist_current] = line;
+	gui.cmd_hist_current++;
 	// circle at the end
-	if (com.cmd_hist_current == com.cmd_hist_size)
-		com.cmd_hist_current = 0;
-	if (com.cmd_history[com.cmd_hist_current]) {
-		free(com.cmd_history[com.cmd_hist_current]);
-		com.cmd_history[com.cmd_hist_current] = NULL;
+	if (gui.cmd_hist_current == gui.cmd_hist_size)
+		gui.cmd_hist_current = 0;
+	if (gui.cmd_history[gui.cmd_hist_current]) {
+		free(gui.cmd_history[gui.cmd_hist_current]);
+		gui.cmd_history[gui.cmd_hist_current] = NULL;
 	}
-	com.cmd_hist_display = com.cmd_hist_current;
+	gui.cmd_hist_display = gui.cmd_hist_current;
 }
 
-/* handler for the single-line console */
-gboolean on_command_key_press_event(GtkWidget *widget, GdkEventKey *event,
-		gpointer user_data) {
-	const gchar *text;
-	int handled = 0;
-	static GtkEntry *entry = NULL;
-	if (!entry)
-		entry = GTK_ENTRY(widget);
-	GtkEditable *editable = GTK_EDITABLE(entry);
-	int entrylength = 0;
-
-	switch (event->keyval) {
-	case GDK_KEY_Return:
-	case GDK_KEY_KP_Enter:
-		handled = 1;
-		text = gtk_entry_get_text(entry);
-		history_add_line(strdup(text));
-		if (!(processcommand(text))) {
-			gtk_entry_set_text(entry, "");
-			set_precision_switch();
-		}
-		break;
-	case GDK_KEY_Up:
-		handled = 1;
-		if (!com.cmd_history)
-			break;
-		if (com.cmd_hist_display > 0) {
-			if (com.cmd_history[com.cmd_hist_display - 1])
-				--com.cmd_hist_display;
-			// display previous entry
-			gtk_entry_set_text(entry, com.cmd_history[com.cmd_hist_display]);
-		} else if (com.cmd_history[com.cmd_hist_size - 1]) {
-			// ring back, display previous
-			com.cmd_hist_display = com.cmd_hist_size - 1;
-			gtk_entry_set_text(entry, com.cmd_history[com.cmd_hist_display]);
-		}
-		entrylength = gtk_entry_get_text_length(entry);
-		gtk_editable_set_position(editable, entrylength);
-		break;
-	case GDK_KEY_Down:
-		handled = 1;
-		if (!com.cmd_history)
-			break;
-		if (com.cmd_hist_display == com.cmd_hist_current)
-			break;
-		if (com.cmd_hist_display == com.cmd_hist_size - 1) {
-			if (com.cmd_hist_current == 0) {
-				// ring forward, end
-				gtk_entry_set_text(entry, "");
-				com.cmd_hist_display++;
-			} else if (com.cmd_history[0]) {
-				// ring forward, display next
-				com.cmd_hist_display = 0;
-				gtk_entry_set_text(entry, com.cmd_history[0]);
-			}
-		} else {
-			if (com.cmd_hist_display == com.cmd_hist_current - 1) {
-				// end
-				gtk_entry_set_text(entry, "");
-				com.cmd_hist_display++;
-			} else if (com.cmd_history[com.cmd_hist_display + 1]) {
-				// display next
-				gtk_entry_set_text(entry,
-						com.cmd_history[++com.cmd_hist_display]);
-			}
-		}
-		entrylength = gtk_entry_get_text_length(entry);
-		gtk_editable_set_position(editable, entrylength);
-		break;
-	case GDK_KEY_Page_Up:
-	case GDK_KEY_Page_Down:
-		handled = 1;
-		// go to first and last in history
-		break;
+void on_command_activate(GtkEntry *entry, gpointer user_data) {
+	const gchar *text = gtk_entry_get_text(entry);
+	history_add_line(strdup(text));
+	if (!(processcommand(text))) {
+		gtk_entry_set_text(entry, "");
+		set_precision_switch();
 	}
-	return (handled == 1);
 }
