@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -30,7 +30,7 @@
 #include "algos/statistics.h"
 #include "algos/fix_xtrans_af.h"
 #include "filters/cosmetic_correction.h"
-#include "gui/callbacks.h"
+#include "gui/utils.h"
 #include "gui/histogram.h"
 #include "gui/progress_and_log.h"
 #include "io/single_image.h"
@@ -58,10 +58,10 @@ static float evaluateNoiseOfCalibratedImage(fits *fit, fits *dark,
 	area.w = size;
 	area.h = size;
 
-	copyfits(dark, &dark_tmp, CP_ALLOC | CP_COPYA | CP_FORMAT, -1);
-	copyfits(fit, &fit_tmp, CP_ALLOC | CP_COPYA | CP_FORMAT, -1);
+	ret = copyfits(dark, &dark_tmp, CP_ALLOC | CP_COPYA | CP_FORMAT, -1);
+	if (!ret) ret = copyfits(fit, &fit_tmp, CP_ALLOC | CP_COPYA | CP_FORMAT, -1);
 
-	ret = soper(&dark_tmp, k, OPER_MUL, allow_32bit_output);
+	if (!ret) ret = soper(&dark_tmp, k, OPER_MUL, allow_32bit_output);
 	if (!ret) ret = imoper(&fit_tmp, &dark_tmp, OPER_SUB, allow_32bit_output);
 	if (ret) {
 		clearfits(&dark_tmp);
@@ -70,12 +70,13 @@ static float evaluateNoiseOfCalibratedImage(fits *fit, fits *dark,
 	}
 
 	for (chan = 0; chan < fit->naxes[2]; chan++) {
-		imstats *stat = statistics(NULL, -1, &fit_tmp, chan, &area, STATS_BASIC, FALSE);
+		/* STATS_SIGMEAN computes mean and normvalue */
+		imstats *stat = statistics(NULL, -1, &fit_tmp, chan, &area, STATS_SIGMEAN, FALSE);
 		if (!stat) {
 			siril_log_message(_("Error: statistics computation failed.\n"));
 			return -1.0;
 		}
-		noise += (float) stat->sigma;
+		noise += stat->sigma / stat->normValue;
 		free_stats(stat);
 	}
 	clearfits(&dark_tmp);
@@ -97,6 +98,7 @@ static float goldenSectionSearch(fits *raw, fits *dark, float a, float b,
 	d = a + GR * (b - a);
 	fc = evaluateNoiseOfCalibratedImage(raw, dark, c, allow_32bits);
 	fd = evaluateNoiseOfCalibratedImage(raw, dark, d, allow_32bits);
+	if (fc == fd) return 1.f;
 	do {
 		siril_debug_print("Iter: %d (%1.2f, %1.2f)\n", ++iter, c, d);
 		if (fc < 0.f || fd < 0.f)
@@ -123,7 +125,9 @@ static int preprocess(fits *raw, struct preprocessing_data *args) {
 	int ret = 0;
 
 	if (args->use_bias) {
-		ret = imoper(raw, args->bias, OPER_SUB, args->allow_32bit_output);
+		if (args->bias_level < FLT_MAX) { // an offset level has been defined
+			ret = soper(raw, args->bias_level, OPER_SUB, args->allow_32bit_output);
+		} else ret = imoper(raw, args->bias, OPER_SUB, args->allow_32bit_output);
 	}
 
 	/* if dark optimization, the master-dark has already been subtracted */
@@ -158,15 +162,15 @@ static int darkOptimization(fits *raw, struct preprocessing_data *args) {
 	fits dark_tmp = { 0 };
 
 	if (memcmp(raw->naxes, dark->naxes, sizeof raw->naxes)) {
-		siril_log_message(_("imoper: images must have same dimensions\n"));
+		siril_log_color_message(_("Images must have same dimensions\n"), "red");
 		return 1;
 	}
 
+	// TODO: avoid duplicating with soper+imoper together
 	if (copyfits(dark, &dark_tmp, CP_ALLOC | CP_COPYA | CP_FORMAT, 0))
 		return 1;
 
 	/* Minimization of background noise to find better k */
-	//invalidate_stats_from_fit(raw);	// why?
 	k0 = goldenSectionSearch(raw, &dark_tmp, lo, up, 0.001f, args->allow_32bit_output);
 	if (k0 < 0.f) {
 		ret = -1;
@@ -181,14 +185,93 @@ static int darkOptimization(fits *raw, struct preprocessing_data *args) {
 	return ret;
 }
 
-static int64_t prepro_compute_size_hook(struct generic_seq_args *args, int nb_images) {
+static gint64 prepro_compute_size_hook(struct generic_seq_args *args, int nb_images) {
 	struct preprocessing_data *prepro = args->user;
-	int64_t size = seq_compute_size(args->seq, nb_images, args->output_type);
+	gint64 size = seq_compute_size(args->seq, nb_images, args->output_type);
 	if (prepro->debayer)
 		size *= 3;
 	return size;
 }
 
+/* we need to account for the possible 3 master frames loaded at the start and
+ * the memory it takes to calibrate the images */
+static int prepro_compute_mem_hook(struct generic_seq_args *args, gboolean for_writer) {
+	int nb_masters = 0;
+	struct preprocessing_data *prepro = args->user;
+	if (prepro->use_flat && prepro->flat) nb_masters++;
+	if (prepro->use_dark && prepro->dark) nb_masters++;
+	if (prepro->use_bias && prepro->bias) nb_masters++;
+	int input_is_float = get_data_type(args->seq->bitpix) == DATA_FLOAT;
+	int output_is_float = input_is_float || !com.pref.force_to_16bit;
+	/*
+	 * are the masters float? yes if !com.pref.force_to_16bit, raw images too
+	 *
+	 * if (prepro->use_dark_optim && prepro->use_dark)
+	 *	image gets three copies
+	 *
+	 * if (prepro->debayer)
+	 * 	O(9n) for ushort
+	 * 	O(3n) for float, same as dark optim
+	 */
+	unsigned int MB_per_input_image, MB_avail;
+	int limit = compute_nb_images_fit_memory(args->seq, 1.0, FALSE, &MB_per_input_image, NULL, &MB_avail);
+	if (!input_is_float && output_is_float)
+		MB_per_input_image *= 2;
+	MB_avail -= nb_masters * MB_per_input_image;
+
+	unsigned int required = MB_per_input_image;
+	unsigned int MB_per_output_image = MB_per_input_image;
+	if (prepro->debayer) {
+		if (input_is_float || output_is_float)
+			MB_per_output_image *= 3;
+		else MB_per_output_image *= 9;
+		required = MB_per_input_image + MB_per_output_image;
+	}
+	else if (prepro->use_dark_optim && prepro->use_dark) {
+		required = 4 * MB_per_input_image;
+	}
+
+	if (limit > 0) {
+		int thread_limit = MB_avail / required;
+		if (thread_limit > com.max_thread)
+                        thread_limit = com.max_thread;
+
+		if (for_writer) {
+                        /* we allow the already allocated thread_limit images,
+                         * plus how many images can be stored in what remains
+                         * unused by the main processing */
+                        limit = thread_limit + (MB_avail - required * thread_limit) / MB_per_output_image;
+			siril_debug_print("%d MB avail for writer\n", MB_avail - required * thread_limit);
+                } else limit = thread_limit;
+	}
+	if (limit == 0) {
+		gchar *mem_per_thread = g_format_size_full(required * BYTES_IN_A_MB, G_FORMAT_SIZE_IEC_UNITS);
+		gchar *mem_available = g_format_size_full(MB_avail * BYTES_IN_A_MB, G_FORMAT_SIZE_IEC_UNITS);
+
+		siril_log_color_message(_("%s: not enough memory to do this operation (%s required per image, %s considered available)\n"),
+				"red", args->description, mem_per_thread, mem_available);
+
+		g_free(mem_per_thread);
+		g_free(mem_available);
+	} else {
+#ifdef _OPENMP
+		if (for_writer) {
+			int max_queue_size = com.max_thread * 3;
+			if (limit > max_queue_size)
+				limit = max_queue_size;
+		}
+		siril_debug_print("Memory required per thread: %u MB, per input image: %u MB, per output image %u MB, limiting to %d %s\n",
+				required, MB_per_input_image, MB_per_output_image, limit, for_writer ? "images" : "threads");
+#else
+		if (!for_writer)
+			limit = 1;
+		else if (limit > 3)
+			limit = 3;
+#endif
+	}
+	return limit;
+}
+
 static int prepro_prepare_hook(struct generic_seq_args *args) {
 	struct preprocessing_data *prepro = args->user;
 
@@ -223,6 +306,19 @@ static int prepro_prepare_hook(struct generic_seq_args *args) {
 			prepro->normalisation = stat->mean;
 			free_stats(stat);
 
+			// special cases for division factor, caused by how imoper works with many cases
+			if (DATA_USHORT == prepro->flat->type && prepro->allow_32bit_output)
+				prepro->normalisation *= INV_USHRT_MAX_SINGLE;
+			else if (DATA_FLOAT == prepro->flat->type && !prepro->allow_32bit_output) {
+				if (prepro->seq) { // sequence case
+					printf("normalizing for seq\n");
+					prepro->normalisation *= (prepro->seq->bitpix == BYTE_IMG) ? UCHAR_MAX_SINGLE : USHRT_MAX_SINGLE; // imoper deals with bitdepth through norm value
+				} else { // single-image case
+					printf("normalizing for single image\n");
+					prepro->normalisation *= (gfit.orig_bitpix == BYTE_IMG) ? UCHAR_MAX_SINGLE : USHRT_MAX_SINGLE; // imoper deals with bitdepth through norm value
+				}
+			}
+
 			siril_log_message(_("Normalisation value auto evaluated: %.3f\n"),
 					prepro->normalisation);
 		}
@@ -233,7 +329,7 @@ static int prepro_prepare_hook(struct generic_seq_args *args) {
 		fix_xtrans_ac(prepro->dark);
 	}
 
-	if (prepro->fix_xtrans && prepro->use_bias) {
+	if (prepro->fix_xtrans && prepro->use_bias && prepro->bias) { // check for existence of bias in case of synth offset
 		fix_xtrans_ac(prepro->bias);
 	}
 
@@ -282,8 +378,8 @@ static int prepro_image_hook(struct generic_seq_args *args, int out_index, int i
 	}
 
 	if (prepro->debayer) {
-		// not for SER because it is done on-the-fly
-		if (!prepro->seq || prepro->seq->type == SEQ_REGULAR || prepro->seq->type == SEQ_FITSEQ) {
+		// debayering SER is now allowed - https://gitlab.com/free-astro/siril/-/issues/549	
+		if (!prepro->seq || prepro->seq->type == SEQ_REGULAR || prepro->seq->type == SEQ_FITSEQ || prepro->seq->type == SEQ_SER ) {
 			debayer_if_needed(TYPEFITS, fit, TRUE);
 		}
 
@@ -334,34 +430,24 @@ gpointer prepro_worker(gpointer p) {
 }
 
 void start_sequence_preprocessing(struct preprocessing_data *prepro) {
-	struct generic_seq_args *args = malloc(sizeof(struct generic_seq_args));
-	args->seq = prepro->seq;
-	args->force_float = !com.pref.force_to_16bit;
-	args->partial_image = FALSE;
-	args->filtering_criterion = seq_filter_all;
-	args->nb_filtered_images = prepro->seq->number;
+	struct generic_seq_args *args = create_default_seqargs(prepro->seq);
+	args->force_float = !com.pref.force_to_16bit && prepro->output_seqtype != SEQ_SER;
 	args->compute_size_hook = prepro_compute_size_hook;
+	args->compute_mem_limits_hook = prepro_compute_mem_hook;
 	args->prepare_hook = prepro_prepare_hook;
 	args->image_hook = prepro_image_hook;
-	args->save_hook = NULL;
 	args->finalize_hook = prepro_finalize_hook;
-	args->idle_function = NULL;
-	args->stop_on_error = TRUE;
 	args->description = _("Preprocessing");
 	args->has_output = TRUE;
+	args->new_seq_prefix = prepro->ppprefix;
+	args->load_new_sequence = TRUE;
+	args->force_ser_output = prepro->seq->type != SEQ_SER && prepro->output_seqtype == SEQ_SER;
+	args->force_fitseq_output = prepro->seq->type != SEQ_FITSEQ && prepro->output_seqtype == SEQ_FITSEQ;
+	args->user = prepro;
 	// float output is always used in case of FITS sequence
 	args->output_type = (args->force_ser_output || com.pref.force_to_16bit ||
 			(args->seq->type == SEQ_SER && !args->force_fitseq_output)) ?
 		DATA_USHORT : DATA_FLOAT;
-	args->upscale_ratio = 1.0;
-	args->new_seq_prefix = prepro->ppprefix;
-	args->load_new_sequence = TRUE;
-	args->force_ser_output = prepro->seq->type != SEQ_SER && prepro->output_seqtype == SEQ_SER;
-	args->new_ser = NULL;
-	args->force_fitseq_output = prepro->seq->type != SEQ_FITSEQ && prepro->output_seqtype == SEQ_FITSEQ;
-	args->new_fitseq = NULL;
-	args->parallel = TRUE;
-	args->user = prepro;
 
 	remove_prefixed_sequence_files(prepro->seq, prepro->ppprefix);
 
@@ -376,7 +462,7 @@ void start_sequence_preprocessing(struct preprocessing_data *prepro) {
 
 /********** SINGLE IMAGE ************/
 int preprocess_single_image(struct preprocessing_data *args) {
-	gchar *dest_filename, *msg;
+	gchar *msg;
 	fits fit = { 0 };
 	int ret = 0;
 
@@ -397,7 +483,7 @@ int preprocess_single_image(struct preprocessing_data *args) {
 		gchar *filename = g_path_get_basename(com.uniq->filename);
 		char *filename_noext = remove_ext_from_filename(filename);
 		g_free(filename);
-		dest_filename = g_strdup_printf("%s%s%s", args->ppprefix, filename_noext, com.pref.ext);
+		gchar *dest_filename = g_strdup_printf("%s%s%s", args->ppprefix, filename_noext, com.pref.ext);
 		msg = g_strdup_printf(_("Saving image %s"), filename_noext);
 		set_progress_bar_data(msg, PROGRESS_NONE);
 		ret = savefits(dest_filename, &fit);
@@ -407,7 +493,6 @@ int preprocess_single_image(struct preprocessing_data *args) {
 			copyfits(&fit, com.uniq->fit, CP_ALLOC | CP_FORMAT | CP_COPYA, 0);
 			if (com.uniq->nb_layers != fit.naxes[2]) {
 				com.uniq->nb_layers = fit.naxes[2];
-				com.uniq->layers = realloc(com.uniq->layers, com.uniq->nb_layers * sizeof(layer_info));
 			}
 			if (com.uniq->filename)
 				free(com.uniq->filename);
@@ -423,9 +508,47 @@ int preprocess_single_image(struct preprocessing_data *args) {
 	return ret;
 }
 
-static void test_for_master_files(struct preprocessing_data *args) {
+int evaluateoffsetlevel(const char* expression, fits *fit) {
+	// try to find an occurence of *
+	// If none -> the level is just an integer to evaluate
+	// If found -> Try to find $ sign to read the offset value and its multiplier
+
+	gchar *expressioncpy = g_strdup(expression);
+	gchar *end = NULL;
+	remove_spaces_from_str(expressioncpy);
+	gchar *mulsignpos = g_strrstr(expressioncpy, "*");
+	int offsetlevel, multiplier;
+	if (!mulsignpos) {
+		offsetlevel = g_ascii_strtoull(expressioncpy, NULL, 10);
+		if (!offsetlevel) goto free_on_error;
+		if (expressioncpy) g_free(expressioncpy);
+		return offsetlevel;
+	}
+	mulsignpos[0] = '\0';
+	mulsignpos += 1;
+	if (!((expressioncpy[0] == '$') || (mulsignpos[0] == '$'))) goto free_on_error; //found a * char but none of the words start with a $
+	if (expressioncpy[0] == '$') {
+		multiplier = g_ascii_strtoull(mulsignpos, &end, 10);
+		if (!g_str_equal(expressioncpy,"$OFFSET")) goto free_on_error;
+	} else {
+		multiplier = g_ascii_strtoull(expressioncpy, &end, 10);
+		if (!g_str_equal(mulsignpos,"$OFFSET")) goto free_on_error;
+	}
+	if (!multiplier) goto free_on_error; // multiplier not parsed
+	if (!(end[0] == '\0')) goto free_on_error; // some characters were found after the multiplier
+	offsetlevel = (int)(multiplier * fit->key_offset);
+	if (expressioncpy) g_free(expressioncpy);
+	return offsetlevel;
+free_on_error:
+	if (expressioncpy) g_free(expressioncpy);
+	return 0;
+}
+
+static gboolean test_for_master_files(struct preprocessing_data *args) {
 	GtkToggleButton *tbutton;
 	GtkEntry *entry;
+	gboolean has_error = FALSE;
+	args->bias_level = FLT_MAX;
 
 	tbutton = GTK_TOGGLE_BUTTON(lookup_widget("useoffset_button"));
 	if (gtk_toggle_button_get_active(tbutton)) {
@@ -436,25 +559,49 @@ static void test_for_master_files(struct preprocessing_data *args) {
 			gtk_toggle_button_set_active(tbutton, FALSE);
 		} else {
 			const char *error = NULL;
-			set_progress_bar_data(_("Opening offset image..."), PROGRESS_NONE);
-			args->bias = calloc(1, sizeof(fits));
-			if (!readfits(filename, args->bias, NULL, !com.pref.force_to_16bit)) {
-				if (args->bias->naxes[2] != gfit.naxes[2]) {
-					error = _("NOT USING OFFSET: number of channels is different");
-				} else if (args->bias->naxes[0] != gfit.naxes[0] ||
-						args->bias->naxes[1] != gfit.naxes[1]) {
-					error = _("NOT USING OFFSET: image dimensions are different");
+			if (filename[0] == '=') { // offset is specified as a level not a file
+				set_progress_bar_data(_("Checking offset level..."), PROGRESS_NONE);
+				int offsetlevel = evaluateoffsetlevel(filename + 1, &gfit);
+				if (!offsetlevel) {
+					error = _("NOT USING OFFSET: the offset value could not be parsed");
+					args->use_bias = FALSE;
 				} else {
-					args->use_bias = TRUE;
+					siril_log_message(_("Synthetic offset: Level = %d\n"),offsetlevel);
+					int maxlevel = (gfit.orig_bitpix == BYTE_IMG) ? UCHAR_MAX : USHRT_MAX;
+					if ((offsetlevel > maxlevel) || (offsetlevel < -maxlevel) ) {   // not excluding all neg values here to allow defining a pedestal
+						error = _("NOT USING OFFSET: the offset value is not consistent with image bitdepth");
+						args->use_bias = FALSE;
+					} else {
+						args->bias_level = (float)offsetlevel;
+						args->bias_level *= (gfit.orig_bitpix == BYTE_IMG) ? INV_UCHAR_MAX_SINGLE : INV_USHRT_MAX_SINGLE; //converting to [0 1] to use with soper
+						args->use_bias = TRUE;
+					}
 				}
-
-			} else error = _("NOT USING OFFSET: cannot open the file\n");
+			} else {
+				args->bias = calloc(1, sizeof(fits));
+				set_progress_bar_data(_("Opening offset image..."), PROGRESS_NONE);
+				if (!readfits(filename, args->bias, NULL, !com.pref.force_to_16bit)) {
+					if (args->bias->naxes[2] != gfit.naxes[2]) {
+						error = _("NOT USING OFFSET: number of channels is different");
+					} else if (args->bias->naxes[0] != gfit.naxes[0] ||
+							args->bias->naxes[1] != gfit.naxes[1]) {
+						error = _("NOT USING OFFSET: image dimensions are different");
+					} else {
+						args->use_bias = TRUE;
+						// if input is 8b, we assume 32b master needs to be rescaled
+						if ((args->bias->type == DATA_FLOAT) && (gfit.orig_bitpix == BYTE_IMG)) {
+							soper(args->bias, USHRT_MAX_SINGLE / UCHAR_MAX_SINGLE, OPER_MUL, TRUE);
+						}
+					}
+				} else error = _("NOT USING OFFSET: cannot open the file");
+			}
 			if (error) {
-				siril_log_message("%s\n", error);
+				siril_log_color_message("%s\n", "red", error);
 				set_progress_bar_data(error, PROGRESS_DONE);
 				free(args->bias);
 				gtk_entry_set_text(entry, "");
 				args->use_bias = FALSE;
+				has_error = TRUE;
 			}
 		}
 	}
@@ -478,15 +625,20 @@ static void test_for_master_files(struct preprocessing_data *args) {
 					error = _("NOT USING DARK: image dimensions are different");
 				} else {
 					args->use_dark = TRUE;
+					// if input is 8b, we assume 32b master needs to be rescaled
+					if ((args->dark->type == DATA_FLOAT) && (gfit.orig_bitpix == BYTE_IMG)) {
+						soper(args->dark, USHRT_MAX_SINGLE / UCHAR_MAX_SINGLE, OPER_MUL, TRUE);
+					}
 				}
 
-			} else error = _("NOT USING DARK: cannot open the file\n");
+			} else error = _("NOT USING DARK: cannot open the file");
 			if (error) {
-				siril_log_message("%s\n", error);
+				siril_log_color_message("%s\n", "red", error);
 				set_progress_bar_data(error, PROGRESS_DONE);
 				free(args->dark);
 				gtk_entry_set_text(entry, "");
 				args->use_dark = FALSE;
+				has_error = TRUE;
 			}
 		}
 
@@ -494,6 +646,18 @@ static void test_for_master_files(struct preprocessing_data *args) {
 			// dark optimization
 			tbutton = GTK_TOGGLE_BUTTON(lookup_widget("checkDarkOptimize"));
 			args->use_dark_optim = gtk_toggle_button_get_active(tbutton);
+			const char *error = NULL;
+			if ((gfit.orig_bitpix == BYTE_IMG) && args->use_dark_optim) {
+				error = _("Dark optimization: This process cannot be applied to 8b images");
+			}
+			if (error) {
+				siril_log_color_message("%s\n", "red", error);
+				set_progress_bar_data(error, PROGRESS_DONE);
+				free(args->dark);
+				gtk_entry_set_text(entry, "");
+				args->use_dark_optim = FALSE;
+				has_error = TRUE;
+			}
 
 			// cosmetic correction
 			tbutton = GTK_TOGGLE_BUTTON(lookup_widget("cosmEnabledCheck"));
@@ -534,15 +698,17 @@ static void test_for_master_files(struct preprocessing_data *args) {
 					error = _("NOT USING FLAT: image dimensions are different");
 				} else {
 					args->use_flat = TRUE;
+					// no need to deal with bitdepth conversion as flat is just a division (unlike darks which need to be on same scale)
 				}
 
-			} else error = _("NOT USING FLAT: cannot open the file\n");
+			} else error = _("NOT USING FLAT: cannot open the file");
 			if (error) {
-				siril_log_message("%s\n", error);
+				siril_log_color_message("%s\n", "red", error);
 				set_progress_bar_data(error, PROGRESS_DONE);
 				free(args->flat);
 				gtk_entry_set_text(entry, "");
 				args->use_flat = FALSE;
+				has_error = TRUE;
 			}
 
 			if (args->use_flat) {
@@ -550,11 +716,12 @@ static void test_for_master_files(struct preprocessing_data *args) {
 				args->autolevel = gtk_toggle_button_get_active(autobutton);
 				if (!args->autolevel) {
 					GtkEntry *norm_entry = GTK_ENTRY(lookup_widget("entry_flat_norm"));
-					args->normalisation = atof(gtk_entry_get_text(norm_entry));
+					args->normalisation = g_ascii_strtod(gtk_entry_get_text(norm_entry), NULL);
 				}
 			}
 		}
 	}
+	return has_error;
 }
 
 void on_prepro_button_clicked(GtkButton *button, gpointer user_data) {
@@ -573,7 +740,12 @@ void on_prepro_button_clicked(GtkButton *button, gpointer user_data) {
 	GtkComboBox *output_type = GTK_COMBO_BOX(lookup_widget("prepro_output_type_combo"));
 
 	struct preprocessing_data *args = calloc(1, sizeof(struct preprocessing_data));
-	test_for_master_files(args);	// sets most properties
+	if (test_for_master_files(args)) {
+		siril_log_color_message(_("Some errors have been detected, Please check the logs.\n"), "red");
+		return;
+	}
+	if (!args->use_bias && !args->use_dark && !args->use_flat)
+		return;
 
 	siril_log_color_message(_("Preprocessing...\n"), "green");
 	gettimeofday(&args->t_start, NULL);
