@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -27,7 +27,7 @@
 #include "core/processing.h"
 #include "core/sequence_filtering.h"
 #include "core/OS_utils.h"
-#include "gui/callbacks.h"
+#include "gui/utils.h"
 #include "gui/progress_and_log.h"
 #include "io/sequence.h"
 #include "io/ser.h"
@@ -45,9 +45,8 @@ gpointer generic_sequence_worker(gpointer p) {
 	int input_idx;	// index of the frame being processed in the sequence
 	int *index_mapping = NULL;
 	int nb_frames, excluded_frames = 0, progress = 0;
-	float nb_framesf;
 	int abort = 0;	// variable for breaking out of loop
-	GString *desc;	// temporary string description for logs
+	gboolean have_seqwriter = FALSE;
 
 	assert(args);
 	assert(args->seq);
@@ -55,7 +54,7 @@ gpointer generic_sequence_worker(gpointer p) {
 	set_progress_bar_data(NULL, PROGRESS_RESET);
 	gettimeofday(&t_start, NULL);
 
-	if (args->nb_filtered_images > 0)	// XXX can it be zero?
+	if (args->nb_filtered_images > 0)
 		nb_frames = args->nb_filtered_images;
 	else {
 		nb_frames = compute_nb_filtered_images(args->seq, args->filtering_criterion, args->filtering_parameter);
@@ -66,19 +65,29 @@ gpointer generic_sequence_worker(gpointer p) {
 			goto the_end;
 		}
 	}
-	nb_framesf = (float)nb_frames + 0.3f;	// leave margin for rounding errors and post processing
+	float nb_framesf = (float)nb_frames + 0.3f;	// leave margin for rounding errors and post processing
 	args->retval = 0;
 
+#ifdef _OPENMP
+	if (args->max_thread < 1) {
+		if (args->compute_mem_limits_hook)
+			args->max_thread = args->compute_mem_limits_hook(args, FALSE);
+		else args->max_thread = seq_compute_mem_limits(args, FALSE);
+	}
+	if (args->max_thread < 1) {
+		args->retval = 1;
+		goto the_end;
+	}
+	siril_log_message(_("%s: with the current memory and thread limits, up to %d thread(s) can be used\n"),
+			args->description, args->max_thread);
+#endif
+
 	if (args->prepare_hook && args->prepare_hook(args)) {
 		siril_log_message(_("Preparing sequence processing failed.\n"));
 		args->retval = 1;
 		goto the_end;
 	}
 
-	if (args->seq->type == SEQ_FITSEQ) {
-		fitseq_prepare_for_multiple_read(args->seq->fitseq_file);
-	}
-
 	/* We have a sequence in which images can be filtered out. In order to
 	 * distribute the workload fairly among all threads, the main iteration
 	 * should not be on the list of images of the sequence, but on the
@@ -108,7 +117,7 @@ gpointer generic_sequence_worker(gpointer p) {
 	}
 
 	if (args->has_output && !args->partial_image) {
-		int64_t size;
+		gint64 size;
 		if (args->compute_size_hook)
 			size = args->compute_size_hook(args, nb_frames);
 		else size = seq_compute_size(args->seq, nb_frames, args->output_type);
@@ -119,27 +128,26 @@ gpointer generic_sequence_worker(gpointer p) {
 	}
 
 	/* Output print of algorithm description */
-	desc = g_string_new(args->description);
-	if (desc) {
-		desc = g_string_append(desc, _(": processing...\n"));
-		siril_log_color_message(desc->str, "green");
-		g_string_free(desc, TRUE);
+	if (args->description) {
+		gchar *desc = g_strdup_printf(_("%s: processing...\n"), args->description);
+		siril_log_color_message(desc, "green");
+		g_free(desc);
 	}
 
-	gboolean have_seqwriter = args->has_output &&
+	have_seqwriter = args->has_output &&
 		((args->force_fitseq_output || args->seq->type == SEQ_FITSEQ) ||
 		 (args->force_ser_output || args->seq->type == SEQ_SER));
 #ifdef _OPENMP
 	omp_init_lock(&args->lock);
 	if (have_seqwriter)
 		omp_set_schedule(omp_sched_dynamic, 1);
-	else omp_set_schedule(omp_sched_static, 0);
+	else omp_set_schedule(omp_sched_guided, 0);
 #ifdef HAVE_FFMS2
 	// we don't want to enable parallel processing for films, as ffms2 is not thread-safe
-#pragma omp parallel for num_threads(com.max_thread) private(input_idx) schedule(runtime) \
+#pragma omp parallel for num_threads(args->max_thread) private(input_idx) schedule(runtime) \
 	if(args->seq->type != SEQ_AVI && args->parallel && (args->seq->type == SEQ_SER || fits_is_reentrant()))
 #else
-#pragma omp parallel for num_threads(com.max_thread) private(input_idx) schedule(runtime) \
+#pragma omp parallel for num_threads(args->max_thread) private(input_idx) schedule(runtime) \
 	if(args->parallel && (args->seq->type == SEQ_SER || fits_is_reentrant()))
 #endif // HAVE_FFMS2
 #endif // _OPENMP
@@ -169,6 +177,10 @@ gpointer generic_sequence_worker(gpointer p) {
 		thread_id = omp_get_thread_num();
 		if (have_seqwriter) {
 			seqwriter_wait_for_memory();
+			if (abort) {
+				seqwriter_release_memory();
+				continue;
+			}
 		}
 #endif
 
@@ -211,16 +223,10 @@ gpointer generic_sequence_worker(gpointer p) {
 			if (args->stop_on_error)
 				abort = 1;
 			else {
-				//args->seq->imgparam[frame].incl = FALSE;
 #ifdef _OPENMP
-#pragma omp critical
+#pragma omp atomic
 #endif
-				{
-					//if (args->nb_filtered_images > 0)
-					//	args->nb_filtered_images--;
-					//args->seq->selnum--;
-					excluded_frames++;
-				}
+				excluded_frames++;
 			}
 			clearfits(fit);
 			free(fit);
@@ -269,6 +275,12 @@ gpointer generic_sequence_worker(gpointer p) {
 		g_free(msg);
 	}
 
+	/* the finalize hook contains the sequence writer synchronization, it
+	 * should be called before outputing the logs */
+	if (have_seqwriter && args->finalize_hook && args->finalize_hook(args)) {
+		siril_log_message(_("Finalizing sequence processing failed.\n"));
+		abort = 1;
+	}
 	if (abort) {
 		set_progress_bar_data(_("Sequence processing failed. Check the log."), PROGRESS_RESET);
 		siril_log_color_message(_("Sequence processing failed.\n"), "red");
@@ -291,17 +303,11 @@ gpointer generic_sequence_worker(gpointer p) {
 #endif
 the_end:
 	if (index_mapping) free(index_mapping);
-	if (args->finalize_hook && args->finalize_hook(args)) {
+	if (!have_seqwriter && args->finalize_hook && args->finalize_hook(args)) {
 		siril_log_message(_("Finalizing sequence processing failed.\n"));
 		args->retval = 1;
 	}
 
-	if (args->seq->type == SEQ_FITSEQ) {
-		int ret = fitseq_multiple_close(args->seq->fitseq_file);
-		if (!args->retval)
-			args->retval = ret;
-	}
-
 	if (args->already_in_a_thread) {
 		if (args->idle_function)
 			args->idle_function(args);
@@ -331,10 +337,56 @@ gboolean end_generic_sequence(gpointer p) {
 	return end_generic(NULL);
 }
 
+/* If for_writer is false, it computes how many images can be processed in
+ * parallel, with regard to how many of them can fit in memory. It returns at
+ * most com.max_thread.
+ * If for_writer is true, it computes how many images can be stored in the
+ * queue. It returns at most 3 times com.max_thread.
+ */
+int seq_compute_mem_limits(struct generic_seq_args *args, gboolean for_writer) {
+	unsigned int MB_per_image, MB_avail;
+	int thread_limit = compute_nb_images_fit_memory(args->seq, args->upscale_ratio, args->force_float, NULL, &MB_per_image, &MB_avail);
+	int limit = thread_limit;
+	if (limit == 0) {
+		gchar *mem_per_image = g_format_size_full(MB_per_image * BYTES_IN_A_MB, G_FORMAT_SIZE_IEC_UNITS);
+		gchar *mem_available = g_format_size_full(MB_avail * BYTES_IN_A_MB, G_FORMAT_SIZE_IEC_UNITS);
+
+		siril_log_color_message(_("%s: not enough memory to do this operation (%s required per image, %s considered available)\n"),
+				"red", args->description, mem_per_image, mem_available);
+
+		g_free(mem_per_image);
+		g_free(mem_available);
+	} else {
+#ifdef _OPENMP
+		/* number of threads for the main computation */
+		if (limit > com.max_thread)
+			limit = com.max_thread;
+		if (for_writer) {
+			/* we take the remainder for the writer */
+			int max_queue_size = com.max_thread * 3;
+			if (thread_limit - limit > max_queue_size)
+				limit = max_queue_size;
+			else limit = thread_limit - limit;
+			if (limit < 1)
+				limit = 1;	// bug #777
+		}
+#else
+		if (!for_writer)
+			limit = 1;
+		else {
+			limit = max(1, limit-1);
+			if (limit > 3)
+				limit = 3;
+		}
+#endif
+	}
+	return limit;
+}
+
 int seq_prepare_hook(struct generic_seq_args *args) {
 	int retval = 0;
 	g_assert(args->has_output); // don't call this hook otherwise
-	if (args->force_ser_output || args->seq->type == SEQ_SER) {
+	if (args->force_ser_output || (args->seq->type == SEQ_SER && !args->force_fitseq_output)) {
 		gchar *dest;
 		const char *ptr = strrchr(args->seq->seqname, G_DIR_SEPARATOR);
 		if (ptr)
@@ -349,7 +401,7 @@ int seq_prepare_hook(struct generic_seq_args *args) {
 		}
 		g_free(dest);
 	}
-	else if (args->force_fitseq_output || args->seq->type == SEQ_FITSEQ) {
+	else if (args->force_fitseq_output || (args->seq->type == SEQ_FITSEQ && !args->force_ser_output)) {
 		gchar *dest;
 		const char *ptr = strrchr(args->seq->seqname, G_DIR_SEPARATOR);
 		if (ptr)
@@ -366,43 +418,42 @@ int seq_prepare_hook(struct generic_seq_args *args) {
 	}
 	else return 0;
 
+	if (!retval)
+		retval = seq_prepare_writer(args);
+	return retval;
+}
+
+int seq_prepare_writer(struct generic_seq_args *args) {
 	int limit = 0;
-#ifdef _OPENMP
-	limit = compute_nb_images_fit_memory(args->seq, args->upscale_ratio, args->force_float, NULL, NULL);
-	if (limit < 0)
-		limit = com.max_thread * 3; // we still don't want an impossible build-up of images to write
-	else if (limit == 0) {
-		siril_log_color_message(_("Configured memory limits do not seem to allow these images to be processed, aborting\n"), "red");
-		if (args->force_ser_output || args->seq->type == SEQ_SER) {
+	if (args->compute_mem_limits_hook)
+		limit = args->compute_mem_limits_hook(args, TRUE);
+	else limit = seq_compute_mem_limits(args, TRUE); // the default
+
+	if (limit == 0) {
+		if (args->force_ser_output || (args->seq->type == SEQ_SER && !args->force_fitseq_output)) {
 			ser_close_file(args->new_ser);
 			free(args->new_ser);
 			args->new_ser = NULL;
 		}
-		else if (args->force_fitseq_output || args->seq->type == SEQ_FITSEQ) {
+		else if (args->force_fitseq_output || (args->seq->type == SEQ_FITSEQ && !args->force_ser_output)) {
 			fitseq_close_file(args->new_fitseq);
 			free(args->new_fitseq);
 			args->new_fitseq = NULL;
 		}
 		return 1;
-	} else {
-		// there doesn't seem to be any interest in having a larger queue
-		int max_queue_size = com.max_thread * 3;
-		if (limit > max_queue_size)
-			limit = max_queue_size;
 	}
-#endif
 	seqwriter_set_max_active_blocks(limit);
-	return retval;
+	return 0;
 }
 
 int seq_finalize_hook(struct generic_seq_args *args) {
 	int retval = 0;
 	g_assert(args->has_output); // don't call this hook otherwise
-	if ((args->force_ser_output || args->seq->type == SEQ_SER) && args->new_ser) {
+	if ((args->force_ser_output || (args->seq->type == SEQ_SER && !args->force_fitseq_output)) && args->new_ser) {
 		retval = ser_write_and_close(args->new_ser);
 		free(args->new_ser);
 	}
-	else if ((args->force_fitseq_output || args->seq->type == SEQ_FITSEQ) && args->new_fitseq) {
+	else if ((args->force_fitseq_output || (args->seq->type == SEQ_FITSEQ && !args->force_ser_output)) && args->new_fitseq) {
 		retval = fitseq_close_file(args->new_fitseq);
 		free(args->new_fitseq);
 	}
@@ -412,11 +463,13 @@ int seq_finalize_hook(struct generic_seq_args *args) {
 /* In SER, all images must be in a contiguous sequence, so we use the out_index.
  * In FITS sequences, to keep track of image accross processings, we keep the
  * input file number all along (in_index is the index in the sequence, not the name).
+ * The 2nd condition ensures that any force condition prevails over opposite
+ * input-type condition
  */
 int generic_save(struct generic_seq_args *args, int out_index, int in_index, fits *fit) {
-	if (args->force_ser_output || args->seq->type == SEQ_SER) {
+	if (args->force_ser_output || (args->seq->type == SEQ_SER && !args->force_fitseq_output)) {
 		return ser_write_frame_from_fit(args->new_ser, fit, out_index);
-	} else if (args->force_fitseq_output || args->seq->type == SEQ_FITSEQ) {
+	} else if (args->force_fitseq_output || (args->seq->type == SEQ_FITSEQ && !args->force_ser_output)) {
 		return fitseq_write_image(args->new_fitseq, fit, out_index);
 	} else {
 		char *dest = fit_sequence_get_image_filename_prefixed(args->seq,
@@ -447,6 +500,7 @@ void start_in_new_thread(gpointer (*f)(gpointer), gpointer p) {
 
 	com.run_thread = TRUE;
 	g_mutex_unlock(&com.mutex);
+	set_cursor_waiting(TRUE);
 	com.thread = g_thread_new("processing", f, p);
 }
 
@@ -461,6 +515,7 @@ void start_in_reserved_thread(gpointer (*f)(gpointer), gpointer p) {
 
 	com.run_thread = TRUE;
 	g_mutex_unlock(&com.mutex);
+	set_cursor_waiting(TRUE);
 	com.thread = g_thread_new("processing", f, p);
 }
 
@@ -534,6 +589,12 @@ guint siril_add_idle(GSourceFunc idle_function, gpointer data) {
 	return 0;
 }
 
+gboolean get_script_thread_run() {
+	/* we don't need mutex for this one because it's only checked by user
+	 * input, which is much slower than the function execution */
+	return com.script_thread != NULL;
+}
+
 void wait_for_script_thread() {
 	if (com.script_thread)
 		g_thread_join(com.script_thread);
@@ -547,3 +608,14 @@ void on_processes_button_cancel_clicked(GtkButton *button, gpointer user_data) {
 	stop_processing_thread();
 	wait_for_script_thread();
 }
+
+struct generic_seq_args *create_default_seqargs(sequence *seq) {
+	struct generic_seq_args *args = calloc(1, sizeof(struct generic_seq_args));
+	args->seq = seq;
+	args->filtering_criterion = seq_filter_all;
+	args->nb_filtered_images = seq->number;
+	args->stop_on_error = TRUE;
+	args->upscale_ratio = 1.0;
+	args->parallel = TRUE;
+	return args;
+}
