@@ -43,7 +43,7 @@ int readpic(const char *name, fits *fit);
 /****************** image_formats_libraries.h ******************/
 #ifdef HAVE_LIBTIFF
 int readtif(const char *name, fits *fit, gboolean force_float);
-int savetif(const char *name, fits *fit, uint16 bitspersample);
+int savetif(const char *name, fits *fit, uint16_t bitspersample);
 #endif
 
 #ifdef HAVE_LIBJPEG
@@ -72,9 +72,11 @@ WORD round_to_WORD(double x);
 BYTE round_to_BYTE(double x);
 BYTE roundf_to_BYTE(float f);
 WORD roundf_to_WORD(float f);
+int round_to_ceiling_multiple(int x, int factor);
 BYTE conv_to_BYTE(double x);
 int truncate_to_int32(uint64_t x);
 WORD truncate_to_WORD(int x);
+BYTE truncate_to_BYTE(WORD x);
 int set_int_in_interval(int val, int low, int high);
 float set_float_in_interval(float val, float low, float high);
 double set_double_in_interval(double val, double low, double high);
@@ -88,6 +90,7 @@ WORD *float_buffer_to_ushort(float *buffer, size_t ndata);
 float *uchar_buffer_to_float(BYTE *buffer, size_t ndata);
 float *ushort_buffer_to_float(WORD *buffer, size_t ndata);
 float *ushort8_buffer_to_float(WORD *buffer, size_t ndata);
+gboolean test_double_eq(double a, double b, double epsilon);
 uint16_t change_endianness16(uint16_t x);
 uint16_t cpu_to_le16(uint16_t x);
 uint16_t cpu_to_be16(uint16_t x);
@@ -104,30 +107,31 @@ uint64_t cpu_to_be64(uint64_t x);
 uint64_t le64_to_cpu(uint64_t x);
 uint64_t be64_to_cpu(uint64_t x);
 gboolean isrgb(fits *fit);
-gboolean ends_with(const char *str, const char *ending);
 int get_extension_index(const char *filename);
 image_type get_type_from_filename(const gchar *filename);
+char* remove_ext_from_filename(const char *basename);
+gchar *replace_ext(const char *path, const char *new_ext);
+gboolean string_is_a_path(const char *file);
 int is_readable_file(const char *filename);
 gboolean is_forbiden_in_filename(gchar c);
 gboolean file_name_has_invalid_chars(const char *name);
 int stat_file(const char *filename2, image_type *type, char **realname);
 const char* get_filename_ext(const char *filename);
 
-int changedir(const char *dir, gchar **err);
+int siril_change_dir(const char *dir, gchar **err);
 gchar* get_locale_filename(const gchar *path);
 int update_sequences_list(const char *sequence_name_to_select);
 void expand_home_in_filename(char *filename, int size);
 double get_normalized_value(fits*);
 void swap_param(double*, double*);
-char* remove_ext_from_filename(const char *basename);
 gchar* str_append(char **data, const char *newdata);
-char* format_basename(char *root);
+char *format_basename(char *root, gboolean can_free);
 float compute_slope(WORD *lo, WORD *hi);
-void load_css_style_sheet();
-double encodeJD(dateTime dt);
 gchar *siril_get_file_info(const gchar *filename, GdkPixbuf *pixbuf);
 gchar *siril_truncate_str(gchar *str, gint size);
-GtkWidget* popover_new(GtkWidget *widget, const gchar *text);
+char **glist_to_array(GList *list, int *arg_count);
+gchar* url_cleanup(const gchar *uri_string);
+void remove_spaces_from_str(gchar *s);
 
 /****************** quantize.h ***************/
 int siril_fits_img_stats_ushort(WORD *array, long nx, long ny, int nullcheck,
@@ -162,16 +166,6 @@ int writeseqfile(sequence *seq);
 gboolean existseq(const char *name);
 int buildseqfile(sequence *seq, int force_recompute);
 
-/****************** registration_preview.h ******************/
-void redraw_previews();
-void set_preview_area(int preview_area, int centerX, int centerY);
-void init_mouse();
-void adjust_reginfo();
-void on_spinbut_shift_value_change(GtkSpinButton *spinbutton,
-		gpointer user_data);
-void test_and_allocate_reference_image(int vport);
-void enable_view_reference_checkbox(gboolean status);
-
 /****************** statistics_list.h ******************/
 void computeStat();
 
