@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -30,9 +30,11 @@
 #include <fcntl.h>
 #include <string.h>
 #include <assert.h>
+#include <math.h>
 #ifdef _WIN32
 #include <windows.h>
 #endif
+#include <glib.h>
 
 #include "core/siril.h"
 #include "core/proto.h"
@@ -41,10 +43,14 @@
 #include "io/conversion.h"
 #include "io/ser.h"
 #include "io/sequence.h"
-#include "gui/callbacks.h"
+#include "gui/utils.h"
 #include "gui/progress_and_log.h"
 #include "io/single_image.h"
 
+#if GLIB_CHECK_VERSION(2,68,0)
+#define g_memdup g_memdup2
+#endif
+
 /**
  * Round double value to an integer
  * @param x value to round
@@ -103,7 +109,7 @@ BYTE round_to_BYTE(double x) {
  * @return a truncated and rounded BYTE
  */
 BYTE roundf_to_BYTE(float f) {
-	if (f <= 0.5f) return 0;
+	if (f < 0.5f) return 0;
 	if (f >= UCHAR_MAX - 0.5f) return UCHAR_MAX;
 	return (BYTE)(f + 0.5f);
 }
@@ -114,11 +120,23 @@ BYTE roundf_to_BYTE(float f) {
  * @return a truncated and rounded WORD
  */
 WORD roundf_to_WORD(float f) {
-	if (f <= 0.5f) return 0;
+	if (f < 0.5f) return 0;
 	if (f >= USHRT_MAX - 0.5f) return USHRT_MAX;
 	return (WORD)(f + 0.5f);
 }
 
+/**
+ * Compute a ceiling factor
+ * @param x the number to test
+ * @param factor the factor
+ * @return x if it is a factor of factor or the next factor
+ */
+int round_to_ceiling_multiple(int x, int factor) {
+	if (x % factor == 0)
+		return x;
+	return (x / factor + 1) * factor;
+}
+
 /**
  * convert double value to a BYTE
  * @param x value to convert
@@ -152,6 +170,12 @@ WORD truncate_to_WORD(int x) {
 	return (WORD)x;
 }
 
+BYTE truncate_to_BYTE(WORD x) {
+	if (x > UCHAR_MAX)
+		return UCHAR_MAX;
+	return (BYTE)x;
+}
+
 /**
  * Clamp an integer value in the interval given by [low, high]
  * @param val value to be checked
@@ -316,6 +340,17 @@ float *ushort8_buffer_to_float(WORD *buffer, size_t ndata) {
 	return buf;
 }
 
+/**
+ * Test equality between two double number
+ * @param a
+ * @param b
+ * @param epsilon
+ * @return
+ */
+gboolean test_double_eq(double a, double b, double epsilon) {
+	return (fabs(a - b) <= epsilon);
+}
+
 /**
  * change endianness of a 16 bit unsigned int
  * @param x value to convert
@@ -492,45 +527,26 @@ gboolean isrgb(fits *fit) {
 	return (fit->naxis == 3);
 }
 
-/**
- *  Looks whether the string str ends with ending. This is case insensitive
- *  @param str the string to check
- *  @param ending the suffix to look for
- *  @return TRUE if str ends with ending
- */
-gboolean ends_with(const char *str, const char *ending) {
-	if (!str || str[0] == '\0')
-		return FALSE;
-	if (!ending || ending[0] == '\0')
-		return TRUE;
-	int ending_len = strlen(ending);
-	int str_len = strlen(str);
-	if (ending_len > str_len)
-		return FALSE;
-	return !strncasecmp(str + str_len - ending_len, ending, ending_len);
-}
-
 /**
  *  Searches for an extension '.something' in filename from the end
- *  @param filename input filename
+ *  @param filename input filename or path
  *  @return the index of the first '.' found
  */
 int get_extension_index(const char *filename) {
-	int i;
 	if (filename == NULL || filename[0] == '\0')
 		return -1;
-	i = strlen(filename) - 1;
-	do {
+	for (int i = strlen(filename) - 1; i > 0; i--) {
+		if (filename[i] == '\\' || filename[i] == '/')
+			break;
 		if (filename[i] == '.')
 			return i;
-		i--;
-	} while (i > 0);
+	}
 	return -1;
 }
 
 /**
  * Get the extension of a file, without the dot.
- * @param filename input filename
+ * @param filename input filename or path
  * @return extension pointed from the filename itself or NULL
  */
 const char *get_filename_ext(const char *filename) {
@@ -562,6 +578,60 @@ image_type get_type_from_filename(const gchar *filename) {
 	return get_type_for_extension(ext);
 }
 
+/**
+ * Removes extension of the filename or path
+ * @param filename file path with extension
+ * @return newly allocated filename without extension
+ */
+char *remove_ext_from_filename(const char *filename) {
+	char *file = NULL;
+	int ext_index = -1;
+
+	for (int i = strlen(filename) - 1; i > 0; i--) {
+		if (filename[i] == '\\' || filename[i] == '/')
+			break;
+		if (filename[i] == '.') {
+			ext_index = i;
+			break;
+		}
+	}
+	if (ext_index == -1)
+		return strdup(filename);
+
+	file = malloc(ext_index + 1);
+	strncpy(file, filename, ext_index);
+	file[ext_index] = '\0';
+	return file;
+}
+
+/**
+ * Replaces the extension of a file name or path
+ * @param path the original path
+ * @param new_ext the next extension to put
+ * @return a new string with the new extension
+ */
+gchar *replace_ext(const char *path, const char *new_ext) {
+	int idx = get_extension_index(path);
+	gchar *retval = g_strdup(path);
+	if (idx != -1)
+		retval[idx] = '\0';
+	return str_append(&retval, new_ext);
+}
+
+/**
+ * Check is a string contains directory separators and thus represent a path
+ * @param file the string to test
+ * @return true if it contains a separator
+ */
+gboolean string_is_a_path(const char *file) {
+	int len = strlen(file);
+	for (int i = 0; i < len; i++) {
+		if (file[i] == '\\' || file[i] == '/')
+			return TRUE;
+	}
+	return FALSE;
+}
+
 /**
  * Tests whether the given file is either regular or a symlink
  * @param filename input
@@ -662,15 +732,155 @@ int stat_file(const char *filename, image_type *type, char **realname) {
 	return 1;
 }
 
+static gchar* siril_canonicalize_filename(const gchar *filename,
+		const gchar *relative_to)
+{
+#if GLIB_CHECK_VERSION(2,58,0)
+	return g_canonicalize_filename(filename, relative_to);
+}
+#else
+/**
+ * g_canonicalize_filename:
+ * @filename: (type filename): the name of the file
+ * @relative_to: (type filename) (nullable): the relative directory, or %NULL
+ * to use the current working directory
+ *
+ * Gets the canonical file name from @filename. All triple slashes are turned into
+ * single slashes, and all `..` and `.`s resolved against @relative_to.
+ *
+ * Symlinks are not followed, and the returned path is guaranteed to be absolute.
+ *
+ * If @filename is an absolute path, @relative_to is ignored. Otherwise,
+ * @relative_to will be prepended to @filename to make it absolute. @relative_to
+ * must be an absolute path, or %NULL. If @relative_to is %NULL, it'll fallback
+ * to g_get_current_dir().
+ *
+ * This function never fails, and will canonicalize file paths even if they don't
+ * exist.
+ *
+ * No file system I/O is done.
+ *
+ * Returns: (type filename) (transfer full): a newly allocated string with the
+ * canonical file path
+ * Since: 2.58
+ */
+  gchar *canon, *start, *p, *q;
+  guint i;
+
+  g_return_val_if_fail (relative_to == NULL || g_path_is_absolute (relative_to), NULL);
+
+  if (!g_path_is_absolute (filename))
+    {
+      gchar *cwd_allocated = NULL;
+      const gchar  *cwd;
+
+      if (relative_to != NULL)
+        cwd = relative_to;
+      else
+        cwd = cwd_allocated = g_get_current_dir ();
+
+      canon = g_build_filename (cwd, filename, NULL);
+      g_free (cwd_allocated);
+    }
+  else
+    {
+      canon = g_strdup (filename);
+    }
+
+  start = (char *)g_path_skip_root (canon);
+
+  if (start == NULL)
+    {
+      /* This shouldn't really happen, as g_get_current_dir() should
+         return an absolute pathname, but bug 573843 shows this is
+         not always happening */
+      g_free (canon);
+      return g_build_filename (G_DIR_SEPARATOR_S, filename, NULL);
+    }
+
+  /* POSIX allows double slashes at the start to
+   * mean something special (as does windows too).
+   * So, "//" != "/", but more than two slashes
+   * is treated as "/".
+   */
+  i = 0;
+  for (p = start - 1;
+       (p >= canon) &&
+         G_IS_DIR_SEPARATOR (*p);
+       p--)
+    i++;
+  if (i > 2)
+    {
+      i -= 1;
+      start -= i;
+      memmove (start, start+i, strlen (start+i) + 1);
+    }
+
+  /* Make sure we're using the canonical dir separator */
+  p++;
+  while (p < start && G_IS_DIR_SEPARATOR (*p))
+    *p++ = G_DIR_SEPARATOR;
+
+  p = start;
+  while (*p != 0)
+    {
+      if (p[0] == '.' && (p[1] == 0 || G_IS_DIR_SEPARATOR (p[1])))
+        {
+          memmove (p, p+1, strlen (p+1)+1);
+        }
+      else if (p[0] == '.' && p[1] == '.' && (p[2] == 0 || G_IS_DIR_SEPARATOR (p[2])))
+        {
+          q = p + 2;
+          /* Skip previous separator */
+          p = p - 2;
+          if (p < start)
+            p = start;
+          while (p > start && !G_IS_DIR_SEPARATOR (*p))
+            p--;
+          if (G_IS_DIR_SEPARATOR (*p))
+            *p++ = G_DIR_SEPARATOR;
+          memmove (p, q, strlen (q)+1);
+        }
+      else
+        {
+          /* Skip until next separator */
+          while (*p != 0 && !G_IS_DIR_SEPARATOR (*p))
+            p++;
+
+          if (*p != 0)
+            {
+              /* Canonicalize one separator */
+              *p++ = G_DIR_SEPARATOR;
+            }
+        }
+
+      /* Remove additional separators */
+      q = p;
+      while (*q && G_IS_DIR_SEPARATOR (*q))
+        q++;
+
+      if (p != q)
+        memmove (p, q, strlen (q) + 1);
+    }
+
+  /* Remove trailing slashes */
+  if (p > start && G_IS_DIR_SEPARATOR (*(p-1)))
+    *(p-1) = 0;
+
+  return canon;
+}
+#endif
+
 /** Try to change the CWD to the argument, absolute or relative.
  *  If success, the new CWD is written to com.wd
  *  @param[in] dir absolute or relative path we want to set as cwd
  *  @param[out] err error message when return value is different of 1. Can be NULL if message is not needed.
  *  @return 0 if success, any other values for error
  */
-int changedir(const char *dir, gchar **err) {
+int siril_change_dir(const char *dir, gchar **err) {
 	gchar *error = NULL;
 	int retval = 0;
+	char *new_dir = NULL;
 
 	if (dir == NULL || dir[0] == '\0') {
 		error = siril_log_message(_("Unknown error\n"));
@@ -692,11 +902,20 @@ int changedir(const char *dir, gchar **err) {
 			/* do we need to search for sequences in the directory now? We still need to
 			 * press the check seq button to display the list, and this is also done there. */
 			/* check_seq();
-			 update_sequence_list();*/
-			g_free(com.wd);
-			com.wd = g_get_current_dir();
-			siril_log_message(_("Setting CWD (Current Working Directory) to '%s'\n"), com.wd);
-			retval = 0;
+			   update_sequence_list();*/
+			// Don't follow symbolic links
+			if (g_path_is_absolute(dir)) {
+				new_dir = g_memdup(dir, strlen(dir) + 1);
+				g_free(com.wd);
+				com.wd = new_dir;
+			} else {
+				new_dir = siril_canonicalize_filename(dir, com.wd);
+				g_free(com.wd);
+				com.wd = new_dir;
+			}
+
+		  siril_log_message(_("Setting CWD (Current Working Directory) to '%s'\n"), com.wd);
+		  retval = 0;
 		} else {
 			error = siril_log_message(_("Could not change directory to '%s'.\n"), dir);
 			retval = 1;
@@ -716,6 +935,10 @@ gchar *get_locale_filename(const gchar *path) {
 	gchar *str;
 #ifdef _WIN32
 	str = g_win32_locale_filename_from_utf8(path);
+	if (!str) {
+		siril_log_color_message("Conversion of the filename to system codepage failed. Please consider removing all wide chars.\n", "red");
+		return g_strdup(path);
+	}
 #else // _WIN32
 	str = g_strdup(path);
 #endif // _WIN32
@@ -793,7 +1016,7 @@ int update_sequences_list(const char *sequence_name_to_select) {
 	gtk_combo_box_text_remove_all(seqcombo);
 
 	if (sequence_name_to_select) {
-	       if (ends_with(sequence_name_to_select, ".seq"))
+	       if (g_str_has_suffix(sequence_name_to_select, ".seq"))
 		       seqname = strdup(sequence_name_to_select);
 	       else {
 		       seqname = malloc(strlen(sequence_name_to_select) + 5);
@@ -895,32 +1118,6 @@ double get_normalized_value(fits *fit) {
 	return -1.0;
 }
 
-/**
- * Removes extension of the filename
- * @param filename file path with extension
- * @return newly allocated filename without extension
- */
-char *remove_ext_from_filename(const char *filename) {
-	size_t filelen;
-	const char *p;
-	char *file = NULL;
-
-	p = strrchr(filename, '.');
-
-	if (p == NULL) {
-		file = malloc(1);
-		file[0] = '\0';
-		return file;
-	}
-
-	filelen = p - filename;
-	file = malloc(filelen + 1);
-	strncpy(file, filename, filelen);
-	file[filelen] = '\0';
-
-	return file;
-}
-
 /**
  * append a string to the end of an existing string
  * @param data original string
@@ -944,9 +1141,10 @@ gchar* str_append(gchar** data, const gchar* newdata) {
  * Cut a base name to 120 characters and add a trailing underscore if needed.
  * WARNING: may return a newly allocated string and free the argument
  * @param root the original base name
+ * @param can_free allow root to be freed in case a new string is allocated
  * @return a string ending with trailing underscore
  */
-char *format_basename(char *root) {
+char *format_basename(char *root, gboolean can_free) {
 	int len = strlen(root);
 	if (len > 120) {
 		root[120] = '\0';
@@ -958,7 +1156,8 @@ char *format_basename(char *root) {
 
 	char *appended = malloc(len + 2);
 	sprintf(appended, "%s_", root);
-	free(root);
+	if (can_free)
+		free(root);
 	return appended;
 }
 
@@ -969,114 +1168,11 @@ char *format_basename(char *root) {
  * @return the computed slope
  */
 float compute_slope(WORD *lo, WORD *hi) {
-	if (sequence_is_loaded() && !single_image_is_loaded()) {
-		*hi = com.seq.layers[RLAYER].hi;
-		*lo = com.seq.layers[RLAYER].lo;
-	}
-	else {
-		*hi = com.uniq->layers[RLAYER].hi;
-		*lo = com.uniq->layers[RLAYER].lo;
-	}
+	*lo = gui.lo;
+	*hi = gui.hi;
 	return UCHAR_MAX_SINGLE / (float) (*hi - *lo);
 }
 
-static const gchar *checking_css_filename() {
-	printf(_("Checking GTK version ... GTK-%d.%d\n"), GTK_MAJOR_VERSION, GTK_MINOR_VERSION);
-	if ((GTK_MAJOR_VERSION >= 3) && (GTK_MINOR_VERSION >= 18))
-		return "siril.css";
-	else if ((GTK_MAJOR_VERSION >= 3) && (GTK_MINOR_VERSION < 18))
-		return "siril_old.css";
-	else {
-		return NULL;
-	}
-}
-
-/**
- * Loads the css sheet
- */
-void load_css_style_sheet () {
-	GtkCssProvider *css_provider;
-	GdkDisplay *display;
-	GdkScreen *screen;
-	gchar *CSSFile;
-	const gchar *css_filename;
-
-	css_filename = checking_css_filename();
-	if (css_filename == NULL) {
-		printf(_("The version of GTK does not match requirements: (GTK-%d.%d)\n"), GTK_MAJOR_VERSION, GTK_MINOR_VERSION);
-		exit(1);
-	}
-
-	CSSFile = g_build_filename(siril_get_system_data_dir(), css_filename, NULL);
-	if (!g_file_test (CSSFile, G_FILE_TEST_EXISTS)) {
-		g_error (_("Unable to load CSS style sheet file: %s. Please reinstall Siril\n"), CSSFile);
-	}
-	else {
-		css_provider = gtk_css_provider_new();
-		display = gdk_display_get_default();
-		screen = gdk_display_get_default_screen(display);
-		gtk_style_context_add_provider_for_screen(screen,
-				GTK_STYLE_PROVIDER(css_provider),
-				GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-		gtk_css_provider_load_from_path(css_provider, CSSFile, NULL);
-		g_fprintf(stdout, _("Successfully loaded '%s'\n"), CSSFile);
-		g_object_unref(css_provider);
-	}
-	g_free(CSSFile);
-}
-
-/**
- * From a datetime it computes the Julian date needed in photometry
- * (code borrowed from muniwin)
- * @param dt timestamp in datetime format
- * @return the Julian date
- */
-double encodeJD(dateTime dt) {
-	double jd1;
-	int before, d1, d2;
-
-	/* Check date and time */
-	if (dt.day <= 0 || dt.year <= 0 || dt.month <= 0)
-		return 0;
-
-	/* Compute Julian date from input citizen year, month and day. */
-	/* Tested for YEAR>0 except 1582-10-07/15 */
-	if (dt.year > 1582) {
-		before = 0;
-	} else if (dt.year < 1582) {
-		before = 1;
-	} else if (dt.month > 10) {
-		before = 0;
-	} else if (dt.month < 10) {
-		before = 1;
-	} else if (dt.day >= 15) {
-		before = 0;
-	} else {
-		before = 1;
-	}
-	if (dt.month <= 2) {
-		d1 = (int) (365.25 * (dt.year - 1));
-		d2 = (int) (30.6001 * (dt.month + 13));
-	} else {
-		d1 = (int) (365.25 * (dt.year));
-		d2 = (int) (30.6001 * (dt.month + 1));
-	}
-	jd1 = 1720994.5 + d1 + d2 + dt.day;
-	jd1 += 1.0 * dt.hour / 24;
-	jd1 += 1.0 * dt.min / 1440.0;
-	jd1 += 1.0 * dt.sec / 86400.0;
-	jd1 += 1.0 * dt.ms / 86400000.0;
-
-	if (before) {
-		if (dt.year < 0)
-			return jd1 - 1;
-		else
-			return jd1;
-	} else {
-		return jd1 + 2 - (dt.year / 100) + (dt.year / 400);
-	}
-}
-
 /**
  * Try to get file info, i.e width and height
  * @param filename name of the file
@@ -1128,34 +1224,74 @@ gchar *siril_truncate_str(gchar *str, gint size) {
 }
 
 /**
- * Create a popover with icon and text
- * @param widget is the parent widget where the popover arises from
- * @param text will be shown in the popover
- * @return the GtkWidget of popover
+ *
+ * @param list
+ * @param arg_count
+ * @return
  */
-GtkWidget* popover_new(GtkWidget *widget, const gchar *text) {
-	GtkWidget *popover, *box, *image, *label;
+char **glist_to_array(GList *list, int *arg_count) {
+	int count;
+	if (arg_count && *arg_count > 0)
+		count = *arg_count;
+	else {
+		count = g_list_length(list);
+		if (arg_count)
+			*arg_count = count;
+	}
+	char **array = malloc(count * sizeof(char *));
+	if (!array) {
+		PRINT_ALLOC_ERR;
+		return NULL;
+	}
+	GList *orig_list = list;
+	for (int i = 0; i < count && list; list = list->next, i++)
+		array[i] = g_strdup(list->data);
+	g_list_free_full(orig_list, g_free);
+	return array;
+}
 
-	popover = gtk_popover_new(widget);
-	label = gtk_label_new(NULL);
-	box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);
-	image = gtk_image_new_from_icon_name("dialog-information-symbolic",
-			GTK_ICON_SIZE_DIALOG);
+/**
+ *
+ * @param uri_string
+ * @return a new allocated string
+ */
+gchar* url_cleanup(const gchar *uri_string) {
+	GString *copy;
+	const gchar *end;
 
-	gtk_label_set_markup(GTK_LABEL(label), text);
-	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
-	gtk_label_set_max_width_chars(GTK_LABEL(label), 64);
+	/* Skip leading whitespace */
+	while (g_ascii_isspace(*uri_string))
+		uri_string++;
 
-	gtk_box_pack_start(GTK_BOX(box), image, FALSE, FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);
-	gtk_container_add(GTK_CONTAINER(popover), box);
+	/* Ignore trailing whitespace */
+	end = uri_string + strlen(uri_string);
+	while (end > uri_string && g_ascii_isspace(*(end - 1)))
+		end--;
 
-	/* make all sensitive in case where parent is not */
-	gtk_widget_set_sensitive(label, TRUE);
-	gtk_widget_set_sensitive(box, TRUE);
-	gtk_widget_set_sensitive(popover, TRUE);
+	/* Copy the rest, encoding unencoded spaces and stripping other whitespace */
+	copy = g_string_sized_new(end - uri_string);
+	while (uri_string < end) {
+		if (*uri_string == ' ')
+			g_string_append(copy, "%20");
+		else if (g_ascii_isspace(*uri_string))
+			; // @suppress("Suspicious semicolon")
+		else
+			g_string_append_c(copy, *uri_string);
+		uri_string++;
+	}
 
-	gtk_widget_show_all(box);
+	return g_string_free(copy, FALSE);
+}
 
-	return popover;
+/**
+ * Deblanks a string
+ * @param s string to be deblanked
+ */
+void remove_spaces_from_str(gchar *s) {
+	gchar *d = s;
+	do {
+		while (g_ascii_isspace(*d)) {
+			++d;
+		}
+	} while((*s++ = *d++));
 }
