@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -20,7 +20,7 @@
 
 #include "core/siril.h"
 #include "core/proto.h"
-#include "gui/callbacks.h"
+#include "gui/utils.h"
 #include "gui/histogram.h"
 #include "filters/asinh.h"
 #include "filters/deconv.h"
@@ -49,11 +49,13 @@ static const SirilDialogEntry entries[] =
 		{"ImagePlateSolver_Dial", IMAGE_PROCESSING_DIALOG, FALSE, NULL},
 		{"linearmatch_dialog", IMAGE_PROCESSING_DIALOG, FALSE, NULL},
 		{"Median_dialog", IMAGE_PROCESSING_DIALOG, FALSE, NULL},
+		{"pixel_math_dialog", OTHER_DIALOG, FALSE, NULL},
 		{"resample_dialog", IMAGE_PROCESSING_DIALOG, FALSE, NULL},
 		{"rgradient_dialog", IMAGE_PROCESSING_DIALOG, FALSE, NULL},
 		{"rotation_dialog", IMAGE_PROCESSING_DIALOG, FALSE, NULL},
 		{"satu_dialog", IMAGE_PROCESSING_DIALOG, TRUE, apply_satu_cancel},
 		{"SCNR_dialog", IMAGE_PROCESSING_DIALOG,  FALSE, NULL},
+		{"search_objects", SEARCH_ENTRY_DIALOG, FALSE, NULL},
 		{"settings_window", INFORMATION_DIALOG, FALSE, NULL},
 		{"seqlist_dialog", INFORMATION_DIALOG, FALSE, NULL},
 		{"split_cfa_dialog", OTHER_DIALOG, FALSE, NULL},
@@ -74,13 +76,22 @@ static SirilDialogEntry get_entry_by_id(gchar *id) {
 	return retvalue;
 }
 
+static gboolean check_escape(GtkWidget *widget, GdkEventKey *event,
+		gpointer data) {
+	if (event->keyval == GDK_KEY_Escape) {
+		gtk_widget_hide(widget);
+		return TRUE;
+	}
+	return FALSE;
+}
+
 void siril_open_dialog(gchar *id) {
-	int i;
 	gint x, y;
 	gboolean win_already_shown = FALSE;
+	GtkWindow *win = GTK_WINDOW(gtk_builder_get_object(gui.builder, id));
 
 	if (get_entry_by_id(id).type != INFORMATION_DIALOG) {
-		for (i = 0; i < G_N_ELEMENTS(entries); i++) {
+		for (int i = 0; i < G_N_ELEMENTS(entries); i++) {
 			GtkWidget *w = lookup_widget(entries[i].identifier);
 			if (gtk_widget_get_visible(w) && entries[i].type != INFORMATION_DIALOG) {
 				win_already_shown = TRUE;
@@ -92,11 +103,14 @@ void siril_open_dialog(gchar *id) {
 			}
 		}
 	}
-	GtkWindow *win = GTK_WINDOW(lookup_widget(id));
-	if (win_already_shown && x >=0 && y >= 0) {
+	if (get_entry_by_id(id).type == SEARCH_ENTRY_DIALOG) {
+		g_signal_connect(win, "key_press_event", G_CALLBACK(check_escape), NULL);
+	}
+
+	if (win_already_shown && x >= 0 && y >= 0) {
 		gtk_window_move(win, x, y);
 	} else {
-		gtk_window_set_position (win, GTK_WIN_POS_CENTER);
+		gtk_window_set_position(win, GTK_WIN_POS_CENTER);
 	}
 	gtk_window_set_type_hint(win, GDK_WINDOW_TYPE_HINT_DIALOG);
 	gtk_window_set_transient_for(win, GTK_WINDOW(lookup_widget("control_window")));
