@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -20,7 +20,6 @@
 
 #include <gsl/gsl_histogram.h>
 #include <string.h>
-#include <stdint.h>
 #include <math.h>
 #include <float.h>
 #include "core/siril.h"
@@ -31,10 +30,11 @@
 #include "io/single_image.h"
 #include "io/sequence.h"
 #include "gui/image_display.h"
-#include "gui/callbacks.h"	// for lookup_widget()
+#include "gui/utils.h"	// for lookup_widget()
 #include "gui/progress_and_log.h"
 #include "gui/dialogs.h"
 #include "gui/siril_preview.h"
+#include "gui/registration_preview.h"
 #include "core/undo.h"
 
 #include "histogram.h"
@@ -57,7 +57,7 @@ static double histo_color_r[] = { 1.0, 0.0, 0.0, 0.0 };
 static double histo_color_g[] = { 0.0, 1.0, 0.0, 0.0 };
 static double histo_color_b[] = { 0.0, 0.0, 1.0, 0.0 };
 static float graph_height = 0.f;	// the max value of all bins
-static uint64_t clipped[] = { 0, 0 };
+static guint64 clipped[] = { 0, 0 };
 
 static GtkToggleToolButton *toggles[MAXVPORT] = { NULL };
 static GtkToggleToolButton *toggleGrid = NULL, *toggleCurve = NULL;
@@ -99,17 +99,16 @@ static void histo_startup() {
 }
 
 static void histo_close(gboolean revert) {
-	int i;
 	if (revert) {
 		set_cursor_waiting(TRUE);
 
-		for (i = 0; i < gfit.naxes[2]; i++) {
+		for (int i = 0; i < gfit.naxes[2]; i++) {
 			set_histogram(hist_backup[i], i);
 			hist_backup[i] = NULL;
 		}
 		copy_backup_to_gfit();
 		adjust_cutoff_from_updated_gfit();
-		redraw(com.cvport, REMAP_ALL);
+		redraw(REMAP_ALL);
 		redraw_previews();
 		set_cursor_waiting(FALSE);
 	}
@@ -127,7 +126,7 @@ static void histo_recompute() {
 	// com.layers_hist should be good, update_histo_mtf() is always called before
 
 	adjust_cutoff_from_updated_gfit();
-	redraw(com.cvport, REMAP_ALL);
+	redraw(REMAP_ALL);
 	redraw_previews();
 }
 
@@ -208,10 +207,10 @@ static void set_histo_toggles_names() {
 		gtk_widget_set_tooltip_text(GTK_WIDGET(toggles[0]), _("Gray channel"));
 		GtkWidget *w;
 		if (com.pref.combo_theme == 0) {
-			image = g_build_filename(siril_get_system_data_dir(), "pixmaps", "monochrome_dark.png", NULL);
+			image = g_build_filename(siril_get_system_data_dir(), "pixmaps", "monochrome_dark.svg", NULL);
 			w = gtk_image_new_from_file(image);
 		} else {
-			image = g_build_filename(siril_get_system_data_dir(), "pixmaps", "monochrome.png", NULL);
+			image = g_build_filename(siril_get_system_data_dir(), "pixmaps", "monochrome.svg", NULL);
 			w = gtk_image_new_from_file(image);
 		}
 		gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(toggles[0]), w);
@@ -228,7 +227,7 @@ static void set_histo_toggles_names() {
 			gtk_widget_set_visible(GTK_WIDGET(toggles[3]), FALSE);
 
 	} else {
-		image = g_build_filename(siril_get_system_data_dir(), "pixmaps", "r.png", NULL);
+		image = g_build_filename(siril_get_system_data_dir(), "pixmaps", "r.svg", NULL);
 		gtk_widget_set_tooltip_text(GTK_WIDGET(toggles[0]), _("Red channel"));
 		GtkWidget *w = gtk_image_new_from_file(image);
 		gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(toggles[0]), w);
@@ -251,7 +250,7 @@ static void set_histo_toggles_names() {
 static double get_histoZoomValueH() {
 	static GtkAdjustment *histoAdjZoomH = NULL;
 	if (!histoAdjZoomH)
-		histoAdjZoomH = GTK_ADJUSTMENT(gtk_builder_get_object(builder, "histoAdjZoomH"));
+		histoAdjZoomH = GTK_ADJUSTMENT(gtk_builder_get_object(gui.builder, "histoAdjZoomH"));
 
 	return gtk_adjustment_get_value(histoAdjZoomH);
 }
@@ -259,7 +258,7 @@ static double get_histoZoomValueH() {
 static double get_histoZoomValueV() {
 	static GtkAdjustment *histoAdjZoomV = NULL;
 	if (!histoAdjZoomV)
-		histoAdjZoomV = GTK_ADJUSTMENT(gtk_builder_get_object(builder, "histoAdjZoomV"));
+		histoAdjZoomV = GTK_ADJUSTMENT(gtk_builder_get_object(gui.builder, "histoAdjZoomV"));
 
 	return gtk_adjustment_get_value(histoAdjZoomV);
 }
@@ -284,8 +283,10 @@ static void adjust_histogram_vport_size() {
 }
 
 size_t get_histo_size(fits *fit) {
-	if (fit->type == DATA_USHORT)
-		return (size_t)get_normalized_value(fit);
+	if (fit->type == DATA_USHORT) {
+		if (fit->orig_bitpix == BYTE_IMG)
+			return UCHAR_MAX;
+	}
 	return (size_t)USHRT_MAX;
 }
 
@@ -566,7 +567,7 @@ static void apply_mtf_to_histo(gsl_histogram *histo, float norm,
 		WORD mtf;
 		float binval = gsl_histogram_get(histo, i);
 		float pxl = ((float)i / norm);
-		uint64_t clip[2] = { 0, 0 };
+		guint64 clip[2] = { 0, 0 };
 
 		if (i < round_to_WORD(lo * norm)) {
 			pxl = lo;
@@ -734,7 +735,9 @@ float findMidtonesBalance(fits *fit, float *shadows, float *highlights) {
 		stat[i] = statistics(NULL, -1, fit, i, NULL, STATS_BASIC | STATS_MAD, TRUE);
 		if (!stat[i]) {
 			siril_log_message(_("Error: statistics computation failed.\n"));
-			return 0.f;
+			*shadows = 0.0f;
+			*highlights = 1.0f;
+			return 0.2f;	// better than linear, but not too risky
 		}
 
 		if (stat[i]->median / stat[i]->normValue > 0.5)
@@ -863,7 +866,8 @@ void on_button_histo_apply_clicked(GtkButton *button, gpointer user_data) {
 		return;
 	}
 
-	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget("checkMTFSeq")))
+	GtkToggleButton *seq_button = GTK_TOGGLE_BUTTON(lookup_widget("checkMTFSeq"));
+	if (gtk_toggle_button_get_active(seq_button)
 			&& sequence_is_loaded()) {
 		/* Apply to the whole sequence */
 		struct mtf_data *args = malloc(sizeof(struct mtf_data));
@@ -884,6 +888,7 @@ void on_button_histo_apply_clicked(GtkButton *button, gpointer user_data) {
 		siril_close_dialog("histogram_dialog");
 
 		/* apply the process */
+		gtk_toggle_button_set_active(seq_button, FALSE);
 		apply_mtf_to_sequence(args);
 
 	} else {
@@ -916,8 +921,8 @@ void apply_histo_cancel() {
 void on_histoZoom100_clicked(GtkButton *button, gpointer user_data) {
 	GtkAdjustment *histoAdjZoomH, *histoAdjZoomV;
 
-	histoAdjZoomH = GTK_ADJUSTMENT(gtk_builder_get_object(builder, "histoAdjZoomH"));
-	histoAdjZoomV = GTK_ADJUSTMENT(gtk_builder_get_object(builder, "histoAdjZoomV"));
+	histoAdjZoomH = GTK_ADJUSTMENT(gtk_builder_get_object(gui.builder, "histoAdjZoomH"));
+	histoAdjZoomV = GTK_ADJUSTMENT(gtk_builder_get_object(gui.builder, "histoAdjZoomV"));
 
 	gtk_adjustment_set_value(histoAdjZoomH, 1.0);
 	gtk_adjustment_set_value(histoAdjZoomV, 1.0);
@@ -944,28 +949,20 @@ void on_histoToolAutoStretch_clicked(GtkToolButton *button, gpointer user_data)
 	set_cursor_waiting(FALSE);
 }
 
-void on_menuitem_histo_activate(GtkMenuItem *menuitem, gpointer user_data) {
-	set_cursor_waiting(TRUE);
-
-	siril_open_dialog("histogram_dialog");
-	set_cursor_waiting(FALSE);
-}
-
-void toggle_histogram_window_visibility(GtkToolButton *button, gpointer user_data) {
-	if (gtk_widget_get_visible((GtkWidget *)user_data)) {
+void toggle_histogram_window_visibility() {
+	if (gtk_widget_get_visible(lookup_widget("histogram_dialog"))) {
 		set_cursor_waiting(TRUE);
 		reset_cursors_and_values();
 		histo_close(TRUE);
 		set_cursor_waiting(FALSE);
 		siril_close_dialog("histogram_dialog");
 	} else {
-		on_menuitem_histo_activate(NULL, NULL);
+		siril_open_dialog("histogram_dialog");
 	}
 }
 
 gboolean on_drawingarea_histograms_motion_notify_event(GtkWidget *widget, GdkEventMotion *event,
 		gpointer user_data) {
-
 	int width = get_width_of_histo();
 	int height = get_height_of_histo();
 
@@ -1073,7 +1070,7 @@ gboolean on_drawingarea_histograms_button_release_event(GtkWidget *widget,
 }
 
 void on_histoMidEntry_activate(GtkEntry *entry, gpointer user_data) {
-	float mid = atof(gtk_entry_get_text(entry));
+	float mid = g_ascii_strtod(gtk_entry_get_text(entry), NULL);
 	if (mid <= _shadows) mid = _shadows;
 	if (mid >= _highlights) mid = _highlights;
 	_midtones = mid;
@@ -1087,7 +1084,7 @@ void on_histoMidEntry_activate(GtkEntry *entry, gpointer user_data) {
 }
 
 void on_histoShadEntry_activate(GtkEntry *entry, gpointer user_data) {
-	float lo = atof(gtk_entry_get_text(entry));
+	float lo = g_ascii_strtod(gtk_entry_get_text(entry), NULL);
 	if (lo <= 0.f) lo = 0.f;
 	if (lo >= _highlights) lo = _highlights;
 	_shadows = lo;
@@ -1101,7 +1098,7 @@ void on_histoShadEntry_activate(GtkEntry *entry, gpointer user_data) {
 }
 
 void on_histoHighEntry_activate(GtkEntry *entry, gpointer user_data) {
-	float hi = atof(gtk_entry_get_text(entry));
+	float hi = g_ascii_strtod(gtk_entry_get_text(entry), NULL);
 	if (hi <= _shadows) hi = _shadows;
 	if (hi >= 1.f) hi = 1.f;
 	_highlights = hi;
@@ -1115,30 +1112,18 @@ void on_histoHighEntry_activate(GtkEntry *entry, gpointer user_data) {
 }
 
 void apply_mtf_to_sequence(struct mtf_data *mtf_args) {
-	struct generic_seq_args *args = malloc(sizeof(struct generic_seq_args));
-	args->seq = mtf_args->seq;
-	args->force_float = FALSE;
-	args->partial_image = FALSE;
+	struct generic_seq_args *args = create_default_seqargs(mtf_args->seq);
 	args->filtering_criterion = seq_filter_included;
 	args->nb_filtered_images = mtf_args->seq->selnum;
-	args->compute_size_hook = NULL;
 	args->prepare_hook = seq_prepare_hook;
 	args->finalize_hook = seq_finalize_hook;
-	args->save_hook = NULL;
 	args->image_hook = mtf_image_hook;
-	args->idle_function = NULL;
 	args->stop_on_error = FALSE;
 	args->description = _("Midtone Transfer Function");
 	args->has_output = TRUE;
 	args->new_seq_prefix = mtf_args->seqEntry;
 	args->load_new_sequence = TRUE;
-	args->force_ser_output = FALSE;
-	args->new_ser = NULL;
-	args->force_fitseq_output = FALSE;
-	args->new_fitseq = NULL;
 	args->user = mtf_args;
-	args->already_in_a_thread = FALSE;
-	args->parallel = TRUE;
 
 	mtf_args->fit = NULL;	// not used here
 
