@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -26,6 +26,7 @@
 #include "core/undo.h"
 #include "gui/progress_and_log.h"
 #include "gui/image_display.h"
+#include "gui/registration_preview.h"
 #include "io/single_image.h"
 #include "io/image_format_fits.h"
 
@@ -52,7 +53,7 @@ static gboolean update_preview(gpointer user_data) {
 	set_progress_bar_data(NULL, PROGRESS_DONE);
 	if (im->show_preview) {
 		adjust_cutoff_from_updated_gfit();
-		redraw(com.cvport, REMAP_ALL);
+		redraw(REMAP_ALL);
 		redraw_previews();
 		set_cursor_waiting(FALSE);
 	}
@@ -67,16 +68,20 @@ static void free_struct(gpointer user_data) {
 }
 
 void copy_gfit_to_backup() {
-	copyfits(&gfit, &preview_gfit_backup, CP_ALLOC | CP_COPYA | CP_FORMAT, -1);
+	if (copyfits(&gfit, &preview_gfit_backup, CP_ALLOC | CP_COPYA | CP_FORMAT, -1)) {
+		siril_log_message(_("Image copy error in previews\n"));
+		return;
+	}
 	preview_is_active = TRUE;
 }
 
 void copy_backup_to_gfit() {
-	copyfits(&preview_gfit_backup, &gfit, CP_COPYA, -1);
+	if (copyfits(&preview_gfit_backup, &gfit, CP_COPYA, -1))
+		siril_log_message(_("Image copy error in previews\n"));
 }
 
 fits *get_preview_gfit_backup() {
-	return &preview_gfit_backup;
+	return (is_preview_active()) ? &preview_gfit_backup : &gfit;
 }
 
 gboolean is_preview_active() {
@@ -96,7 +101,7 @@ void siril_preview_hide() {
 	copy_backup_to_gfit();
 	clear_backup();
 	adjust_cutoff_from_updated_gfit();
-	redraw(com.cvport, REMAP_ALL);
+	redraw(REMAP_ALL);
 	redraw_previews();
 }
 
