@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -22,11 +22,7 @@
  * on big endian systems.
  */
 
-#include <sys/types.h>
-#include <sys/time.h>
-#include <sys/stat.h>
 #include <fcntl.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 #include <math.h>
@@ -35,7 +31,8 @@
 #endif
 #include "core/siril.h"
 #include "core/proto.h"
-#include "gui/callbacks.h"
+#include "core/siril_date.h"
+#include "gui/utils.h"
 #include "gui/progress_and_log.h"
 #include "algos/demosaicing.h"
 #include "io/conversion.h"
@@ -44,136 +41,26 @@
 
 static gboolean user_warned = FALSE;
 
-/* 62135596800 sec from year 0001 to 01 janv. 1970 00:00:00 GMT */
-static const uint64_t epochTicks = 621355968000000000UL;
-static const uint64_t ticksPerSecond = 10000000;
-
 static int ser_write_header(struct ser_struct *ser_file);
 static int ser_write_image_for_writer(struct seqwriter_data *writer, fits *image, int index);
 static int ser_write_frame_from_fit_internal(struct ser_struct *ser_file, fits *fit, int frame_no);
 
-/* Given a SER timestamp, return a char string representation
- * MUST be freed
- */
-static char *ser_timestamp(uint64_t timestamp) {
-	char *str = malloc(64);
-	uint64_t t1970_ms = (timestamp - epochTicks) / 10000;
-	time_t secs = t1970_ms / 1000;
-	int ms = t1970_ms % 1000;
-	struct tm *t;
-#ifdef HAVE_GMTIME_R
-	struct tm t_;
-#endif
-
-#ifdef _WIN32
-	t = gmtime (&secs);
-#else
-#ifdef HAVE_GMTIME_R
-	t = gmtime_r (&secs, &t_);
-#else
-	t = gmtime(&secs);
-#endif /* HAVE_GMTIME_R */
-#endif /* _WIN32 */
-
-	/* If the gmtime() call has failed, "secs" is too big. */
-	if (t == NULL) {
-		free(str);
-		return NULL;
-	}
-
-	sprintf(str, "%04d-%02d-%02dT%02d:%02d:%02d.%03d", t->tm_year + 1900,
-			t->tm_mon + 1, t->tm_mday, t->tm_hour, t->tm_min, t->tm_sec, ms);
-
-	return str;
-}
 
 /* Output SER timestamp */
-static int display_date(uint64_t timestamp, char *txt) {
+static int display_date(guint64 timestamp, char *txt) {
 	if (timestamp == 0)
 		return -1;
 
-	char *str = ser_timestamp(timestamp);
-	if (str) {
-		fprintf(stdout, "%s%s\n", txt, str);
+	GDateTime *date = ser_timestamp_to_date_time(timestamp);
+	if (date) {
+		gchar *str = date_time_to_FITS_date(date);
+		siril_log_message("%s%s\n", txt, str);
 		free(str);
+		g_date_time_unref(date);
 	}
 	return 0;
 }
 
-static time_t mktime_utc(struct tm *tm) {
-	time_t retval;
-
-#ifndef HAVE_TIMEGM
-	static const gint days_before[] = { 0, 31, 59, 90, 120, 151, 181, 212, 243,
-			273, 304, 334 };
-#endif
-
-#ifndef HAVE_TIMEGM
-	if (tm->tm_mon < 0 || tm->tm_mon > 11)
-		return (time_t) -1;
-
-	retval = (tm->tm_year - 70) * 365;
-	retval += (tm->tm_year - 68) / 4;
-	retval += days_before[tm->tm_mon] + tm->tm_mday - 1;
-
-	if (tm->tm_year % 4 == 0 && tm->tm_mon < 2)
-		retval -= 1;
-
-	retval = ((((retval * 24) + tm->tm_hour) * 60) + tm->tm_min) * 60
-			+ tm->tm_sec;
-#else
-	retval = timegm (tm);
-#endif /* !HAVE_TIMEGM */
-
-	return retval;
-}
-
-/* Convert FITS keyword DATE in a UNIX time format
- * DATE match this pattern: 1900-01-01T00:00:00
- */
-static int FITS_date_key_to_Unix_time(char *date, uint64_t *utc,
-		uint64_t *local) {
-	struct tm timeinfo = { };
-	time_t ut, t;
-	int year = 0, month = 0, day = 0, hour = 0, min = 0, ms = 0;
-	float sec = 0.0;
-
-	if (date[0] == '\0')
-		return -1;
-
-	sscanf(date, "%04d-%02d-%02dT%02d:%02d:%f", &year, &month, &day, &hour,
-			&min, &sec);
-
-	timeinfo.tm_year = year - 1900;
-	timeinfo.tm_mon = month - 1;
-	timeinfo.tm_mday = day;
-	timeinfo.tm_hour = hour;
-	timeinfo.tm_min = min;
-	timeinfo.tm_sec = (int) sec;
-	ms = ((int) (sec * 1000) % 1000);
-
-	// Hopefully these are not needed
-	timeinfo.tm_wday = 0;
-	timeinfo.tm_yday = 0;
-	timeinfo.tm_isdst = -1;
-
-	/* get UTC time from timeinfo* */
-	ut = mktime_utc(&timeinfo);
-	ut *= ticksPerSecond;
-	ut += epochTicks;
-	ut += ms * 10000;
-	*utc = (uint64_t) ut;
-
-	/* get local time from timeinfo* */
-	t = mktime(&timeinfo);
-	t *= ticksPerSecond;
-	t += epochTicks;
-	t += ms * 10000;
-	*local = (uint64_t) t;
-
-	return 0;
-}
-
 static char *convert_color_id_to_char(ser_color color_id) {
 	switch (color_id) {
 	case SER_MONO:
@@ -205,10 +92,9 @@ static char *convert_color_id_to_char(ser_color color_id) {
 
 /* reads timestamps from the trailer of the file and stores them in ser_file->ts */
 static int ser_read_timestamp(struct ser_struct *ser_file) {
-	int i;
 	gboolean timestamps_in_order = TRUE;
-	uint64_t previous_ts = 0L;
-	int64_t frame_size;
+	guint64 previous_ts = 0L;
+	gint64 frame_size;
 
 	ser_file->fps = -1.0;	// will be calculated from the timestamps
 
@@ -219,8 +105,8 @@ static int ser_read_timestamp(struct ser_struct *ser_file) {
 
 	frame_size = ser_file->image_width *
 		ser_file->image_height * ser_file->number_of_planes;
-	int64_t offset = SER_HEADER_LEN + frame_size *
-		(int64_t)ser_file->byte_pixel_depth * (int64_t)ser_file->frame_count;
+	gint64 offset = SER_HEADER_LEN + frame_size *
+		(gint64)ser_file->byte_pixel_depth * (gint64)ser_file->frame_count;
 	/* Check if file is large enough to have timestamps */
 	if (ser_file->filesize >= offset + (8 * ser_file->frame_count)) {
 		ser_file->ts = calloc(8, ser_file->frame_count);
@@ -231,8 +117,8 @@ static int ser_read_timestamp(struct ser_struct *ser_file) {
 		ser_file->ts_alloc = ser_file->frame_count;
 
 		// Seek to start of timestamps
-		for (i = 0; i < ser_file->frame_count; i++) {
-			if ((int64_t)-1 == fseek64(ser_file->file, offset+(i*8), SEEK_SET))
+		for (int i = 0; i < ser_file->frame_count; i++) {
+			if ((gint64) -1 == fseek64(ser_file->file, offset + (i * 8), SEEK_SET))
 				return -1;
 
 			if (8 != fread(&ser_file->ts[i], 1, 8, ser_file->file))
@@ -242,11 +128,11 @@ static int ser_read_timestamp(struct ser_struct *ser_file) {
 		}
 
 		/* Check order of Timestamps */
-		uint64_t *ts_ptr = ser_file->ts;
-		uint64_t min_ts = *ts_ptr;
-		uint64_t max_ts = *ts_ptr;
+		guint64 *ts_ptr = ser_file->ts;
+		guint64 min_ts = *ts_ptr;
+		guint64 max_ts = *ts_ptr;
 
-		for (i = 0; i < ser_file->frame_count; i++) {
+		for (int i = 0; i < ser_file->frame_count; i++) {
 			if (*ts_ptr < previous_ts) {
 				// Timestamps are not in order
 				timestamps_in_order = FALSE;
@@ -288,10 +174,10 @@ static int ser_read_timestamp(struct ser_struct *ser_file) {
 
 static int ser_recompute_frame_count(struct ser_struct *ser_file) {
 	int frame_count_calculated;
-	int64_t filesize = ser_file->filesize;
+	gint64 filesize = ser_file->filesize;
 
 	siril_log_message(_("Trying to fix broken SER file...\n"));
-	int64_t frame_size = ser_file->image_width * ser_file->image_height;
+	gint64 frame_size = ser_file->image_width * ser_file->image_height;
 	if (frame_size == 0)
 		return 0;
 
@@ -353,6 +239,9 @@ static int ser_read_header(struct ser_struct *ser_file) {
 	memcpy(ser_file->observer, header + 42, 40);
 	memcpy(ser_file->instrument, header + 82, 40);
 	memcpy(ser_file->telescope, header + 122, 40);
+	ser_file->observer[39] = '\0';
+	ser_file->instrument[39] = '\0';
+	ser_file->telescope[39] = '\0';
 
 	/* internal representations of header data */
 	if (ser_file->bit_pixel_depth <= 8)
@@ -383,8 +272,7 @@ static int ser_read_header(struct ser_struct *ser_file) {
 }
 
 static int ser_write_timestamps(struct ser_struct *ser_file) {
-	int i;
-	int64_t frame_size;
+	gint64 frame_size;
 
 	if (!ser_file->frame_count || ser_file->image_width <= 0 ||
 			ser_file->image_height <= 0 || ser_file->byte_pixel_depth <= 0 ||
@@ -395,15 +283,15 @@ static int ser_write_timestamps(struct ser_struct *ser_file) {
 		// Seek to start of timestamps
 		frame_size = ser_file->image_width * ser_file->image_height
 			* ser_file->number_of_planes;
-		int64_t offset = SER_HEADER_LEN + frame_size * 
-			(int64_t)ser_file->byte_pixel_depth * (int64_t)ser_file->frame_count;
+		gint64 offset = SER_HEADER_LEN + frame_size *
+			(gint64)ser_file->byte_pixel_depth * (gint64)ser_file->frame_count;
 
-		for (i = 0; i < ser_file->frame_count; i++) {
-			uint64_t ts;
+		for (int i = 0; i < ser_file->frame_count; i++) {
+			guint64 ts;
 
 			if (i >= ser_file->ts_alloc)
 				break;
-			if ((int64_t)-1 == fseek64(ser_file->file, offset+(i*8), SEEK_SET)) {
+			if ((gint64)-1 == fseek64(ser_file->file, offset+(i*8), SEEK_SET)) {
 				return -1;
 			}
 
@@ -425,7 +313,7 @@ static int ser_write_header(struct ser_struct *ser_file) {
 
 	if (!ser_file || ser_file->file == NULL)
 		return -1;
-	if ((int64_t) -1 == fseek64(ser_file->file, 0, SEEK_SET)) {
+	if ((gint64) -1 == fseek64(ser_file->file, 0, SEEK_SET)) {
 		perror("seek");
 		return -1;
 	}
@@ -489,7 +377,7 @@ static int ser_write_header_from_fit(struct ser_struct *ser_file, fits *fit) {
 		ser_file->byte_pixel_depth = SER_PIXEL_DEPTH_16;
 		ser_file->bit_pixel_depth = 16;
 	} else {
-		siril_log_message(_("Writing to SER files from larger than 16-bit FITS images is not yet implemented\n"));
+		siril_log_message(_("Writing a 32-bit image to SER files is not supported.\n"));
 		return 1;
 	}
 	if (fit->instrume[0] != 0) {
@@ -505,8 +393,8 @@ static int ser_write_header_from_fit(struct ser_struct *ser_file, fits *fit) {
 		memcpy(ser_file->telescope, fit->telescop, 40);
 	}
 
-	if (FITS_date_key_to_Unix_time(fit->date_obs, &ser_file->date_utc, &ser_file->date) == -1)
-		FITS_date_key_to_Unix_time(fit->date, &ser_file->date_utc, &ser_file->date);
+	if (fit->date_obs)
+		ser_file->date = date_time_to_ser_timestamp(fit->date_obs);
 	return 0;
 }
 
@@ -529,7 +417,7 @@ static int get_SER_Bayer_Pattern(ser_color pattern) {
  * read in it, depending on ser_file's endianess and pixel depth, data is
  * reorganized to match Siril's data format . */
 static void ser_manage_endianess_and_depth(struct ser_struct *ser_file,
-		WORD *data, int64_t frame_size) {
+		WORD *data, gint64 frame_size) {
 	int i;
 	if (ser_file->byte_pixel_depth == SER_PIXEL_DEPTH_8) {
 		// inline conversion to 16 bit
@@ -554,7 +442,7 @@ static int ser_alloc_ts(struct ser_struct *ser_file, int frame_no) {
 	omp_set_lock(&ser_file->ts_lock);
 #endif
 	if (ser_file->ts_alloc <= frame_no) {
-		uint64_t *new = realloc(ser_file->ts, (frame_no + 1) * 2 * sizeof(uint64_t));
+		guint64 *new = realloc(ser_file->ts, (frame_no + 1) * 2 * sizeof(guint64));
 		if (!new) {
 			PRINT_ALLOC_ERR;
 			retval = 1;
@@ -577,7 +465,7 @@ gboolean ser_is_cfa(struct ser_struct *ser_file) {
 	return ser_file && (ser_file->color_id == SER_BAYER_RGGB || 
 			ser_file->color_id == SER_BAYER_GRBG || 
 			ser_file->color_id == SER_BAYER_GBRG || 
-			ser_file->color_id == SER_BAYER_BGGR); 
+			ser_file->color_id == SER_BAYER_BGGR);
 	// SER_BAYER_CYYM SER_BAYER_YCMY SER_BAYER_YMCY SER_BAYER_MYYC are not
 	// supported yet so returning false for them here is good
 }
@@ -587,7 +475,7 @@ void ser_convertTimeStamp(struct ser_struct *ser_file, GSList *timestamp) {
 	int i = 0;
 	if (ser_file->ts)
 		free(ser_file->ts);
-	ser_file->ts = calloc(8, ser_file->frame_count);
+	ser_file->ts = calloc(sizeof(guint64), ser_file->frame_count);
 	if (!ser_file->ts) {
 		PRINT_ALLOC_ERR;
 		return;
@@ -596,10 +484,9 @@ void ser_convertTimeStamp(struct ser_struct *ser_file, GSList *timestamp) {
 
 	GSList *t = timestamp;
 	while (t && i < ser_file->frame_count) {
-		uint64_t utc, local;
-		FITS_date_key_to_Unix_time(t->data, &utc, &local);
+		guint64 utc = date_time_to_ser_timestamp((GDateTime *)t->data);
 		t = t->next;
-		memcpy(&ser_file->ts[i], &utc, 8);
+		memcpy(&ser_file->ts[i], &utc, sizeof(guint64));
 		i++;
 	}
 }
@@ -607,27 +494,34 @@ void ser_convertTimeStamp(struct ser_struct *ser_file, GSList *timestamp) {
 void ser_display_info(struct ser_struct *ser_file) {
 	char *color = convert_color_id_to_char(ser_file->color_id);
 
-	fprintf(stdout, "=========== SER file info ==============\n");
-	fprintf(stdout, "file id: %s\n", ser_file->file_id);
-	fprintf(stdout, "lu id: %d\n", ser_file->lu_id);
-	fprintf(stdout, "little endian: %d\n", ser_file->little_endian);
-	fprintf(stdout, "sensor type: %s\n", color);
-	fprintf(stdout, "image size: %d x %d (%d bits)\n", ser_file->image_width,
+	siril_log_message("=========== SER file info ==============\n");
+	if (ser_file->filename)
+		siril_log_message("for file '%s'\n", ser_file->filename);
+	if (ser_file->file_id && strcmp(ser_file->file_id, "LUCAM-RECORDER"))
+		siril_log_message("file id: %s\n", ser_file->file_id);
+	if (ser_file->lu_id != 0)
+		siril_log_message("lu id: %d\n", ser_file->lu_id);
+	siril_log_message("image size: %d x %d (%d bits)\n", ser_file->image_width,
 			ser_file->image_height, ser_file->bit_pixel_depth);
-	fprintf(stdout, "frame count: %u\n", ser_file->frame_count);
-	fprintf(stdout, "observer: %.40s\n", ser_file->observer);
-	fprintf(stdout, "instrument: %.40s\n", ser_file->instrument);
-	fprintf(stdout, "telescope: %.40s\n", ser_file->telescope);
+	siril_log_message("sensor type: %s\n", color);
+	siril_log_message("frame count: %u\n", ser_file->frame_count);
+	if (ser_file->observer[0] != '\0')
+		siril_log_message("observer: %.40s\n", ser_file->observer);
+	if (ser_file->instrument[0] != '\0')
+		siril_log_message("instrument: %.40s\n", ser_file->instrument);
+	if (ser_file->telescope[0] != '\0')
+		siril_log_message("telescope: %.40s\n", ser_file->telescope);
 	display_date(ser_file->date, "local time: ");
 	display_date(ser_file->date_utc, "UTC time: ");
-	fprintf(stdout, "fps: %.3lf\n", ser_file->fps);
-	fprintf(stdout, "========================================\n");
+	if (ser_file->fps > 0.0)
+		siril_log_message("fps: %.3lf\n", ser_file->fps);
+	siril_log_message("========================================\n");
 }
 
-static int ser_end_write(struct ser_struct *ser_file) {
+static int ser_end_write(struct ser_struct *ser_file, gboolean abort) {
 	int retval = 0;
 	if (ser_file->writer) {
-		retval = stop_writer(ser_file->writer);
+		retval = stop_writer(ser_file->writer, abort);
 		ser_file->frame_count = ser_file->writer->frame_count;
 		free(ser_file->writer);
 		ser_file->writer = NULL;
@@ -637,7 +531,7 @@ static int ser_end_write(struct ser_struct *ser_file) {
 
 int ser_close_and_delete_file(struct ser_struct *ser_file) {
 	if (ser_file == NULL) return -1;
-	int retval = ser_end_write(ser_file);
+	int retval = ser_end_write(ser_file, TRUE);
 	char *filename = ser_file->filename;
 	ser_file->filename = NULL;
 	ser_close_file(ser_file); // closes, frees and zeroes
@@ -649,7 +543,7 @@ int ser_close_and_delete_file(struct ser_struct *ser_file) {
 
 int ser_write_and_close(struct ser_struct *ser_file) {
 	if (ser_file == NULL) return -1;
-	int retval = ser_end_write(ser_file);
+	int retval = ser_end_write(ser_file, FALSE);
 	if (!ser_file->frame_count) {
 		siril_log_color_message(_("The SER sequence is being created with no image in it.\n"), "red");
 		ser_close_and_delete_file(ser_file);
@@ -700,7 +594,8 @@ int ser_create_file(const char *filename, struct ser_struct *ser_file,
 		/* we write the header now, but it should be written again
 		 * before closing in case the number of the image in the new
 		 * SER changes from the copied SER */
-		ser_write_header(ser_file);
+		if (ser_write_header(ser_file))
+			return 1;
 	} else {	// new SER
 		ser_file->file_id = strdup("LUCAM-RECORDER");
 		ser_file->lu_id = 0;
@@ -726,6 +621,13 @@ int ser_create_file(const char *filename, struct ser_struct *ser_file,
 	return 0;
 }
 
+int ser_reset_to_monochrome(struct ser_struct *ser_file) {
+	ser_file->color_id = SER_MONO;
+	if (ser_file->number_of_planes > 0)
+		ser_file->number_of_planes = 1;
+	return ser_write_header(ser_file);
+}
+
 static int ser_write_image_for_writer(struct seqwriter_data *writer, fits *image, int index) {
 	struct ser_struct *ser_file = (struct ser_struct *)writer->sequence;
 
@@ -759,6 +661,7 @@ int ser_open_file(const char *filename, struct ser_struct *ser_file) {
 
 int ser_close_file(struct ser_struct *ser_file) {
 	int retval = 0;
+	user_warned = FALSE;
 	if (!ser_file)
 		return -1;
 	if (ser_file->file) {
@@ -784,11 +687,41 @@ void ser_init_struct(struct ser_struct *ser_file) {
 	memset(ser_file, 0, sizeof(struct ser_struct));
 }
 
+int ser_metadata_as_fits(struct ser_struct *ser_file, fits *fit) {
+	ser_color type_ser = ser_file->color_id;
+	if (!com.pref.debayer.open_debayer && type_ser != SER_RGB && type_ser != SER_BGR) {
+		type_ser = SER_MONO;
+	}
+	switch (type_ser) {
+	case SER_MONO:
+		fit->naxis = 2;
+		fit->naxes[2] = 1;
+		break;
+	case SER_BAYER_RGGB:
+	case SER_BAYER_BGGR:
+	case SER_BAYER_GBRG:
+	case SER_BAYER_GRBG:
+	case SER_BGR:
+	case SER_RGB:
+		fit->naxis = 3;
+		fit->naxes[2] = 3;
+		break;
+	default:
+		return 1;
+	}
+	fit->naxes[0] = fit->rx = ser_file->image_width;
+	fit->naxes[1] = fit->ry = ser_file->image_height;
+	fit->bitpix = (ser_file->byte_pixel_depth == SER_PIXEL_DEPTH_8) ? BYTE_IMG : USHORT_IMG;
+	fit->orig_bitpix = fit->bitpix;
+	fit->binning_x = fit->binning_y = 1;
+	return 0;
+}
+
 /* reads a frame on an already opened SER sequence.
  * frame number starts at 0 */
-int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit) {
+int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit, gboolean force_float, gboolean open_debayer) {
 	int retval = 0, i, j, swap = 0;
-	int64_t offset, frame_size;
+	gint64 offset, frame_size;
 	size_t read_size;
 	WORD *olddata, *tmp;
 	if (!ser_file || ser_file->file == NULL || !ser_file->number_of_planes ||
@@ -808,13 +741,13 @@ int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit) {
 	}
 
 	offset = SER_HEADER_LEN	+ frame_size *
-		(int64_t)ser_file->byte_pixel_depth * (int64_t)frame_no;
+		(gint64)ser_file->byte_pixel_depth * (gint64)frame_no;
 	/*fprintf(stdout, "offset is %lu (frame %d, %d pixels, %d-byte)\n", offset,
 	 frame_no, frame_size, ser_file->pixel_bytedepth);*/
 #ifdef _OPENMP
 	omp_set_lock(&ser_file->fd_lock);
 #endif
-	if ((int64_t)-1 == fseek64(ser_file->file, offset, SEEK_SET)) {
+	if ((gint64)-1 == fseek64(ser_file->file, offset, SEEK_SET)) {
 		perror("fseek in SER");
 		retval = -1;
 	} else {
@@ -833,13 +766,40 @@ int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit) {
 	fit->orig_bitpix = fit->bitpix;
 	fit->binning_x = fit->binning_y = 1;
 
-	/* If the user checks the SER CFA box, the video is opened in B&W
-	 * RGB and BGR are not coming from raw data. In consequence CFA does
-	 * not exist for these kind of cam */
+	/* If opening images debayered is not activated, read the image as CFA monochrome */
+	const gchar *pattern = NULL;
+
 	ser_color type_ser = ser_file->color_id;
-	if (!com.pref.debayer.open_debayer && type_ser != SER_RGB && type_ser != SER_BGR)
+	if (!open_debayer && type_ser != SER_RGB && type_ser != SER_BGR) {
 		type_ser = SER_MONO;
 
+		if (com.pref.debayer.use_bayer_header) {
+			if (ser_file->color_id == SER_BAYER_RGGB)
+				pattern = "RGGB";
+			else if (ser_file->color_id == SER_BAYER_BGGR)
+				pattern = "BGGR";
+			else if (ser_file->color_id == SER_BAYER_GBRG)
+				pattern = "GBRG";
+			else if (ser_file->color_id == SER_BAYER_GRBG)
+				pattern = "GRBG";
+		} else {
+			pattern = filter_pattern[com.pref.debayer.bayer_pattern];
+			if (!user_warned) {
+				if (ser_file->color_id == SER_MONO)
+					siril_log_color_message(_("Forcing SER frame as CFA instead of monochrome, because Bayer information from file has been overridden in preferences\n"), "salmon");
+				else siril_log_color_message(_("Forcing SER Bayer pattern to %s as configured in the preferences\n"), "salmon", pattern);
+				user_warned = TRUE;
+			}
+		}
+	} else if (open_debayer && type_ser == SER_MONO && !com.pref.debayer.use_bayer_header) {
+		pattern = filter_pattern[com.pref.debayer.bayer_pattern];
+		type_ser = retrieveBayerPatternFromChar(pattern) + 8;
+	}
+	if (pattern) {
+		strcpy(fit->bayer_pattern, pattern);
+		strncpy(fit->row_order, "BOTTOM-UP", FLEN_VALUE);
+	}
+
 	switch (type_ser) {
 	case SER_MONO:
 		fit->naxis = 2;
@@ -919,13 +879,24 @@ int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit) {
 
 	/* copy the SER timestamp to the fits */
 	if (ser_file->ts) {
-		char *timestamp = ser_timestamp(ser_file->ts[frame_no]);
+		GDateTime *timestamp = ser_timestamp_to_date_time(ser_file->ts[frame_no]);
 		if (timestamp) {
-			g_snprintf(fit->date_obs, FLEN_VALUE, "%s", timestamp);
-			free(timestamp);
+			if (fit->date_obs) {
+				g_date_time_unref(fit->date_obs);
+			}
+			fit->date_obs = timestamp;
 		}
 	}
 
+	if (force_float) {
+		float *newbuf;
+		size_t pixel_count = fit->naxes[0] * fit->naxes[1] * fit->naxes[2];
+		if (fit->bitpix == BYTE_IMG)
+			newbuf = ushort8_buffer_to_float(fit->data, pixel_count);
+		else newbuf = ushort_buffer_to_float(fit->data, pixel_count);
+		fit_replace_buffer(fit, newbuf, DATA_FLOAT);
+	}
+
 	fits_flip_top_to_bottom(fit);
 	fit->top_down = FALSE;
 
@@ -974,7 +945,7 @@ int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit) {
  * */
 static int read_area_from_image(struct ser_struct *ser_file, const int frame_no,
 		WORD *outbuf, const rectangle *area, const int layer) {
-	int64_t offset, frame_size;
+	gint64 offset, frame_size;
 	int retval = 0;
 	WORD *read_buffer;
 	size_t read_size = ser_file->image_width * area->h * ser_file->byte_pixel_depth;
@@ -1001,7 +972,7 @@ static int read_area_from_image(struct ser_struct *ser_file, const int frame_no,
 		area->y * ser_file->image_width *
 		ser_file->byte_pixel_depth * (layer != -1 ? 3 : 1);	// requested area
 
-	if ((int64_t)-1 == fseek64(ser_file->file, offset, SEEK_SET)) {
+	if ((gint64)-1 == fseek64(ser_file->file, offset, SEEK_SET)) {
 		perror("fseek in SER");
 		retval = -1;
 	} else {
@@ -1164,10 +1135,12 @@ int ser_read_opened_partial_fits(struct ser_struct *ser_file, int layer,
 		return -1;
 	fit->top_down = TRUE;
 	if (ser_file->ts) {
-		char *timestamp = ser_timestamp(ser_file->ts[frame_no]);
+		GDateTime *timestamp = ser_timestamp_to_date_time(ser_file->ts[frame_no]);
 		if (timestamp) {
-			g_snprintf(fit->date_obs, FLEN_VALUE, "%s", timestamp);
-			free(timestamp);
+			if (fit->date_obs) {
+				g_date_time_unref(fit->date_obs);
+			}
+			fit->date_obs = timestamp;
 		}
 	}
 	return ser_read_opened_partial(ser_file, layer, frame_no, fit->pdata[0], area);
@@ -1183,9 +1156,9 @@ int ser_write_frame_from_fit(struct ser_struct *ser_file, fits *fit, int frame_n
 static int ser_write_frame_from_fit_internal(struct ser_struct *ser_file, fits *fit, int frame_no) {
 	int pixel, plane, dest;
 	int ret, retval = 0;
-	int64_t offset, frame_size;
-	BYTE *data8 = NULL;			// for 8-bit files
-	WORD *data16 = NULL;		// for 16-bit files
+	gint64 offset, frame_size;
+	BYTE *data8 = NULL;	// for 8-bit files
+	WORD *data16 = NULL;	// for 16-bit files
 
 	if (!ser_file || ser_file->file == NULL || !fit)
 		return -1;
@@ -1205,7 +1178,7 @@ static int ser_write_frame_from_fit_internal(struct ser_struct *ser_file, fits *
 		ser_file->number_of_planes;
 
 	offset = SER_HEADER_LEN	+ frame_size *
-			(int64_t)ser_file->byte_pixel_depth * (int64_t)frame_no;
+			(gint64)ser_file->byte_pixel_depth * (gint64)frame_no;
 
 	if (ser_file->byte_pixel_depth == SER_PIXEL_DEPTH_8) {
 		data8 = malloc(frame_size * ser_file->byte_pixel_depth);
@@ -1226,7 +1199,7 @@ static int ser_write_frame_from_fit_internal(struct ser_struct *ser_file, fits *
 		for (pixel = 0; pixel < ser_file->image_width * ser_file->image_height;
 				pixel++) {
 			if (ser_file->byte_pixel_depth == SER_PIXEL_DEPTH_8)
-				data8[dest] = round_to_BYTE(fit->pdata[plane][pixel]);
+				data8[dest] = truncate_to_BYTE(fit->pdata[plane][pixel]);
 			else {
 				if (ser_file->little_endian == SER_BIG_ENDIAN)
 					data16[dest] = (fit->pdata[plane][pixel] >> 8 | fit->pdata[plane][pixel] << 8);
@@ -1240,7 +1213,7 @@ static int ser_write_frame_from_fit_internal(struct ser_struct *ser_file, fits *
 #ifdef _OPENMP
 	omp_set_lock(&ser_file->fd_lock);
 #endif
-	if ((int64_t)-1 == fseek64(ser_file->file, offset, SEEK_SET)) {
+	if ((gint64)-1 == fseek64(ser_file->file, offset, SEEK_SET)) {
 #ifdef _OPENMP
 		omp_unset_lock(&ser_file->fd_lock);
 #endif
@@ -1276,9 +1249,9 @@ static int ser_write_frame_from_fit_internal(struct ser_struct *ser_file, fits *
 #endif
 	ser_file->frame_count++;
 
-	if (!ser_alloc_ts(ser_file, frame_no)) {
-		uint64_t utc, local;
-		FITS_date_key_to_Unix_time(fit->date_obs, &utc, &local);
+	if (fit->date_obs && !ser_alloc_ts(ser_file, frame_no)) {
+		guint64 utc;
+		utc = date_time_to_ser_timestamp(fit->date_obs);
 		ser_file->ts[frame_no] = utc;
 	}
 
@@ -1288,8 +1261,8 @@ free_and_quit:
 	return retval;
 }
 
-int64_t ser_compute_file_size(struct ser_struct *ser_file, int nb_frames) {
-	int64_t frame_size, size = ser_file->filesize;
+gint64 ser_compute_file_size(struct ser_struct *ser_file, int nb_frames) {
+	gint64 frame_size, size = ser_file->filesize;
 
 	if (nb_frames != ser_file->frame_count) {
 		frame_size = (size - SER_HEADER_LEN) / ser_file->frame_count;
@@ -1298,6 +1271,13 @@ int64_t ser_compute_file_size(struct ser_struct *ser_file, int nb_frames) {
 	return size;
 }
 
+int import_metadata_from_serfile(struct ser_struct *ser_file, fits *to) {
+	strncpy(to->instrume, ser_file->instrument, FLEN_VALUE);
+	strncpy(to->observer, ser_file->observer, FLEN_VALUE);
+	strncpy(to->telescop, ser_file->telescope, FLEN_VALUE);
+	return 0;
+}
+
 static GdkPixbufDestroyNotify free_preview_data(guchar *pixels, gpointer data) {
 	free(pixels);
 	free(data);
@@ -1324,8 +1304,8 @@ GdkPixbuf* get_thumbnail_from_ser(char *filename, gchar **descr) {
 		return NULL;
 	}
 	float *pix = malloc(MAX_SIZE * sizeof(float));
-	float *ima_data = NULL, *ptr, byte, n, m, max, min, wd, avr;
-	guchar *pixbuf_data = NULL, *pptr;
+	float *ima_data = NULL, *ptr, byte, n, max, min, wd, avr;
+	guchar *pixbuf_data = NULL;
 
 	w = ser.image_width;
 	h = ser.image_height;
@@ -1333,7 +1313,9 @@ GdkPixbuf* get_thumbnail_from_ser(char *filename, gchar **descr) {
 	ima_data = malloc(sz * sizeof(float));
 	pixbuf_data = malloc(3 * MAX_SIZE * MAX_SIZE * sizeof(guchar));
 
-	ser_read_frame(&ser, 0, &fit);
+	/* here no need to debayer, for performance purposes
+	 * we just display monochrome display */
+	ser_read_frame(&ser, 0, &fit, FALSE, FALSE);
 
 	for (i = 0; i < sz; i++) {
 		ima_data[i + 0] = (float)fit.pdata[RLAYER][i];
@@ -1367,7 +1349,7 @@ GdkPixbuf* get_thumbnail_from_ser(char *filename, gchar **descr) {
 		//pptr = &pixbuf_data[i * Ws * 3];
 		for (j = 0; j < MAX_SIZE; j++)
 			pix[j] = 0;
-		m = 0.f; // amount of strings read in block
+		float m = 0.f; // amount of strings read in block
 		for (l = 0; l < pixScale; l++, m++) { // cycle through a block lines
 			ptr = &ima_data[M * w];
 			N = 0; // number of column
@@ -1409,11 +1391,11 @@ GdkPixbuf* get_thumbnail_from_ser(char *filename, gchar **descr) {
 		wd /= avr;
 	ptr = ima_data;
 	for (i = Hs - 1; i > -1; i--) {	// fill pixbuf mirroring image by vertical
-		pptr = &pixbuf_data[Ws * i * 3];
+		guchar *pptr = &pixbuf_data[Ws * i * 3];
 		for (j = 0; j < Ws; j++) {
-			*pptr++ = (guchar) round_to_BYTE(255.f * (*ptr - min) / wd);
-			*pptr++ = (guchar) round_to_BYTE(255.f * (*ptr - min) / wd);
-			*pptr++ = (guchar) round_to_BYTE(255.f * (*ptr - min) / wd);
+			*pptr++ = (guchar) roundf_to_BYTE(255.f * (*ptr - min) / wd);
+			*pptr++ = (guchar) roundf_to_BYTE(255.f * (*ptr - min) / wd);
+			*pptr++ = (guchar) roundf_to_BYTE(255.f * (*ptr - min) / wd);
 			ptr++;
 		}
 	}
@@ -1433,3 +1415,10 @@ GdkPixbuf* get_thumbnail_from_ser(char *filename, gchar **descr) {
 	*descr = description;
 	return pixbuf;
 }
+
+GDateTime *ser_read_frame_date(struct ser_struct *ser_file, int frame_no) {
+	if (ser_file->ts)
+		return ser_timestamp_to_date_time(ser_file->ts[frame_no]);
+	return NULL;
+}
+
