@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -19,7 +19,6 @@
  */
 
 #include <stdlib.h>
-#include <stdint.h>
 #include <string.h>
 #include <float.h>
 #include <sys/stat.h>
@@ -29,10 +28,12 @@
 #include "core/initfile.h"
 #include "core/OS_utils.h"
 #include "gui/callbacks.h"
+#include "gui/utils.h"
 #include "gui/image_display.h"
 #include "gui/progress_and_log.h"
 #include "gui/PSF_list.h"
 #include "gui/sequence_list.h"
+#include "gui/registration_preview.h"
 #include "io/image_format_fits.h"
 #include "io/sequence.h"
 #include "io/single_image.h"
@@ -46,8 +47,8 @@
 #include "stacking.h"
 
 static struct stacking_args stackparam = {	// parameters passed to stacking
-	NULL, NULL, -1, NULL, -1.0, 0, NULL, NULL, NULL, FALSE, { 0, 0 }, -1, 0,
-	{ 0, 0 }, NO_REJEC, NO_NORM, { NULL, NULL, NULL}, FALSE, -1
+	NULL, NULL, -1, NULL, -1.0, 0, NULL, NULL, NULL, FALSE, { 0, 0 }, -1,
+	{ 0, 0 }, NULL, NO_REJEC, NO_NORM, { NULL, NULL, NULL}, FALSE, -1
 };
 
 #define MAX_FILTERS 5
@@ -87,11 +88,16 @@ void initialize_stacking_methods() {
 		gtk_spin_button_set_value(high, com.pref.stack.linear_high);
 		break;
 	case SIGMA:
+	case MAD:
 	case SIGMEDIAN:
 	case WINSORIZED:
 		gtk_spin_button_set_value(low, com.pref.stack.sigma_low);
 		gtk_spin_button_set_value(high, com.pref.stack.sigma_high);
 		break;
+	case GESDT:
+		gtk_spin_button_set_value(low, 0.3);
+		gtk_spin_button_set_value(high, 0.05);
+		break;
 	default:
 		return;
 	}
@@ -144,9 +150,6 @@ void main_stack(struct stacking_args *args) {
 		}
 	}
 
-	if (args->seq->type == SEQ_FITSEQ)
-		fitseq_prepare_for_multiple_read(args->seq->fitseq_file);
-
 	siril_log_message(args->description);
 	if (args->use_32bit_output)
 		siril_log_message(_("Stacking result will be stored as a 32-bit image\n"));
@@ -158,7 +161,6 @@ void main_stack(struct stacking_args *args) {
 	if (upscale_sequence(args)) // does nothing if args->seq->upscale_at_stacking <= 1.05
 		return;
 	// 3. stack
-	args->max_number_of_rows = stack_get_max_number_of_rows(args->seq, args->nb_images_to_stack);
 	args->retval = args->method(args);
 }
 
@@ -180,20 +182,21 @@ static void start_stacking() {
 	gchar *error = NULL;
 	static GtkComboBox *method_combo = NULL, *rejec_combo = NULL, *norm_combo = NULL;
 	static GtkEntry *output_file = NULL;
-	static GtkToggleButton *overwrite = NULL, *force_norm = NULL;
+	static GtkToggleButton *overwrite = NULL, *force_norm = NULL, *weight_button = NULL;
 	static GtkSpinButton *sigSpin[2] = {NULL, NULL};
 	static GtkWidget *norm_to_max = NULL;
 
 	if (method_combo == NULL) {
-		method_combo = GTK_COMBO_BOX(gtk_builder_get_object(builder, "comboboxstack_methods"));
-		output_file = GTK_ENTRY(gtk_builder_get_object(builder, "entryresultfile"));
-		overwrite = GTK_TOGGLE_BUTTON(gtk_builder_get_object(builder, "checkbutoverwrite"));
+		method_combo = GTK_COMBO_BOX(gtk_builder_get_object(gui.builder, "comboboxstack_methods"));
+		output_file = GTK_ENTRY(gtk_builder_get_object(gui.builder, "entryresultfile"));
+		overwrite = GTK_TOGGLE_BUTTON(gtk_builder_get_object(gui.builder, "checkbutoverwrite"));
 		sigSpin[0] = GTK_SPIN_BUTTON(lookup_widget("stack_siglow_button"));
 		sigSpin[1] = GTK_SPIN_BUTTON(lookup_widget("stack_sighigh_button"));
 		rejec_combo = GTK_COMBO_BOX(lookup_widget("comborejection"));
 		norm_combo = GTK_COMBO_BOX(lookup_widget("combonormalize"));
 		force_norm = GTK_TOGGLE_BUTTON(lookup_widget("checkforcenorm"));
 		norm_to_max = lookup_widget("check_normalise_to_max");
+		weight_button = GTK_TOGGLE_BUTTON(lookup_widget("stack_weight_button"));
 	}
 
 	if (get_thread_run()) {
@@ -206,12 +209,12 @@ static void start_stacking() {
 	stackparam.type_of_rejection = gtk_combo_box_get_active(rejec_combo);
 	stackparam.normalize = gtk_combo_box_get_active(norm_combo);
 	stackparam.force_norm = gtk_toggle_button_get_active(force_norm);
-	stackparam.output_norm= gtk_toggle_button_get_active(
-			GTK_TOGGLE_BUTTON(norm_to_max)) && gtk_widget_is_visible(norm_to_max);
+	stackparam.output_norm = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(norm_to_max)) && gtk_widget_is_visible(norm_to_max);
 	stackparam.coeff.offset = NULL;
 	stackparam.coeff.mul = NULL;
 	stackparam.coeff.scale = NULL;
 	stackparam.method =	stacking_methods[gtk_combo_box_get_active(method_combo)];
+	stackparam.apply_weight = gtk_toggle_button_get_active(weight_button) && (gtk_combo_box_get_active(norm_combo) != NO_NORM);
 
 	stackparam.use_32bit_output = evaluate_stacking_should_output_32bits(stackparam.method,
 			&com.seq, stackparam.nb_images_to_stack, &error);
@@ -224,7 +227,7 @@ static void start_stacking() {
 	if (stackparam.method != stack_median && stackparam.method != stack_mean_with_rejection)
 		stackparam.normalize = NO_NORM;
 	stackparam.seq = &com.seq;
-	stackparam.reglayer = get_registration_layer(&com.seq);
+	stackparam.reglayer = get_registration_layer(stackparam.seq);
 	siril_log_color_message(_("Stacking will use registration data of layer %d if some exist.\n"), "salmon", stackparam.reglayer);
 
 	/* Do not display that cause it uses the generic function that already
@@ -245,7 +248,7 @@ static void start_stacking() {
 }
 
 static void _show_summary(struct stacking_args *args) {
-	const char *norm_str, *rej_str;
+	const char *norm_str;
 
 	siril_log_message(_("Integration of %d images:\n"), args->nb_images_to_stack);
 
@@ -297,31 +300,40 @@ static void _show_summary(struct stacking_args *args) {
 		siril_log_message(_("Rejection parameters ...... none\n"));
 	}
 	else {
+		const char *rej_str;
 
 		switch (args->type_of_rejection) {
 		default:
 		case NO_REJEC:
-			rej_str = _("none");
+			rej_str = _("None");
 			break;
 		case PERCENTILE:
-			rej_str = _("percentile clipping");
+			rej_str = _("Percentile Clipping");
 			break;
 		case SIGMA:
-			rej_str = _("sigma clipping");
+			rej_str = _("Sigma Clipping");
+			break;
+		case MAD:
+			rej_str = _("MAD Clipping");
 			break;
 		case SIGMEDIAN:
-			rej_str = _("median sigma clipping");
+			rej_str = _("Median sigma Clipping");
 			break;
 		case WINSORIZED:
-			rej_str = _("Winsorized sigma clipping");
+			rej_str = _("Winsorized Sigma Clipping");
 			break;
 		case LINEARFIT:
-			rej_str = _("linear fit clipping");
+			rej_str = _("Linear Fit Clipping");
+			break;
+		case GESDT:
+			rej_str = _("Generalized Extreme Studentized Deviate Test");
 			break;
 		}
 		siril_log_message(_("Pixel rejection ........... %s\n"), rej_str);
-		siril_log_message(_("Rejection parameters ...... low=%.3f high=%.3f\n"),
+		if (args->type_of_rejection != GESDT) {
+			siril_log_message(_("Rejection parameters ...... low=%.3f high=%.3f\n"),
 				args->sig[0], args->sig[1]);
+		}
 	}
 }
 
@@ -355,8 +367,6 @@ static gboolean end_stacking(gpointer p) {
 	struct stacking_args *args = (struct stacking_args *)p;
 	fprintf(stdout, "Ending stacking idle function, retval=%d\n", args->retval);
 	stop_processing_thread();	// can it be done here in case there is no thread?
-	if (args->seq->type == SEQ_FITSEQ)
-		fitseq_multiple_close(args->seq->fitseq_file);
 
 	if (args->retval == ST_OK) {
 		clear_stars_list();
@@ -370,7 +380,6 @@ static gboolean end_stacking(gpointer p) {
 		com.uniq = calloc(1, sizeof(single));
 		com.uniq->comment = strdup(_("Stacking result image"));
 		com.uniq->nb_layers = gfit.naxes[2];
-		com.uniq->layers = calloc(com.uniq->nb_layers, sizeof(layer_info));
 		com.uniq->fit = &gfit;
 		/* Giving summary if average rejection stacking */
 		_show_summary(args);
@@ -379,9 +388,9 @@ static gboolean end_stacking(gpointer p) {
 
 		/* save stacking result */
 		if (args->output_filename != NULL && args->output_filename[0] != '\0') {
-			GStatBuf st;
-			if (!g_stat(args->output_filename, &st)) {
-				int failed = !args->output_overwrite;
+			int failed = 0;
+			if (g_file_test(args->output_filename, G_FILE_TEST_EXISTS)) {
+				failed = !args->output_overwrite;
 				if (!failed) {
 					if (g_unlink(args->output_filename) == -1)
 						failed = 1;
@@ -392,20 +401,25 @@ static gboolean end_stacking(gpointer p) {
 						com.uniq->fileexist = TRUE;
 					}
 				}
-				if (failed) {
-					com.uniq->filename = strdup(_("Unsaved stacking result"));
-					com.uniq->fileexist = FALSE;
-				}
 			}
 			else {
+				gchar *dirname = g_path_get_dirname(args->output_filename);
+				if (g_mkdir_with_parents(dirname, 0755) < 0) {
+					siril_log_color_message(_("Cannot create output folder: %s\n"), "red", dirname);
+					failed = 1;
+				}
+				g_free(dirname);
 				if (!savefits(args->output_filename, &gfit)) {
 					com.uniq->filename = strdup(args->output_filename);
 					com.uniq->fileexist = TRUE;
 				} else {
-					com.uniq->filename = strdup(_("Unsaved stacking result"));
-					com.uniq->fileexist = FALSE;
+					failed = 1;
 				}
 			}
+			if (failed) {
+				com.uniq->filename = strdup(_("Unsaved stacking result"));
+				com.uniq->fileexist = FALSE;
+			}
 			display_filename();
 			set_precision_switch(); // set precision on screen
 		}
@@ -415,7 +429,7 @@ static gboolean end_stacking(gpointer p) {
 		waiting_for_thread();		// bgnoise
 		initialize_display_mode();
 
-		sliders_mode_set_state(com.sliders);
+		sliders_mode_set_state(gui.sliders);
 		set_cutoff_sliders_max_values();
 		set_sliders_value_to_gfit();
 		adjust_cutoff_from_updated_gfit();	// computes min and max
@@ -425,7 +439,7 @@ static gboolean end_stacking(gpointer p) {
 		/* update menus */
 		update_MenuItem();
 
-		redraw(com.cvport, REMAP_ALL);
+		redraw(REMAP_ALL);
 		redraw_previews();
 		sequence_list_change_current();
 		update_stack_interface(TRUE);
@@ -457,7 +471,7 @@ void on_seqstack_button_clicked (GtkButton *button, gpointer user_data){
 void on_comboboxstack_methods_changed (GtkComboBox *box, gpointer user_data) {
 	static GtkNotebook* notebook = NULL;
 	if (!notebook)
-		notebook = GTK_NOTEBOOK(gtk_builder_get_object(builder, "notebook4"));
+		notebook = GTK_NOTEBOOK(gtk_builder_get_object(gui.builder, "notebook4"));
 	com.pref.stack.method = gtk_combo_box_get_active(box);
 
 	gtk_notebook_set_current_page(notebook, com.pref.stack.method);
@@ -468,8 +482,9 @@ void on_comboboxstack_methods_changed (GtkComboBox *box, gpointer user_data) {
 void on_combonormalize_changed (GtkComboBox *box, gpointer user_data) {
 	GtkWidget *widgetnormalize = lookup_widget("combonormalize");
 	GtkWidget *force_norm = lookup_widget("checkforcenorm");
-	gtk_widget_set_sensitive(force_norm,
-			gtk_combo_box_get_active(GTK_COMBO_BOX(widgetnormalize)) != 0);
+	GtkWidget *use_weight = lookup_widget("stack_weight_button");
+	gtk_widget_set_sensitive(force_norm, gtk_combo_box_get_active(GTK_COMBO_BOX(widgetnormalize)) != 0);
+	gtk_widget_set_sensitive(use_weight, gtk_combo_box_get_active(GTK_COMBO_BOX(widgetnormalize)) != 0);
 }
 
 void on_stack_siglow_button_value_changed(GtkSpinButton *button, gpointer user_data) {
@@ -483,10 +498,14 @@ void on_stack_siglow_button_value_changed(GtkSpinButton *button, gpointer user_d
 		com.pref.stack.linear_low = gtk_spin_button_get_value(button);
 		break;
 	case SIGMA:
+	case MAD:
 	case SIGMEDIAN:
 	case WINSORIZED:
 		com.pref.stack.sigma_low = gtk_spin_button_get_value(button);
 		break;
+	case GESDT:
+		com.pref.stack.esdt_outliers = gtk_spin_button_get_value(button);
+		break;
 	default:
 		return;
 	}
@@ -504,17 +523,21 @@ void on_stack_sighigh_button_value_changed(GtkSpinButton *button, gpointer user_
 		com.pref.stack.linear_high = gtk_spin_button_get_value(button);
 		break;
 	case SIGMA:
+	case MAD:
 	case SIGMEDIAN:
 	case WINSORIZED:
 		com.pref.stack.sigma_high = gtk_spin_button_get_value(button);
 		break;
+	case GESDT:
+		com.pref.stack.esdt_significance = gtk_spin_button_get_value(button);
+		break;
 	default:
 		return;
 	}
 	writeinitfile();
 }
 
-void on_comborejection_changed (GtkComboBox *box, gpointer user_data) {
+void on_comborejection_changed(GtkComboBox *box, gpointer user_data) {
 	rejection type_of_rejection = gtk_combo_box_get_active(box);
 	static GtkWidget *labellow = NULL, *labelhigh = NULL;
 	static GtkWidget *siglow = NULL, *sighigh = NULL;
@@ -541,6 +564,8 @@ void on_comborejection_changed (GtkComboBox *box, gpointer user_data) {
 			gtk_widget_set_visible(sighigh, TRUE);
 			gtk_widget_set_visible(labellow, TRUE);
 			gtk_widget_set_visible(labelhigh, TRUE);
+			gtk_widget_set_tooltip_text(siglow, _("Low clipping factor for the percentile clipping rejection algorithm."));
+			gtk_widget_set_tooltip_text(sighigh, _("High clipping factor for the percentile clipping rejection algorithm."));
 			gtk_spin_button_set_value(GTK_SPIN_BUTTON(siglow), com.pref.stack.percentile_low);
 			gtk_spin_button_set_value(GTK_SPIN_BUTTON(sighigh), com.pref.stack.percentile_high);
 			gtk_spin_button_set_range(GTK_SPIN_BUTTON(siglow), 0.0, 1.0);
@@ -553,6 +578,8 @@ void on_comborejection_changed (GtkComboBox *box, gpointer user_data) {
 			gtk_widget_set_visible(sighigh, TRUE);
 			gtk_widget_set_visible(labellow, TRUE);
 			gtk_widget_set_visible(labelhigh, TRUE);
+			gtk_widget_set_tooltip_text(siglow, _("Tolerance for low pixel values of the linear fit clipping algorithm, in sigma units."));
+			gtk_widget_set_tooltip_text(sighigh, _("Tolerance for high pixel values of the linear fit clipping algorithm, in sigma units."));
 			gtk_spin_button_set_range(GTK_SPIN_BUTTON(siglow), 0.0, 10.0);
 			gtk_spin_button_set_range(GTK_SPIN_BUTTON(sighigh), 0.0, 10.0);
 			gtk_spin_button_set_value(GTK_SPIN_BUTTON(siglow), com.pref.stack.linear_low);
@@ -562,17 +589,36 @@ void on_comborejection_changed (GtkComboBox *box, gpointer user_data) {
 			break;
 		default:
 		case SIGMA:
+		case MAD:
+		case SIGMEDIAN:
 		case WINSORIZED:
 			gtk_widget_set_visible(siglow, TRUE);
 			gtk_widget_set_visible(sighigh, TRUE);
 			gtk_widget_set_visible(labellow, TRUE);
 			gtk_widget_set_visible(labelhigh, TRUE);
+			gtk_widget_set_tooltip_text(siglow, _("Low clipping factor for the sigma clipping rejection algorithm."));
+			gtk_widget_set_tooltip_text(sighigh, _("High clipping factor for the sigma clipping rejection algorithm."));
 			gtk_spin_button_set_range(GTK_SPIN_BUTTON(siglow), 0.0, 10.0);
 			gtk_spin_button_set_range(GTK_SPIN_BUTTON(sighigh), 0.0, 10.0);
 			gtk_spin_button_set_value(GTK_SPIN_BUTTON(siglow), com.pref.stack.sigma_low);
 			gtk_spin_button_set_value(GTK_SPIN_BUTTON(sighigh), com.pref.stack.sigma_high);
 			gtk_label_set_text(GTK_LABEL(labellow), _("Sigma low: "));
 			gtk_label_set_text(GTK_LABEL(labelhigh), _("Sigma high: "));
+			break;
+		case GESDT:
+			gtk_widget_set_visible(siglow, TRUE);
+			gtk_widget_set_visible(sighigh, TRUE);
+			gtk_widget_set_visible(labellow, TRUE);
+			gtk_widget_set_visible(labelhigh, TRUE);
+			gtk_widget_set_tooltip_text(siglow, _("Expected fraction of maximum outliers for the Generalized Extreme Studentized Deviate Test algorithm."));
+			gtk_widget_set_tooltip_text(sighigh, _("Probability of making a false positive for the Generalized Extreme Studentized Deviate Test algorithm. "
+					"Increasing this value will reject more pixels."));
+			gtk_spin_button_set_range(GTK_SPIN_BUTTON(siglow), 0.0, 1.0);
+			gtk_spin_button_set_range(GTK_SPIN_BUTTON(sighigh), 0.0, 1.0);
+			gtk_spin_button_set_value(GTK_SPIN_BUTTON(siglow), 0.3);
+			gtk_spin_button_set_value(GTK_SPIN_BUTTON(sighigh), 0.05);
+			gtk_label_set_text(GTK_LABEL(labellow), _("ESD Outliers: "));
+			gtk_label_set_text(GTK_LABEL(labelhigh), _("ESD Significance: "));
 	}
 
 	g_signal_handlers_unblock_by_func(GTK_SPIN_BUTTON(siglow), on_stack_siglow_button_value_changed, NULL);
@@ -582,27 +628,6 @@ void on_comborejection_changed (GtkComboBox *box, gpointer user_data) {
 	writeinitfile();
 }
 
-int stack_get_max_number_of_rows(sequence *seq, int nb_images_to_stack) {
-	int max_memory = get_max_memory_in_MB();
-	if (max_memory > 0) {
-		siril_log_message(_("Using %d MB memory maximum for stacking\n"), max_memory);
-		int elem_size = get_data_type(seq->bitpix) == DATA_FLOAT ? sizeof(float) : sizeof(WORD);
-		uint64_t number_of_rows = (uint64_t)max_memory * BYTES_IN_A_MB /
-			((uint64_t)seq->rx * nb_images_to_stack * elem_size * com.max_thread);
-		// this is how many rows we can load in parallel from all images of the
-		// sequence and be under the limit defined in config in megabytes.
-		// We want to avoid having blocks larger than the half or they will decrease parallelism
-		if (number_of_rows > seq->ry)
-			return seq->ry;
-		if (number_of_rows * 2 > seq->ry)
-			return seq->ry / 2;
-		return truncate_to_int32(number_of_rows);
-	} else {
-		siril_log_message(_("Not using limits on maximum memory for stacking\n"));
-		return (seq->ry / 4) + 1;
-	}
-}
-
 int find_refimage_in_indices(int *indices, int nb, int ref) {
 	int i;
 	for (i = 0; i < nb; i++) {
@@ -739,8 +764,6 @@ static void update_filter_label() {
 	static GtkComboBox *filter_combo[] = {NULL, NULL, NULL};
 	static GtkLabel *filter_label[] = {NULL, NULL, NULL};
 	gchar *filter_str;
-	double param;
-	int filter, type;
 
 	if (!filter_combo[0]) {
 		filter_combo[0] = GTK_COMBO_BOX(lookup_widget("combofilter1"));
@@ -751,13 +774,13 @@ static void update_filter_label() {
 		filter_label[2] = GTK_LABEL(lookup_widget("labelfilter3"));
 	}
 
-	for (filter = 0; filter < 3; filter++) {
+	for (int filter = 0; filter < 3; filter++) {
 		if (!gtk_widget_get_visible(GTK_WIDGET(filter_combo[filter]))) {
 			break;
 		}
 
-		type = gtk_combo_box_get_active(filter_combo[filter]);
-		param = stackfilters[filter].param;
+		int type = gtk_combo_box_get_active(filter_combo[filter]);
+		double param = stackfilters[filter].param;
 		if (param == DBL_MIN || param == DBL_MAX || param == 0.0) {
 			if (type == ALL_IMAGES || type == SELECTED_IMAGES)
 				filter_str = g_strdup("");
@@ -874,5 +897,6 @@ static void stacking_args_deep_copy(struct stacking_args *from, struct stacking_
 static void stacking_args_deep_free(struct stacking_args *args) {
 	free(args->image_indices);
 	free(args->description);
+	free(args->critical_value);
 	free(args);
 }
