@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -25,14 +25,15 @@
 #include "core/proto.h"
 #include "core/processing.h"
 #include "io/sequence.h"
+#include "io/ser.h"
 #include "io/image_format_fits.h"
 #include "stacking.h"
 #include "gui/progress_and_log.h"
 
 struct sum_stacking_data {
-	uint64_t *sum[3];	// the new image's channels
+	guint64 *sum[3];	// the new image's channels
 	double *fsum[3];	// the new image's channels, for float input image
-	double exposure;	// sum of the exposures
+	double livetime;	// sum of the exposures
 	int reglayer;		// layer used for registration data
 	int ref_image;		// reference image index in the stacked sequence
 	gboolean input_32bits;	// input is a sequence of 32-bit float images
@@ -58,7 +59,7 @@ static int sum_stacking_prepare_hook(struct generic_seq_args *args) {
 		}
 		ssdata->sum[0] = NULL;
 	} else {
-		ssdata->sum[0] = calloc(nbdata, sizeof(uint64_t) * args->seq->nb_layers);
+		ssdata->sum[0] = calloc(nbdata, sizeof(guint64) * args->seq->nb_layers);
 		if (ssdata->sum[0] == NULL){
 			PRINT_ALLOC_ERR;
 			return ST_ALLOC_ERROR;
@@ -73,7 +74,7 @@ static int sum_stacking_prepare_hook(struct generic_seq_args *args) {
 		ssdata->fsum[0] = NULL;
 	}
 
-	ssdata->exposure = 0.0;
+	ssdata->livetime = 0.0;
 	return ST_OK;
 }
 
@@ -85,7 +86,7 @@ static int sum_stacking_image_hook(struct generic_seq_args *args, int o, int i,
 #ifdef _OPENMP
 #pragma omp atomic
 #endif
-	ssdata->exposure += fit->exposure;
+	ssdata->livetime += fit->exposure;
 	
 	if (ssdata->reglayer != -1 && args->seq->regparam[ssdata->reglayer]) {
 		shiftx = round_to_int(args->seq->regparam[ssdata->reglayer][i].shiftx * (float)args->seq->upscale_at_stacking);
@@ -127,11 +128,19 @@ static int sum_stacking_image_hook(struct generic_seq_args *args, int o, int i,
 // convert the result and store it into gfit
 static int sum_stacking_finalize_hook(struct generic_seq_args *args) {
 	struct sum_stacking_data *ssdata = args->user;
-	uint64_t max = 0L;	// max value of the image's channels
+	guint64 max = 0L;	// max value of the image's channels
 	double fmax = 0.0;
 	size_t i, nbdata;
 	int layer;
 
+	if (args->retval) {
+		if (ssdata->sum[0]) free(ssdata->sum[0]);
+		if (ssdata->fsum[0]) free(ssdata->fsum[0]);
+		free(ssdata);
+		args->user = NULL;
+		return args->retval;
+	}
+
 	nbdata = args->seq->ry * args->seq->rx * args->seq->nb_layers;
 	// find the max first
 	if (ssdata->input_32bits) {
@@ -157,15 +166,21 @@ static int sum_stacking_finalize_hook(struct generic_seq_args *args) {
 		return ST_GENERIC_ERROR;
 
 	/* We copy metadata from reference to the final fit */
+	int ref = ssdata->ref_image;
 	if (args->seq->type == SEQ_REGULAR) {
-		int ref = ssdata->ref_image;
 		if (!seq_open_image(args->seq, ref)) {
 			import_metadata_from_fitsfile(args->seq->fptr[ref], &gfit);
 			seq_close_image(args->seq, ref);
 		}
+	} else if (args->seq->type == SEQ_FITSEQ) {
+		if (!fitseq_set_current_frame(args->seq->fitseq_file, ref))
+			import_metadata_from_fitsfile(args->seq->fitseq_file->fptr, &gfit);
+	} else if (args->seq->type == SEQ_SER) {
+		import_metadata_from_serfile(args->seq->ser_file, &gfit);
 	}
 
-	gfit.exposure = ssdata->exposure;
+	gfit.livetime = ssdata->livetime;
+	gfit.stacknt = args->nb_filtered_images;
 	nbdata = args->seq->ry * args->seq->rx;
 
 	if (ssdata->output_32bits) {
@@ -181,7 +196,7 @@ static int sum_stacking_finalize_hook(struct generic_seq_args *args) {
 		} else {
 			double ratio = 1.0 / (double)max;
 			for (layer=0; layer<args->seq->nb_layers; ++layer){
-				uint64_t *from = ssdata->sum[layer];
+				guint64 *from = ssdata->sum[layer];
 				float *to = gfit.fpdata[layer];
 				for (i=0; i < nbdata; ++i) {
 					*to++ = (float)((double)(*from++) * ratio);
@@ -196,7 +211,7 @@ static int sum_stacking_finalize_hook(struct generic_seq_args *args) {
 		}
 
 		for (layer=0; layer<args->seq->nb_layers; ++layer){
-			uint64_t *from = ssdata->sum[layer];
+			guint64 *from = ssdata->sum[layer];
 			WORD *to = gfit.pdata[layer];
 			for (i=0; i < nbdata; ++i) {
 				if (ratio == 1.0)
@@ -215,25 +230,17 @@ static int sum_stacking_finalize_hook(struct generic_seq_args *args) {
 }
 
 int stack_summing_generic(struct stacking_args *stackargs) {
-	struct generic_seq_args *args = malloc(sizeof(struct generic_seq_args));
-	args->seq = stackargs->seq;
-	args->force_float = FALSE;
-	args->partial_image = FALSE;
+	struct generic_seq_args *args = create_default_seqargs(stackargs->seq);
 	args->filtering_criterion = stackargs->filtering_criterion;
 	args->filtering_parameter = stackargs->filtering_parameter;
 	args->nb_filtered_images = stackargs->nb_images_to_stack;
 	args->prepare_hook = sum_stacking_prepare_hook;
 	args->image_hook = sum_stacking_image_hook;
-	args->save_hook = NULL;
 	args->finalize_hook = sum_stacking_finalize_hook;
-	args->idle_function = NULL;
-	args->stop_on_error = TRUE;
 	args->description = _("Sum stacking");
-	args->has_output = FALSE;
 	args->already_in_a_thread = TRUE;
-	args->parallel = TRUE;
 
-	struct sum_stacking_data *ssdata = malloc(sizeof(struct sum_stacking_data));
+	struct sum_stacking_data *ssdata = calloc(1, sizeof(struct sum_stacking_data));
 	ssdata->reglayer = stackargs->reglayer;
 	ssdata->ref_image = stackargs->ref_image;
 	assert(ssdata->ref_image >= 0 && ssdata->ref_image < args->seq->number);
